/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include "phosh-idle-dbus.h"

#include <string.h>
#ifdef G_OS_UNIX
#  include <gio/gunixfdlist.h>
#endif

#ifdef G_ENABLE_DEBUG
#define g_marshal_value_peek_boolean(v)  g_value_get_boolean (v)
#define g_marshal_value_peek_char(v)     g_value_get_schar (v)
#define g_marshal_value_peek_uchar(v)    g_value_get_uchar (v)
#define g_marshal_value_peek_int(v)      g_value_get_int (v)
#define g_marshal_value_peek_uint(v)     g_value_get_uint (v)
#define g_marshal_value_peek_long(v)     g_value_get_long (v)
#define g_marshal_value_peek_ulong(v)    g_value_get_ulong (v)
#define g_marshal_value_peek_int64(v)    g_value_get_int64 (v)
#define g_marshal_value_peek_uint64(v)   g_value_get_uint64 (v)
#define g_marshal_value_peek_enum(v)     g_value_get_enum (v)
#define g_marshal_value_peek_flags(v)    g_value_get_flags (v)
#define g_marshal_value_peek_float(v)    g_value_get_float (v)
#define g_marshal_value_peek_double(v)   g_value_get_double (v)
#define g_marshal_value_peek_string(v)   (char*) g_value_get_string (v)
#define g_marshal_value_peek_param(v)    g_value_get_param (v)
#define g_marshal_value_peek_boxed(v)    g_value_get_boxed (v)
#define g_marshal_value_peek_pointer(v)  g_value_get_pointer (v)
#define g_marshal_value_peek_object(v)   g_value_get_object (v)
#define g_marshal_value_peek_variant(v)  g_value_get_variant (v)
#else /* !G_ENABLE_DEBUG */
/* WARNING: This code accesses GValues directly, which is UNSUPPORTED API.
 *          Do not access GValues directly in your code. Instead, use the
 *          g_value_get_*() functions
 */
#define g_marshal_value_peek_boolean(v)  (v)->data[0].v_int
#define g_marshal_value_peek_char(v)     (v)->data[0].v_int
#define g_marshal_value_peek_uchar(v)    (v)->data[0].v_uint
#define g_marshal_value_peek_int(v)      (v)->data[0].v_int
#define g_marshal_value_peek_uint(v)     (v)->data[0].v_uint
#define g_marshal_value_peek_long(v)     (v)->data[0].v_long
#define g_marshal_value_peek_ulong(v)    (v)->data[0].v_ulong
#define g_marshal_value_peek_int64(v)    (v)->data[0].v_int64
#define g_marshal_value_peek_uint64(v)   (v)->data[0].v_uint64
#define g_marshal_value_peek_enum(v)     (v)->data[0].v_long
#define g_marshal_value_peek_flags(v)    (v)->data[0].v_ulong
#define g_marshal_value_peek_float(v)    (v)->data[0].v_float
#define g_marshal_value_peek_double(v)   (v)->data[0].v_double
#define g_marshal_value_peek_string(v)   (v)->data[0].v_pointer
#define g_marshal_value_peek_param(v)    (v)->data[0].v_pointer
#define g_marshal_value_peek_boxed(v)    (v)->data[0].v_pointer
#define g_marshal_value_peek_pointer(v)  (v)->data[0].v_pointer
#define g_marshal_value_peek_object(v)   (v)->data[0].v_pointer
#define g_marshal_value_peek_variant(v)  (v)->data[0].v_pointer
#endif /* !G_ENABLE_DEBUG */

typedef struct
{
  GDBusArgInfo parent_struct;
  gboolean use_gvariant;
} _ExtendedGDBusArgInfo;

typedef struct
{
  GDBusMethodInfo parent_struct;
  const gchar *signal_name;
  gboolean pass_fdlist;
} _ExtendedGDBusMethodInfo;

typedef struct
{
  GDBusSignalInfo parent_struct;
  const gchar *signal_name;
} _ExtendedGDBusSignalInfo;

typedef struct
{
  GDBusPropertyInfo parent_struct;
  const gchar *hyphen_name;
  guint use_gvariant : 1;
  guint emits_changed_signal : 1;
} _ExtendedGDBusPropertyInfo;

typedef struct
{
  GDBusInterfaceInfo parent_struct;
  const gchar *hyphen_name;
} _ExtendedGDBusInterfaceInfo;

typedef struct
{
  const _ExtendedGDBusPropertyInfo *info;
  guint prop_id;
  GValue orig_value; /* the value before the change */
} ChangedProperty;

static void
_changed_property_free (ChangedProperty *data)
{
  g_value_unset (&data->orig_value);
  g_free (data);
}

static gboolean
_g_strv_equal0 (gchar **a, gchar **b)
{
  gboolean ret = FALSE;
  guint n;
  if (a == NULL && b == NULL)
    {
      ret = TRUE;
      goto out;
    }
  if (a == NULL || b == NULL)
    goto out;
  if (g_strv_length (a) != g_strv_length (b))
    goto out;
  for (n = 0; a[n] != NULL; n++)
    if (g_strcmp0 (a[n], b[n]) != 0)
      goto out;
  ret = TRUE;
out:
  return ret;
}

static gboolean
_g_variant_equal0 (GVariant *a, GVariant *b)
{
  gboolean ret = FALSE;
  if (a == NULL && b == NULL)
    {
      ret = TRUE;
      goto out;
    }
  if (a == NULL || b == NULL)
    goto out;
  ret = g_variant_equal (a, b);
out:
  return ret;
}

G_GNUC_UNUSED static gboolean
_g_value_equal (const GValue *a, const GValue *b)
{
  gboolean ret = FALSE;
  g_assert (G_VALUE_TYPE (a) == G_VALUE_TYPE (b));
  switch (G_VALUE_TYPE (a))
    {
      case G_TYPE_BOOLEAN:
        ret = (g_value_get_boolean (a) == g_value_get_boolean (b));
        break;
      case G_TYPE_UCHAR:
        ret = (g_value_get_uchar (a) == g_value_get_uchar (b));
        break;
      case G_TYPE_INT:
        ret = (g_value_get_int (a) == g_value_get_int (b));
        break;
      case G_TYPE_UINT:
        ret = (g_value_get_uint (a) == g_value_get_uint (b));
        break;
      case G_TYPE_INT64:
        ret = (g_value_get_int64 (a) == g_value_get_int64 (b));
        break;
      case G_TYPE_UINT64:
        ret = (g_value_get_uint64 (a) == g_value_get_uint64 (b));
        break;
      case G_TYPE_DOUBLE:
        {
          /* Avoid -Wfloat-equal warnings by doing a direct bit compare */
          gdouble da = g_value_get_double (a);
          gdouble db = g_value_get_double (b);
          ret = memcmp (&da, &db, sizeof (gdouble)) == 0;
        }
        break;
      case G_TYPE_STRING:
        ret = (g_strcmp0 (g_value_get_string (a), g_value_get_string (b)) == 0);
        break;
      case G_TYPE_VARIANT:
        ret = _g_variant_equal0 (g_value_get_variant (a), g_value_get_variant (b));
        break;
      default:
        if (G_VALUE_TYPE (a) == G_TYPE_STRV)
          ret = _g_strv_equal0 (g_value_get_boxed (a), g_value_get_boxed (b));
        else
          g_critical ("_g_value_equal() does not handle type %s", g_type_name (G_VALUE_TYPE (a)));
        break;
    }
  return ret;
}

static void
_g_dbus_codegen_marshal_BOOLEAN__OBJECT (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef gboolean (*_GDbusCodegenMarshalBoolean_ObjectFunc)
       (void *data1,
        GDBusMethodInvocation *arg_method_invocation,
        void *data2);
  _GDbusCodegenMarshalBoolean_ObjectFunc callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 2);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalBoolean_ObjectFunc)
    (marshal_data ? marshal_data : cc->callback);

  v_return =
    callback (data1,
              g_marshal_value_peek_object (param_values + 1),
              data2);

  g_value_set_boolean (return_value, v_return);
}

static void
_g_dbus_codegen_marshal_BOOLEAN__OBJECT_UINT64 (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef gboolean (*_GDbusCodegenMarshalBoolean_ObjectUint64Func)
       (void *data1,
        GDBusMethodInvocation *arg_method_invocation,
        guint64 arg_interval,
        void *data2);
  _GDbusCodegenMarshalBoolean_ObjectUint64Func callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 3);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalBoolean_ObjectUint64Func)
    (marshal_data ? marshal_data : cc->callback);

  v_return =
    callback (data1,
              g_marshal_value_peek_object (param_values + 1),
              g_marshal_value_peek_uint64 (param_values + 2),
              data2);

  g_value_set_boolean (return_value, v_return);
}

static void
_g_dbus_codegen_marshal_BOOLEAN__OBJECT_UINT (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef gboolean (*_GDbusCodegenMarshalBoolean_ObjectUintFunc)
       (void *data1,
        GDBusMethodInvocation *arg_method_invocation,
        guint arg_id,
        void *data2);
  _GDbusCodegenMarshalBoolean_ObjectUintFunc callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 3);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalBoolean_ObjectUintFunc)
    (marshal_data ? marshal_data : cc->callback);

  v_return =
    callback (data1,
              g_marshal_value_peek_object (param_values + 1),
              g_marshal_value_peek_uint (param_values + 2),
              data2);

  g_value_set_boolean (return_value, v_return);
}

/* ------------------------------------------------------------------------
 * Code for interface org.gnome.Mutter.IdleMonitor
 * ------------------------------------------------------------------------
 */

/**
 * SECTION:PhoshIdleDBusIdleMonitor
 * @title: PhoshIdleDBusIdleMonitor
 * @short_description: Generated C code for the org.gnome.Mutter.IdleMonitor D-Bus interface
 *
 * This section contains code for working with the <link linkend="gdbus-interface-org-gnome-Mutter-IdleMonitor.top_of_page">org.gnome.Mutter.IdleMonitor</link> D-Bus interface in C.
 */

enum
{
  PHOSH_IDLE_DBUS__IDLE_MONITOR_WATCH_FIRED,
};

static unsigned PHOSH_IDLE_DBUS__IDLE_MONITOR_SIGNALS[1] = { 0 };

/* ---- Introspection data for org.gnome.Mutter.IdleMonitor ---- */

static const _ExtendedGDBusArgInfo _phosh_idle_dbus_idle_monitor_method_info_get_idletime_OUT_ARG_idletime =
{
  {
    -1,
    (gchar *) "idletime",
    (gchar *) "t",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _phosh_idle_dbus_idle_monitor_method_info_get_idletime_OUT_ARG_pointers[] =
{
  &_phosh_idle_dbus_idle_monitor_method_info_get_idletime_OUT_ARG_idletime.parent_struct,
  NULL
};

static const _ExtendedGDBusMethodInfo _phosh_idle_dbus_idle_monitor_method_info_get_idletime =
{
  {
    -1,
    (gchar *) "GetIdletime",
    NULL,
    (GDBusArgInfo **) &_phosh_idle_dbus_idle_monitor_method_info_get_idletime_OUT_ARG_pointers,
    NULL
  },
  "handle-get-idletime",
  FALSE
};

static const _ExtendedGDBusArgInfo _phosh_idle_dbus_idle_monitor_method_info_add_idle_watch_IN_ARG_interval =
{
  {
    -1,
    (gchar *) "interval",
    (gchar *) "t",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _phosh_idle_dbus_idle_monitor_method_info_add_idle_watch_IN_ARG_pointers[] =
{
  &_phosh_idle_dbus_idle_monitor_method_info_add_idle_watch_IN_ARG_interval.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _phosh_idle_dbus_idle_monitor_method_info_add_idle_watch_OUT_ARG_id =
{
  {
    -1,
    (gchar *) "id",
    (gchar *) "u",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _phosh_idle_dbus_idle_monitor_method_info_add_idle_watch_OUT_ARG_pointers[] =
{
  &_phosh_idle_dbus_idle_monitor_method_info_add_idle_watch_OUT_ARG_id.parent_struct,
  NULL
};

static const _ExtendedGDBusMethodInfo _phosh_idle_dbus_idle_monitor_method_info_add_idle_watch =
{
  {
    -1,
    (gchar *) "AddIdleWatch",
    (GDBusArgInfo **) &_phosh_idle_dbus_idle_monitor_method_info_add_idle_watch_IN_ARG_pointers,
    (GDBusArgInfo **) &_phosh_idle_dbus_idle_monitor_method_info_add_idle_watch_OUT_ARG_pointers,
    NULL
  },
  "handle-add-idle-watch",
  FALSE
};

static const _ExtendedGDBusArgInfo _phosh_idle_dbus_idle_monitor_method_info_add_user_active_watch_OUT_ARG_id =
{
  {
    -1,
    (gchar *) "id",
    (gchar *) "u",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _phosh_idle_dbus_idle_monitor_method_info_add_user_active_watch_OUT_ARG_pointers[] =
{
  &_phosh_idle_dbus_idle_monitor_method_info_add_user_active_watch_OUT_ARG_id.parent_struct,
  NULL
};

static const _ExtendedGDBusMethodInfo _phosh_idle_dbus_idle_monitor_method_info_add_user_active_watch =
{
  {
    -1,
    (gchar *) "AddUserActiveWatch",
    NULL,
    (GDBusArgInfo **) &_phosh_idle_dbus_idle_monitor_method_info_add_user_active_watch_OUT_ARG_pointers,
    NULL
  },
  "handle-add-user-active-watch",
  FALSE
};

static const _ExtendedGDBusArgInfo _phosh_idle_dbus_idle_monitor_method_info_remove_watch_IN_ARG_id =
{
  {
    -1,
    (gchar *) "id",
    (gchar *) "u",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _phosh_idle_dbus_idle_monitor_method_info_remove_watch_IN_ARG_pointers[] =
{
  &_phosh_idle_dbus_idle_monitor_method_info_remove_watch_IN_ARG_id.parent_struct,
  NULL
};

static const _ExtendedGDBusMethodInfo _phosh_idle_dbus_idle_monitor_method_info_remove_watch =
{
  {
    -1,
    (gchar *) "RemoveWatch",
    (GDBusArgInfo **) &_phosh_idle_dbus_idle_monitor_method_info_remove_watch_IN_ARG_pointers,
    NULL,
    NULL
  },
  "handle-remove-watch",
  FALSE
};

static const GDBusMethodInfo * const _phosh_idle_dbus_idle_monitor_method_info_pointers[] =
{
  &_phosh_idle_dbus_idle_monitor_method_info_get_idletime.parent_struct,
  &_phosh_idle_dbus_idle_monitor_method_info_add_idle_watch.parent_struct,
  &_phosh_idle_dbus_idle_monitor_method_info_add_user_active_watch.parent_struct,
  &_phosh_idle_dbus_idle_monitor_method_info_remove_watch.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _phosh_idle_dbus_idle_monitor_signal_info_watch_fired_ARG_id =
{
  {
    -1,
    (gchar *) "id",
    (gchar *) "u",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _phosh_idle_dbus_idle_monitor_signal_info_watch_fired_ARG_pointers[] =
{
  &_phosh_idle_dbus_idle_monitor_signal_info_watch_fired_ARG_id.parent_struct,
  NULL
};

static const _ExtendedGDBusSignalInfo _phosh_idle_dbus_idle_monitor_signal_info_watch_fired =
{
  {
    -1,
    (gchar *) "WatchFired",
    (GDBusArgInfo **) &_phosh_idle_dbus_idle_monitor_signal_info_watch_fired_ARG_pointers,
    NULL
  },
  "watch-fired"
};

static const GDBusSignalInfo * const _phosh_idle_dbus_idle_monitor_signal_info_pointers[] =
{
  &_phosh_idle_dbus_idle_monitor_signal_info_watch_fired.parent_struct,
  NULL
};

static const _ExtendedGDBusInterfaceInfo _phosh_idle_dbus_idle_monitor_interface_info =
{
  {
    -1,
    (gchar *) "org.gnome.Mutter.IdleMonitor",
    (GDBusMethodInfo **) &_phosh_idle_dbus_idle_monitor_method_info_pointers,
    (GDBusSignalInfo **) &_phosh_idle_dbus_idle_monitor_signal_info_pointers,
    NULL,
    NULL
  },
  "idle-monitor",
};


/**
 * phosh_idle_dbus_idle_monitor_interface_info:
 *
 * Gets a machine-readable description of the <link linkend="gdbus-interface-org-gnome-Mutter-IdleMonitor.top_of_page">org.gnome.Mutter.IdleMonitor</link> D-Bus interface.
 *
 * Returns: (transfer none): A #GDBusInterfaceInfo. Do not free.
 */
GDBusInterfaceInfo *
phosh_idle_dbus_idle_monitor_interface_info (void)
{
  return (GDBusInterfaceInfo *) &_phosh_idle_dbus_idle_monitor_interface_info.parent_struct;
}

/**
 * phosh_idle_dbus_idle_monitor_override_properties:
 * @klass: The class structure for a #GObject derived class.
 * @property_id_begin: The property id to assign to the first overridden property.
 *
 * Overrides all #GObject properties in the #PhoshIdleDBusIdleMonitor interface for a concrete class.
 * The properties are overridden in the order they are defined.
 *
 * Returns: The last property id.
 */
guint
phosh_idle_dbus_idle_monitor_override_properties (GObjectClass *klass G_GNUC_UNUSED, guint property_id_begin)
{
  return property_id_begin - 1;
}


inline static void
phosh_idle_dbus_idle_monitor_signal_marshal_watch_fired (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  g_cclosure_marshal_VOID__UINT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
phosh_idle_dbus_idle_monitor_method_marshal_get_idletime (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
phosh_idle_dbus_idle_monitor_method_marshal_add_idle_watch (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_UINT64 (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
phosh_idle_dbus_idle_monitor_method_marshal_add_user_active_watch (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
phosh_idle_dbus_idle_monitor_method_marshal_remove_watch (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_UINT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}


/**
 * PhoshIdleDBusIdleMonitor:
 *
 * Abstract interface type for the D-Bus interface <link linkend="gdbus-interface-org-gnome-Mutter-IdleMonitor.top_of_page">org.gnome.Mutter.IdleMonitor</link>.
 */

/**
 * PhoshIdleDBusIdleMonitorIface:
 * @parent_iface: The parent interface.
 * @handle_add_idle_watch: Handler for the #PhoshIdleDBusIdleMonitor::handle-add-idle-watch signal.
 * @handle_add_user_active_watch: Handler for the #PhoshIdleDBusIdleMonitor::handle-add-user-active-watch signal.
 * @handle_get_idletime: Handler for the #PhoshIdleDBusIdleMonitor::handle-get-idletime signal.
 * @handle_remove_watch: Handler for the #PhoshIdleDBusIdleMonitor::handle-remove-watch signal.
 * @watch_fired: Handler for the #PhoshIdleDBusIdleMonitor::watch-fired signal.
 *
 * Virtual table for the D-Bus interface <link linkend="gdbus-interface-org-gnome-Mutter-IdleMonitor.top_of_page">org.gnome.Mutter.IdleMonitor</link>.
 */

typedef PhoshIdleDBusIdleMonitorIface PhoshIdleDBusIdleMonitorInterface;
G_DEFINE_INTERFACE (PhoshIdleDBusIdleMonitor, phosh_idle_dbus_idle_monitor, G_TYPE_OBJECT)

static void
phosh_idle_dbus_idle_monitor_default_init (PhoshIdleDBusIdleMonitorIface *iface)
{
  /* GObject signals for incoming D-Bus method calls: */
  /**
   * PhoshIdleDBusIdleMonitor::handle-get-idletime:
   * @object: A #PhoshIdleDBusIdleMonitor.
   * @invocation: A #GDBusMethodInvocation.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-gnome-Mutter-IdleMonitor.GetIdletime">GetIdletime()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call phosh_idle_dbus_idle_monitor_complete_get_idletime() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-get-idletime",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (PhoshIdleDBusIdleMonitorIface, handle_get_idletime),
    g_signal_accumulator_true_handled,
    NULL,
      phosh_idle_dbus_idle_monitor_method_marshal_get_idletime,
    G_TYPE_BOOLEAN,
    1,
    G_TYPE_DBUS_METHOD_INVOCATION);

  /**
   * PhoshIdleDBusIdleMonitor::handle-add-idle-watch:
   * @object: A #PhoshIdleDBusIdleMonitor.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_interval: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-gnome-Mutter-IdleMonitor.AddIdleWatch">AddIdleWatch()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call phosh_idle_dbus_idle_monitor_complete_add_idle_watch() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-add-idle-watch",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (PhoshIdleDBusIdleMonitorIface, handle_add_idle_watch),
    g_signal_accumulator_true_handled,
    NULL,
      phosh_idle_dbus_idle_monitor_method_marshal_add_idle_watch,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_UINT64);

  /**
   * PhoshIdleDBusIdleMonitor::handle-add-user-active-watch:
   * @object: A #PhoshIdleDBusIdleMonitor.
   * @invocation: A #GDBusMethodInvocation.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-gnome-Mutter-IdleMonitor.AddUserActiveWatch">AddUserActiveWatch()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call phosh_idle_dbus_idle_monitor_complete_add_user_active_watch() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-add-user-active-watch",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (PhoshIdleDBusIdleMonitorIface, handle_add_user_active_watch),
    g_signal_accumulator_true_handled,
    NULL,
      phosh_idle_dbus_idle_monitor_method_marshal_add_user_active_watch,
    G_TYPE_BOOLEAN,
    1,
    G_TYPE_DBUS_METHOD_INVOCATION);

  /**
   * PhoshIdleDBusIdleMonitor::handle-remove-watch:
   * @object: A #PhoshIdleDBusIdleMonitor.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_id: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-gnome-Mutter-IdleMonitor.RemoveWatch">RemoveWatch()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call phosh_idle_dbus_idle_monitor_complete_remove_watch() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-remove-watch",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (PhoshIdleDBusIdleMonitorIface, handle_remove_watch),
    g_signal_accumulator_true_handled,
    NULL,
      phosh_idle_dbus_idle_monitor_method_marshal_remove_watch,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_UINT);

  /* GObject signals for received D-Bus signals: */
  /**
   * PhoshIdleDBusIdleMonitor::watch-fired:
   * @object: A #PhoshIdleDBusIdleMonitor.
   * @arg_id: Argument.
   *
   * On the client-side, this signal is emitted whenever the D-Bus signal <link linkend="gdbus-signal-org-gnome-Mutter-IdleMonitor.WatchFired">"WatchFired"</link> is received.
   *
   * On the service-side, this signal can be used with e.g. g_signal_emit_by_name() to make the object emit the D-Bus signal.
   */
  PHOSH_IDLE_DBUS__IDLE_MONITOR_SIGNALS[PHOSH_IDLE_DBUS__IDLE_MONITOR_WATCH_FIRED] =
    g_signal_new ("watch-fired",
      G_TYPE_FROM_INTERFACE (iface),
      G_SIGNAL_RUN_LAST,
      G_STRUCT_OFFSET (PhoshIdleDBusIdleMonitorIface, watch_fired),
      NULL,
      NULL,
      phosh_idle_dbus_idle_monitor_signal_marshal_watch_fired,
      G_TYPE_NONE,
      1, G_TYPE_UINT);

}

/**
 * phosh_idle_dbus_idle_monitor_emit_watch_fired:
 * @object: A #PhoshIdleDBusIdleMonitor.
 * @arg_id: Argument to pass with the signal.
 *
 * Emits the <link linkend="gdbus-signal-org-gnome-Mutter-IdleMonitor.WatchFired">"WatchFired"</link> D-Bus signal.
 */
void
phosh_idle_dbus_idle_monitor_emit_watch_fired (
    PhoshIdleDBusIdleMonitor *object,
    guint arg_id)
{
  g_signal_emit (object, PHOSH_IDLE_DBUS__IDLE_MONITOR_SIGNALS[PHOSH_IDLE_DBUS__IDLE_MONITOR_WATCH_FIRED], 0, arg_id);
}

/**
 * phosh_idle_dbus_idle_monitor_call_get_idletime:
 * @proxy: A #PhoshIdleDBusIdleMonitorProxy.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-gnome-Mutter-IdleMonitor.GetIdletime">GetIdletime()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call phosh_idle_dbus_idle_monitor_call_get_idletime_finish() to get the result of the operation.
 *
 * See phosh_idle_dbus_idle_monitor_call_get_idletime_sync() for the synchronous, blocking version of this method.
 */
void
phosh_idle_dbus_idle_monitor_call_get_idletime (
    PhoshIdleDBusIdleMonitor *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "GetIdletime",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * phosh_idle_dbus_idle_monitor_call_get_idletime_finish:
 * @proxy: A #PhoshIdleDBusIdleMonitorProxy.
 * @out_idletime: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to phosh_idle_dbus_idle_monitor_call_get_idletime().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with phosh_idle_dbus_idle_monitor_call_get_idletime().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
phosh_idle_dbus_idle_monitor_call_get_idletime_finish (
    PhoshIdleDBusIdleMonitor *proxy,
    guint64* out_idletime,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(t)",
                 out_idletime);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * phosh_idle_dbus_idle_monitor_call_get_idletime_sync:
 * @proxy: A #PhoshIdleDBusIdleMonitorProxy.
 * @out_idletime: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-gnome-Mutter-IdleMonitor.GetIdletime">GetIdletime()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See phosh_idle_dbus_idle_monitor_call_get_idletime() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
phosh_idle_dbus_idle_monitor_call_get_idletime_sync (
    PhoshIdleDBusIdleMonitor *proxy,
    guint64* out_idletime,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "GetIdletime",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(t)",
                 out_idletime);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * phosh_idle_dbus_idle_monitor_call_add_idle_watch:
 * @proxy: A #PhoshIdleDBusIdleMonitorProxy.
 * @arg_interval: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-gnome-Mutter-IdleMonitor.AddIdleWatch">AddIdleWatch()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call phosh_idle_dbus_idle_monitor_call_add_idle_watch_finish() to get the result of the operation.
 *
 * See phosh_idle_dbus_idle_monitor_call_add_idle_watch_sync() for the synchronous, blocking version of this method.
 */
void
phosh_idle_dbus_idle_monitor_call_add_idle_watch (
    PhoshIdleDBusIdleMonitor *proxy,
    guint64 arg_interval,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "AddIdleWatch",
    g_variant_new ("(t)",
                   arg_interval),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * phosh_idle_dbus_idle_monitor_call_add_idle_watch_finish:
 * @proxy: A #PhoshIdleDBusIdleMonitorProxy.
 * @out_id: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to phosh_idle_dbus_idle_monitor_call_add_idle_watch().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with phosh_idle_dbus_idle_monitor_call_add_idle_watch().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
phosh_idle_dbus_idle_monitor_call_add_idle_watch_finish (
    PhoshIdleDBusIdleMonitor *proxy,
    guint* out_id,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(u)",
                 out_id);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * phosh_idle_dbus_idle_monitor_call_add_idle_watch_sync:
 * @proxy: A #PhoshIdleDBusIdleMonitorProxy.
 * @arg_interval: Argument to pass with the method invocation.
 * @out_id: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-gnome-Mutter-IdleMonitor.AddIdleWatch">AddIdleWatch()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See phosh_idle_dbus_idle_monitor_call_add_idle_watch() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
phosh_idle_dbus_idle_monitor_call_add_idle_watch_sync (
    PhoshIdleDBusIdleMonitor *proxy,
    guint64 arg_interval,
    guint* out_id,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "AddIdleWatch",
    g_variant_new ("(t)",
                   arg_interval),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(u)",
                 out_id);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * phosh_idle_dbus_idle_monitor_call_add_user_active_watch:
 * @proxy: A #PhoshIdleDBusIdleMonitorProxy.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-gnome-Mutter-IdleMonitor.AddUserActiveWatch">AddUserActiveWatch()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call phosh_idle_dbus_idle_monitor_call_add_user_active_watch_finish() to get the result of the operation.
 *
 * See phosh_idle_dbus_idle_monitor_call_add_user_active_watch_sync() for the synchronous, blocking version of this method.
 */
void
phosh_idle_dbus_idle_monitor_call_add_user_active_watch (
    PhoshIdleDBusIdleMonitor *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "AddUserActiveWatch",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * phosh_idle_dbus_idle_monitor_call_add_user_active_watch_finish:
 * @proxy: A #PhoshIdleDBusIdleMonitorProxy.
 * @out_id: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to phosh_idle_dbus_idle_monitor_call_add_user_active_watch().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with phosh_idle_dbus_idle_monitor_call_add_user_active_watch().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
phosh_idle_dbus_idle_monitor_call_add_user_active_watch_finish (
    PhoshIdleDBusIdleMonitor *proxy,
    guint* out_id,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(u)",
                 out_id);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * phosh_idle_dbus_idle_monitor_call_add_user_active_watch_sync:
 * @proxy: A #PhoshIdleDBusIdleMonitorProxy.
 * @out_id: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-gnome-Mutter-IdleMonitor.AddUserActiveWatch">AddUserActiveWatch()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See phosh_idle_dbus_idle_monitor_call_add_user_active_watch() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
phosh_idle_dbus_idle_monitor_call_add_user_active_watch_sync (
    PhoshIdleDBusIdleMonitor *proxy,
    guint* out_id,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "AddUserActiveWatch",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(u)",
                 out_id);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * phosh_idle_dbus_idle_monitor_call_remove_watch:
 * @proxy: A #PhoshIdleDBusIdleMonitorProxy.
 * @arg_id: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-gnome-Mutter-IdleMonitor.RemoveWatch">RemoveWatch()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call phosh_idle_dbus_idle_monitor_call_remove_watch_finish() to get the result of the operation.
 *
 * See phosh_idle_dbus_idle_monitor_call_remove_watch_sync() for the synchronous, blocking version of this method.
 */
void
phosh_idle_dbus_idle_monitor_call_remove_watch (
    PhoshIdleDBusIdleMonitor *proxy,
    guint arg_id,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "RemoveWatch",
    g_variant_new ("(u)",
                   arg_id),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * phosh_idle_dbus_idle_monitor_call_remove_watch_finish:
 * @proxy: A #PhoshIdleDBusIdleMonitorProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to phosh_idle_dbus_idle_monitor_call_remove_watch().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with phosh_idle_dbus_idle_monitor_call_remove_watch().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
phosh_idle_dbus_idle_monitor_call_remove_watch_finish (
    PhoshIdleDBusIdleMonitor *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * phosh_idle_dbus_idle_monitor_call_remove_watch_sync:
 * @proxy: A #PhoshIdleDBusIdleMonitorProxy.
 * @arg_id: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-gnome-Mutter-IdleMonitor.RemoveWatch">RemoveWatch()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See phosh_idle_dbus_idle_monitor_call_remove_watch() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
phosh_idle_dbus_idle_monitor_call_remove_watch_sync (
    PhoshIdleDBusIdleMonitor *proxy,
    guint arg_id,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "RemoveWatch",
    g_variant_new ("(u)",
                   arg_id),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * phosh_idle_dbus_idle_monitor_complete_get_idletime:
 * @object: A #PhoshIdleDBusIdleMonitor.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @idletime: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-gnome-Mutter-IdleMonitor.GetIdletime">GetIdletime()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
phosh_idle_dbus_idle_monitor_complete_get_idletime (
    PhoshIdleDBusIdleMonitor *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    guint64 idletime)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(t)",
                   idletime));
}

/**
 * phosh_idle_dbus_idle_monitor_complete_add_idle_watch:
 * @object: A #PhoshIdleDBusIdleMonitor.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @id: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-gnome-Mutter-IdleMonitor.AddIdleWatch">AddIdleWatch()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
phosh_idle_dbus_idle_monitor_complete_add_idle_watch (
    PhoshIdleDBusIdleMonitor *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    guint id)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(u)",
                   id));
}

/**
 * phosh_idle_dbus_idle_monitor_complete_add_user_active_watch:
 * @object: A #PhoshIdleDBusIdleMonitor.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @id: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-gnome-Mutter-IdleMonitor.AddUserActiveWatch">AddUserActiveWatch()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
phosh_idle_dbus_idle_monitor_complete_add_user_active_watch (
    PhoshIdleDBusIdleMonitor *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    guint id)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(u)",
                   id));
}

/**
 * phosh_idle_dbus_idle_monitor_complete_remove_watch:
 * @object: A #PhoshIdleDBusIdleMonitor.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-gnome-Mutter-IdleMonitor.RemoveWatch">RemoveWatch()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
phosh_idle_dbus_idle_monitor_complete_remove_watch (
    PhoshIdleDBusIdleMonitor *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/* ------------------------------------------------------------------------ */

/**
 * PhoshIdleDBusIdleMonitorProxy:
 *
 * The #PhoshIdleDBusIdleMonitorProxy structure contains only private data and should only be accessed using the provided API.
 */

/**
 * PhoshIdleDBusIdleMonitorProxyClass:
 * @parent_class: The parent class.
 *
 * Class structure for #PhoshIdleDBusIdleMonitorProxy.
 */

struct _PhoshIdleDBusIdleMonitorProxyPrivate
{
  GData *qdata;
};

static void phosh_idle_dbus_idle_monitor_proxy_iface_init (PhoshIdleDBusIdleMonitorIface *iface);

#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
G_DEFINE_TYPE_WITH_CODE (PhoshIdleDBusIdleMonitorProxy, phosh_idle_dbus_idle_monitor_proxy, G_TYPE_DBUS_PROXY,
                         G_ADD_PRIVATE (PhoshIdleDBusIdleMonitorProxy)
                         G_IMPLEMENT_INTERFACE (PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR, phosh_idle_dbus_idle_monitor_proxy_iface_init))

#else
G_DEFINE_TYPE_WITH_CODE (PhoshIdleDBusIdleMonitorProxy, phosh_idle_dbus_idle_monitor_proxy, G_TYPE_DBUS_PROXY,
                         G_IMPLEMENT_INTERFACE (PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR, phosh_idle_dbus_idle_monitor_proxy_iface_init))

#endif
static void
phosh_idle_dbus_idle_monitor_proxy_finalize (GObject *object)
{
  PhoshIdleDBusIdleMonitorProxy *proxy = PHOSH_IDLE_DBUS_IDLE_MONITOR_PROXY (object);
  g_datalist_clear (&proxy->priv->qdata);
  G_OBJECT_CLASS (phosh_idle_dbus_idle_monitor_proxy_parent_class)->finalize (object);
}

static void
phosh_idle_dbus_idle_monitor_proxy_get_property (GObject      *object G_GNUC_UNUSED,
  guint         prop_id G_GNUC_UNUSED,
  GValue       *value G_GNUC_UNUSED,
  GParamSpec   *pspec G_GNUC_UNUSED)
{
}

static void
phosh_idle_dbus_idle_monitor_proxy_set_property (GObject      *object G_GNUC_UNUSED,
  guint         prop_id G_GNUC_UNUSED,
  const GValue *value G_GNUC_UNUSED,
  GParamSpec   *pspec G_GNUC_UNUSED)
{
}

static void
phosh_idle_dbus_idle_monitor_proxy_g_signal (GDBusProxy *proxy,
  const gchar *sender_name G_GNUC_UNUSED,
  const gchar *signal_name,
  GVariant *parameters)
{
  _ExtendedGDBusSignalInfo *info;
  GVariantIter iter;
  GVariant *child;
  GValue *paramv;
  gsize num_params;
  gsize n;
  guint signal_id;
  info = (_ExtendedGDBusSignalInfo *) g_dbus_interface_info_lookup_signal ((GDBusInterfaceInfo *) &_phosh_idle_dbus_idle_monitor_interface_info.parent_struct, signal_name);
  if (info == NULL)
    return;
  num_params = g_variant_n_children (parameters);
  paramv = g_new0 (GValue, num_params + 1);
  g_value_init (&paramv[0], PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR);
  g_value_set_object (&paramv[0], proxy);
  g_variant_iter_init (&iter, parameters);
  n = 1;
  while ((child = g_variant_iter_next_value (&iter)) != NULL)
    {
      _ExtendedGDBusArgInfo *arg_info = (_ExtendedGDBusArgInfo *) info->parent_struct.args[n - 1];
      if (arg_info->use_gvariant)
        {
          g_value_init (&paramv[n], G_TYPE_VARIANT);
          g_value_set_variant (&paramv[n], child);
          n++;
        }
      else
        g_dbus_gvariant_to_gvalue (child, &paramv[n++]);
      g_variant_unref (child);
    }
  signal_id = g_signal_lookup (info->signal_name, PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR);
  g_signal_emitv (paramv, signal_id, 0, NULL);
  for (n = 0; n < num_params + 1; n++)
    g_value_unset (&paramv[n]);
  g_free (paramv);
}

static void
phosh_idle_dbus_idle_monitor_proxy_g_properties_changed (GDBusProxy *_proxy,
  GVariant *changed_properties,
  const gchar *const *invalidated_properties)
{
  PhoshIdleDBusIdleMonitorProxy *proxy = PHOSH_IDLE_DBUS_IDLE_MONITOR_PROXY (_proxy);
  guint n;
  const gchar *key;
  GVariantIter *iter;
  _ExtendedGDBusPropertyInfo *info;
  g_variant_get (changed_properties, "a{sv}", &iter);
  while (g_variant_iter_next (iter, "{&sv}", &key, NULL))
    {
      info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_phosh_idle_dbus_idle_monitor_interface_info.parent_struct, key);
      g_datalist_remove_data (&proxy->priv->qdata, key);
      if (info != NULL)
        g_object_notify (G_OBJECT (proxy), info->hyphen_name);
    }
  g_variant_iter_free (iter);
  for (n = 0; invalidated_properties[n] != NULL; n++)
    {
      info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_phosh_idle_dbus_idle_monitor_interface_info.parent_struct, invalidated_properties[n]);
      g_datalist_remove_data (&proxy->priv->qdata, invalidated_properties[n]);
      if (info != NULL)
        g_object_notify (G_OBJECT (proxy), info->hyphen_name);
    }
}

static void
phosh_idle_dbus_idle_monitor_proxy_init (PhoshIdleDBusIdleMonitorProxy *proxy)
{
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
  proxy->priv = phosh_idle_dbus_idle_monitor_proxy_get_instance_private (proxy);
#else
  proxy->priv = G_TYPE_INSTANCE_GET_PRIVATE (proxy, PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR_PROXY, PhoshIdleDBusIdleMonitorProxyPrivate);
#endif

  g_dbus_proxy_set_interface_info (G_DBUS_PROXY (proxy), phosh_idle_dbus_idle_monitor_interface_info ());
}

static void
phosh_idle_dbus_idle_monitor_proxy_class_init (PhoshIdleDBusIdleMonitorProxyClass *klass)
{
  GObjectClass *gobject_class;
  GDBusProxyClass *proxy_class;

  gobject_class = G_OBJECT_CLASS (klass);
  gobject_class->finalize     = phosh_idle_dbus_idle_monitor_proxy_finalize;
  gobject_class->get_property = phosh_idle_dbus_idle_monitor_proxy_get_property;
  gobject_class->set_property = phosh_idle_dbus_idle_monitor_proxy_set_property;

  proxy_class = G_DBUS_PROXY_CLASS (klass);
  proxy_class->g_signal = phosh_idle_dbus_idle_monitor_proxy_g_signal;
  proxy_class->g_properties_changed = phosh_idle_dbus_idle_monitor_proxy_g_properties_changed;

#if GLIB_VERSION_MAX_ALLOWED < GLIB_VERSION_2_38
  g_type_class_add_private (klass, sizeof (PhoshIdleDBusIdleMonitorProxyPrivate));
#endif
}

static void
phosh_idle_dbus_idle_monitor_proxy_iface_init (PhoshIdleDBusIdleMonitorIface *iface G_GNUC_UNUSED)
{
}

/**
 * phosh_idle_dbus_idle_monitor_proxy_new:
 * @connection: A #GDBusConnection.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: (nullable): A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-gnome-Mutter-IdleMonitor.top_of_page">org.gnome.Mutter.IdleMonitor</link>. See g_dbus_proxy_new() for more details.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call phosh_idle_dbus_idle_monitor_proxy_new_finish() to get the result of the operation.
 *
 * See phosh_idle_dbus_idle_monitor_proxy_new_sync() for the synchronous, blocking version of this constructor.
 */
void
phosh_idle_dbus_idle_monitor_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data)
{
  g_async_initable_new_async (PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR_PROXY, G_PRIORITY_DEFAULT, cancellable, callback, user_data, "g-flags", flags, "g-name", name, "g-connection", connection, "g-object-path", object_path, "g-interface-name", "org.gnome.Mutter.IdleMonitor", NULL);
}

/**
 * phosh_idle_dbus_idle_monitor_proxy_new_finish:
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to phosh_idle_dbus_idle_monitor_proxy_new().
 * @error: Return location for error or %NULL
 *
 * Finishes an operation started with phosh_idle_dbus_idle_monitor_proxy_new().
 *
 * Returns: (transfer full) (type PhoshIdleDBusIdleMonitorProxy): The constructed proxy object or %NULL if @error is set.
 */
PhoshIdleDBusIdleMonitor *
phosh_idle_dbus_idle_monitor_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error)
{
  GObject *ret;
  GObject *source_object;
  source_object = g_async_result_get_source_object (res);
  ret = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), res, error);
  g_object_unref (source_object);
  if (ret != NULL)
    return PHOSH_IDLE_DBUS_IDLE_MONITOR (ret);
  else
    return NULL;
}

/**
 * phosh_idle_dbus_idle_monitor_proxy_new_sync:
 * @connection: A #GDBusConnection.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: (nullable): A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL
 *
 * Synchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-gnome-Mutter-IdleMonitor.top_of_page">org.gnome.Mutter.IdleMonitor</link>. See g_dbus_proxy_new_sync() for more details.
 *
 * The calling thread is blocked until a reply is received.
 *
 * See phosh_idle_dbus_idle_monitor_proxy_new() for the asynchronous version of this constructor.
 *
 * Returns: (transfer full) (type PhoshIdleDBusIdleMonitorProxy): The constructed proxy object or %NULL if @error is set.
 */
PhoshIdleDBusIdleMonitor *
phosh_idle_dbus_idle_monitor_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error)
{
  GInitable *ret;
  ret = g_initable_new (PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR_PROXY, cancellable, error, "g-flags", flags, "g-name", name, "g-connection", connection, "g-object-path", object_path, "g-interface-name", "org.gnome.Mutter.IdleMonitor", NULL);
  if (ret != NULL)
    return PHOSH_IDLE_DBUS_IDLE_MONITOR (ret);
  else
    return NULL;
}


/**
 * phosh_idle_dbus_idle_monitor_proxy_new_for_bus:
 * @bus_type: A #GBusType.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: A bus name (well-known or unique).
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: User data to pass to @callback.
 *
 * Like phosh_idle_dbus_idle_monitor_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call phosh_idle_dbus_idle_monitor_proxy_new_for_bus_finish() to get the result of the operation.
 *
 * See phosh_idle_dbus_idle_monitor_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
 */
void
phosh_idle_dbus_idle_monitor_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data)
{
  g_async_initable_new_async (PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR_PROXY, G_PRIORITY_DEFAULT, cancellable, callback, user_data, "g-flags", flags, "g-name", name, "g-bus-type", bus_type, "g-object-path", object_path, "g-interface-name", "org.gnome.Mutter.IdleMonitor", NULL);
}

/**
 * phosh_idle_dbus_idle_monitor_proxy_new_for_bus_finish:
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to phosh_idle_dbus_idle_monitor_proxy_new_for_bus().
 * @error: Return location for error or %NULL
 *
 * Finishes an operation started with phosh_idle_dbus_idle_monitor_proxy_new_for_bus().
 *
 * Returns: (transfer full) (type PhoshIdleDBusIdleMonitorProxy): The constructed proxy object or %NULL if @error is set.
 */
PhoshIdleDBusIdleMonitor *
phosh_idle_dbus_idle_monitor_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error)
{
  GObject *ret;
  GObject *source_object;
  source_object = g_async_result_get_source_object (res);
  ret = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), res, error);
  g_object_unref (source_object);
  if (ret != NULL)
    return PHOSH_IDLE_DBUS_IDLE_MONITOR (ret);
  else
    return NULL;
}

/**
 * phosh_idle_dbus_idle_monitor_proxy_new_for_bus_sync:
 * @bus_type: A #GBusType.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: A bus name (well-known or unique).
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL
 *
 * Like phosh_idle_dbus_idle_monitor_proxy_new_sync() but takes a #GBusType instead of a #GDBusConnection.
 *
 * The calling thread is blocked until a reply is received.
 *
 * See phosh_idle_dbus_idle_monitor_proxy_new_for_bus() for the asynchronous version of this constructor.
 *
 * Returns: (transfer full) (type PhoshIdleDBusIdleMonitorProxy): The constructed proxy object or %NULL if @error is set.
 */
PhoshIdleDBusIdleMonitor *
phosh_idle_dbus_idle_monitor_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error)
{
  GInitable *ret;
  ret = g_initable_new (PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR_PROXY, cancellable, error, "g-flags", flags, "g-name", name, "g-bus-type", bus_type, "g-object-path", object_path, "g-interface-name", "org.gnome.Mutter.IdleMonitor", NULL);
  if (ret != NULL)
    return PHOSH_IDLE_DBUS_IDLE_MONITOR (ret);
  else
    return NULL;
}


/* ------------------------------------------------------------------------ */

/**
 * PhoshIdleDBusIdleMonitorSkeleton:
 *
 * The #PhoshIdleDBusIdleMonitorSkeleton structure contains only private data and should only be accessed using the provided API.
 */

/**
 * PhoshIdleDBusIdleMonitorSkeletonClass:
 * @parent_class: The parent class.
 *
 * Class structure for #PhoshIdleDBusIdleMonitorSkeleton.
 */

struct _PhoshIdleDBusIdleMonitorSkeletonPrivate
{
  GValue *properties;
  GList *changed_properties;
  GSource *changed_properties_idle_source;
  GMainContext *context;
  GMutex lock;
};

static void
_phosh_idle_dbus_idle_monitor_skeleton_handle_method_call (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name,
  const gchar *method_name,
  GVariant *parameters,
  GDBusMethodInvocation *invocation,
  gpointer user_data)
{
  PhoshIdleDBusIdleMonitorSkeleton *skeleton = PHOSH_IDLE_DBUS_IDLE_MONITOR_SKELETON (user_data);
  _ExtendedGDBusMethodInfo *info;
  GVariantIter iter;
  GVariant *child;
  GValue *paramv;
  gsize num_params;
  guint num_extra;
  gsize n;
  guint signal_id;
  GValue return_value = G_VALUE_INIT;
  info = (_ExtendedGDBusMethodInfo *) g_dbus_method_invocation_get_method_info (invocation);
  g_assert (info != NULL);
  num_params = g_variant_n_children (parameters);
  num_extra = info->pass_fdlist ? 3 : 2;  paramv = g_new0 (GValue, num_params + num_extra);
  n = 0;
  g_value_init (&paramv[n], PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR);
  g_value_set_object (&paramv[n++], skeleton);
  g_value_init (&paramv[n], G_TYPE_DBUS_METHOD_INVOCATION);
  g_value_set_object (&paramv[n++], invocation);
  if (info->pass_fdlist)
    {
#ifdef G_OS_UNIX
      g_value_init (&paramv[n], G_TYPE_UNIX_FD_LIST);
      g_value_set_object (&paramv[n++], g_dbus_message_get_unix_fd_list (g_dbus_method_invocation_get_message (invocation)));
#else
      g_assert_not_reached ();
#endif
    }
  g_variant_iter_init (&iter, parameters);
  while ((child = g_variant_iter_next_value (&iter)) != NULL)
    {
      _ExtendedGDBusArgInfo *arg_info = (_ExtendedGDBusArgInfo *) info->parent_struct.in_args[n - num_extra];
      if (arg_info->use_gvariant)
        {
          g_value_init (&paramv[n], G_TYPE_VARIANT);
          g_value_set_variant (&paramv[n], child);
          n++;
        }
      else
        g_dbus_gvariant_to_gvalue (child, &paramv[n++]);
      g_variant_unref (child);
    }
  signal_id = g_signal_lookup (info->signal_name, PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR);
  g_value_init (&return_value, G_TYPE_BOOLEAN);
  g_signal_emitv (paramv, signal_id, 0, &return_value);
  if (!g_value_get_boolean (&return_value))
    g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_METHOD, "Method %s is not implemented on interface %s", method_name, interface_name);
  g_value_unset (&return_value);
  for (n = 0; n < num_params + num_extra; n++)
    g_value_unset (&paramv[n]);
  g_free (paramv);
}

static GVariant *
_phosh_idle_dbus_idle_monitor_skeleton_handle_get_property (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name G_GNUC_UNUSED,
  const gchar *property_name,
  GError **error,
  gpointer user_data)
{
  PhoshIdleDBusIdleMonitorSkeleton *skeleton = PHOSH_IDLE_DBUS_IDLE_MONITOR_SKELETON (user_data);
  GValue value = G_VALUE_INIT;
  GParamSpec *pspec;
  _ExtendedGDBusPropertyInfo *info;
  GVariant *ret;
  ret = NULL;
  info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_phosh_idle_dbus_idle_monitor_interface_info.parent_struct, property_name);
  g_assert (info != NULL);
  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (skeleton), info->hyphen_name);
  if (pspec == NULL)
    {
      g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS, "No property with name %s", property_name);
    }
  else
    {
      g_value_init (&value, pspec->value_type);
      g_object_get_property (G_OBJECT (skeleton), info->hyphen_name, &value);
      ret = g_dbus_gvalue_to_gvariant (&value, G_VARIANT_TYPE (info->parent_struct.signature));
      g_value_unset (&value);
    }
  return ret;
}

static gboolean
_phosh_idle_dbus_idle_monitor_skeleton_handle_set_property (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name G_GNUC_UNUSED,
  const gchar *property_name,
  GVariant *variant,
  GError **error,
  gpointer user_data)
{
  PhoshIdleDBusIdleMonitorSkeleton *skeleton = PHOSH_IDLE_DBUS_IDLE_MONITOR_SKELETON (user_data);
  GValue value = G_VALUE_INIT;
  GParamSpec *pspec;
  _ExtendedGDBusPropertyInfo *info;
  gboolean ret;
  ret = FALSE;
  info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_phosh_idle_dbus_idle_monitor_interface_info.parent_struct, property_name);
  g_assert (info != NULL);
  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (skeleton), info->hyphen_name);
  if (pspec == NULL)
    {
      g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS, "No property with name %s", property_name);
    }
  else
    {
      if (info->use_gvariant)
        g_value_set_variant (&value, variant);
      else
        g_dbus_gvariant_to_gvalue (variant, &value);
      g_object_set_property (G_OBJECT (skeleton), info->hyphen_name, &value);
      g_value_unset (&value);
      ret = TRUE;
    }
  return ret;
}

static const GDBusInterfaceVTable _phosh_idle_dbus_idle_monitor_skeleton_vtable =
{
  _phosh_idle_dbus_idle_monitor_skeleton_handle_method_call,
  _phosh_idle_dbus_idle_monitor_skeleton_handle_get_property,
  _phosh_idle_dbus_idle_monitor_skeleton_handle_set_property,
  {NULL}
};

static GDBusInterfaceInfo *
phosh_idle_dbus_idle_monitor_skeleton_dbus_interface_get_info (GDBusInterfaceSkeleton *skeleton G_GNUC_UNUSED)
{
  return phosh_idle_dbus_idle_monitor_interface_info ();
}

static GDBusInterfaceVTable *
phosh_idle_dbus_idle_monitor_skeleton_dbus_interface_get_vtable (GDBusInterfaceSkeleton *skeleton G_GNUC_UNUSED)
{
  return (GDBusInterfaceVTable *) &_phosh_idle_dbus_idle_monitor_skeleton_vtable;
}

static GVariant *
phosh_idle_dbus_idle_monitor_skeleton_dbus_interface_get_properties (GDBusInterfaceSkeleton *_skeleton)
{
  PhoshIdleDBusIdleMonitorSkeleton *skeleton = PHOSH_IDLE_DBUS_IDLE_MONITOR_SKELETON (_skeleton);

  GVariantBuilder builder;
  guint n;
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_84
  g_variant_builder_init_static (&builder, G_VARIANT_TYPE ("a{sv}"));
#else
  g_variant_builder_init(&builder, G_VARIANT_TYPE ("a{sv}"));
#endif
  if (_phosh_idle_dbus_idle_monitor_interface_info.parent_struct.properties == NULL)
    goto out;
  for (n = 0; _phosh_idle_dbus_idle_monitor_interface_info.parent_struct.properties[n] != NULL; n++)
    {
      GDBusPropertyInfo *info = _phosh_idle_dbus_idle_monitor_interface_info.parent_struct.properties[n];
      if (info->flags & G_DBUS_PROPERTY_INFO_FLAGS_READABLE)
        {
          GVariant *value;
          value = _phosh_idle_dbus_idle_monitor_skeleton_handle_get_property (g_dbus_interface_skeleton_get_connection (G_DBUS_INTERFACE_SKELETON (skeleton)), NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)), "org.gnome.Mutter.IdleMonitor", info->name, NULL, skeleton);
          if (value != NULL)
            {
              g_variant_take_ref (value);
              g_variant_builder_add (&builder, "{sv}", info->name, value);
              g_variant_unref (value);
            }
        }
    }
out:
  return g_variant_builder_end (&builder);
}

static void
phosh_idle_dbus_idle_monitor_skeleton_dbus_interface_flush (GDBusInterfaceSkeleton *_skeleton G_GNUC_UNUSED)
{
}

static void
_phosh_idle_dbus_idle_monitor_on_signal_watch_fired (
    PhoshIdleDBusIdleMonitor *object,
    guint arg_id)
{
  PhoshIdleDBusIdleMonitorSkeleton *skeleton = PHOSH_IDLE_DBUS_IDLE_MONITOR_SKELETON (object);

  GList      *connections, *l;
  GVariant   *signal_variant;
  connections = g_dbus_interface_skeleton_get_connections (G_DBUS_INTERFACE_SKELETON (skeleton));

  signal_variant = g_variant_ref_sink (g_variant_new ("(u)",
                   arg_id));
  for (l = connections; l != NULL; l = l->next)
    {
      GDBusConnection *connection = l->data;
      g_dbus_connection_emit_signal (connection,
        NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)), "org.gnome.Mutter.IdleMonitor", "WatchFired",
        signal_variant, NULL);
    }
  g_variant_unref (signal_variant);
  g_list_free_full (connections, g_object_unref);
}

static void phosh_idle_dbus_idle_monitor_skeleton_iface_init (PhoshIdleDBusIdleMonitorIface *iface);
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
G_DEFINE_TYPE_WITH_CODE (PhoshIdleDBusIdleMonitorSkeleton, phosh_idle_dbus_idle_monitor_skeleton, G_TYPE_DBUS_INTERFACE_SKELETON,
                         G_ADD_PRIVATE (PhoshIdleDBusIdleMonitorSkeleton)
                         G_IMPLEMENT_INTERFACE (PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR, phosh_idle_dbus_idle_monitor_skeleton_iface_init))

#else
G_DEFINE_TYPE_WITH_CODE (PhoshIdleDBusIdleMonitorSkeleton, phosh_idle_dbus_idle_monitor_skeleton, G_TYPE_DBUS_INTERFACE_SKELETON,
                         G_IMPLEMENT_INTERFACE (PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR, phosh_idle_dbus_idle_monitor_skeleton_iface_init))

#endif
static void
phosh_idle_dbus_idle_monitor_skeleton_finalize (GObject *object)
{
  PhoshIdleDBusIdleMonitorSkeleton *skeleton = PHOSH_IDLE_DBUS_IDLE_MONITOR_SKELETON (object);
  g_list_free_full (skeleton->priv->changed_properties, (GDestroyNotify) _changed_property_free);
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
  /* coverity[missing_lock : SUPPRESS] */
  g_clear_pointer (&skeleton->priv->changed_properties_idle_source, g_source_destroy);
#else
  if (skeleton->priv->changed_properties_idle_source != NULL)
    g_source_destroy (skeleton->priv->changed_properties_idle_source);
skeleton->priv->changed_properties_idle_source = NULL;
#endif
  g_main_context_unref (skeleton->priv->context);
  g_mutex_clear (&skeleton->priv->lock);
  G_OBJECT_CLASS (phosh_idle_dbus_idle_monitor_skeleton_parent_class)->finalize (object);
}

static void
phosh_idle_dbus_idle_monitor_skeleton_init (PhoshIdleDBusIdleMonitorSkeleton *skeleton)
{
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
  skeleton->priv = phosh_idle_dbus_idle_monitor_skeleton_get_instance_private (skeleton);
#else
  skeleton->priv = G_TYPE_INSTANCE_GET_PRIVATE (skeleton, PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR_SKELETON, PhoshIdleDBusIdleMonitorSkeletonPrivate);
#endif

  g_mutex_init (&skeleton->priv->lock);
  skeleton->priv->context = g_main_context_ref_thread_default ();
}

static void
phosh_idle_dbus_idle_monitor_skeleton_class_init (PhoshIdleDBusIdleMonitorSkeletonClass *klass)
{
  GObjectClass *gobject_class;
  GDBusInterfaceSkeletonClass *skeleton_class;

  gobject_class = G_OBJECT_CLASS (klass);
  gobject_class->finalize = phosh_idle_dbus_idle_monitor_skeleton_finalize;

  skeleton_class = G_DBUS_INTERFACE_SKELETON_CLASS (klass);
  skeleton_class->get_info = phosh_idle_dbus_idle_monitor_skeleton_dbus_interface_get_info;
  skeleton_class->get_properties = phosh_idle_dbus_idle_monitor_skeleton_dbus_interface_get_properties;
  skeleton_class->flush = phosh_idle_dbus_idle_monitor_skeleton_dbus_interface_flush;
  skeleton_class->get_vtable = phosh_idle_dbus_idle_monitor_skeleton_dbus_interface_get_vtable;

#if GLIB_VERSION_MAX_ALLOWED < GLIB_VERSION_2_38
  g_type_class_add_private (klass, sizeof (PhoshIdleDBusIdleMonitorSkeletonPrivate));
#endif
}

static void
phosh_idle_dbus_idle_monitor_skeleton_iface_init (PhoshIdleDBusIdleMonitorIface *iface)
{
  iface->watch_fired = _phosh_idle_dbus_idle_monitor_on_signal_watch_fired;
}

/**
 * phosh_idle_dbus_idle_monitor_skeleton_new:
 *
 * Creates a skeleton object for the D-Bus interface <link linkend="gdbus-interface-org-gnome-Mutter-IdleMonitor.top_of_page">org.gnome.Mutter.IdleMonitor</link>.
 *
 * Returns: (transfer full) (type PhoshIdleDBusIdleMonitorSkeleton): The skeleton object.
 */
PhoshIdleDBusIdleMonitor *
phosh_idle_dbus_idle_monitor_skeleton_new (void)
{
  return PHOSH_IDLE_DBUS_IDLE_MONITOR (g_object_new (PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR_SKELETON, NULL));
}

/* ------------------------------------------------------------------------
 * Code for Object, ObjectProxy and ObjectSkeleton
 * ------------------------------------------------------------------------
 */

/**
 * SECTION:PhoshIdleDBusObject
 * @title: PhoshIdleDBusObject
 * @short_description: Specialized GDBusObject types
 *
 * This section contains the #PhoshIdleDBusObject, #PhoshIdleDBusObjectProxy, and #PhoshIdleDBusObjectSkeleton types which make it easier to work with objects implementing generated types for D-Bus interfaces.
 */

/**
 * PhoshIdleDBusObject:
 *
 * The #PhoshIdleDBusObject type is a specialized container of interfaces.
 */

/**
 * PhoshIdleDBusObjectIface:
 * @parent_iface: The parent interface.
 *
 * Virtual table for the #PhoshIdleDBusObject interface.
 */

typedef PhoshIdleDBusObjectIface PhoshIdleDBusObjectInterface;
G_DEFINE_INTERFACE_WITH_CODE (PhoshIdleDBusObject, phosh_idle_dbus_object, G_TYPE_OBJECT, g_type_interface_add_prerequisite (g_define_type_id, G_TYPE_DBUS_OBJECT);)

static void
phosh_idle_dbus_object_default_init (PhoshIdleDBusObjectIface *iface)
{
  /**
   * PhoshIdleDBusObject:idle-monitor:
   *
   * The #PhoshIdleDBusIdleMonitor instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-gnome-Mutter-IdleMonitor.top_of_page">org.gnome.Mutter.IdleMonitor</link>, if any.
   *
   * Connect to the #GObject::notify signal to get informed of property changes.
   */
  g_object_interface_install_property (iface, g_param_spec_object ("idle-monitor", "idle-monitor", "idle-monitor", PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));

}

/**
 * phosh_idle_dbus_object_get_idle_monitor:
 * @object: A #PhoshIdleDBusObject.
 *
 * Gets the #PhoshIdleDBusIdleMonitor instance for the D-Bus interface <link linkend="gdbus-interface-org-gnome-Mutter-IdleMonitor.top_of_page">org.gnome.Mutter.IdleMonitor</link> on @object, if any.
 *
 * Returns: (transfer full) (nullable): A #PhoshIdleDBusIdleMonitor that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
 */
PhoshIdleDBusIdleMonitor *phosh_idle_dbus_object_get_idle_monitor (PhoshIdleDBusObject *object)
{
  GDBusInterface *ret;
  ret = g_dbus_object_get_interface (G_DBUS_OBJECT (object), "org.gnome.Mutter.IdleMonitor");
  if (ret == NULL)
    return NULL;
  return PHOSH_IDLE_DBUS_IDLE_MONITOR (ret);
}


/**
 * phosh_idle_dbus_object_peek_idle_monitor: (skip)
 * @object: A #PhoshIdleDBusObject.
 *
 * Like phosh_idle_dbus_object_get_idle_monitor() but doesn't increase the reference count on the returned object.
 *
 * It is not safe to use the returned object if you are on another thread than the one where the #GDBusObjectManagerClient or #GDBusObjectManagerServer for @object is running.
 *
 * Returns: (transfer none) (nullable): A #PhoshIdleDBusIdleMonitor or %NULL if @object does not implement the interface. Do not free the returned object, it is owned by @object.
 */
PhoshIdleDBusIdleMonitor *phosh_idle_dbus_object_peek_idle_monitor (PhoshIdleDBusObject *object)
{
  GDBusInterface *ret;
  ret = g_dbus_object_get_interface (G_DBUS_OBJECT (object), "org.gnome.Mutter.IdleMonitor");
  if (ret == NULL)
    return NULL;
  g_object_unref (ret);
  return PHOSH_IDLE_DBUS_IDLE_MONITOR (ret);
}


static void
phosh_idle_dbus_object_notify (GDBusObject *object, GDBusInterface *interface)
{
  _ExtendedGDBusInterfaceInfo *info = (_ExtendedGDBusInterfaceInfo *) g_dbus_interface_get_info (interface);
  /* info can be NULL if the other end is using a D-Bus interface we don't know
   * anything about, for example old generated code in this process talking to
   * newer generated code in the other process. */
  if (info != NULL)
    g_object_notify (G_OBJECT (object), info->hyphen_name);
}

/**
 * PhoshIdleDBusObjectProxy:
 *
 * The #PhoshIdleDBusObjectProxy structure contains only private data and should only be accessed using the provided API.
 */

/**
 * PhoshIdleDBusObjectProxyClass:
 * @parent_class: The parent class.
 *
 * Class structure for #PhoshIdleDBusObjectProxy.
 */

static void
phosh_idle_dbus_object_proxy__phosh_idle_dbus_object_iface_init (PhoshIdleDBusObjectIface *iface G_GNUC_UNUSED)
{
}

static void
phosh_idle_dbus_object_proxy__g_dbus_object_iface_init (GDBusObjectIface *iface)
{
  iface->interface_added = phosh_idle_dbus_object_notify;
  iface->interface_removed = phosh_idle_dbus_object_notify;
}


G_DEFINE_TYPE_WITH_CODE (PhoshIdleDBusObjectProxy, phosh_idle_dbus_object_proxy, G_TYPE_DBUS_OBJECT_PROXY,
                         G_IMPLEMENT_INTERFACE (PHOSH_IDLE_DBUS_TYPE_OBJECT, phosh_idle_dbus_object_proxy__phosh_idle_dbus_object_iface_init)
                         G_IMPLEMENT_INTERFACE (G_TYPE_DBUS_OBJECT, phosh_idle_dbus_object_proxy__g_dbus_object_iface_init))

static void
phosh_idle_dbus_object_proxy_init (PhoshIdleDBusObjectProxy *object G_GNUC_UNUSED)
{
}

static void
phosh_idle_dbus_object_proxy_set_property (GObject      *gobject,
  guint         prop_id,
  const GValue *value G_GNUC_UNUSED,
  GParamSpec   *pspec)
{
  G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
}

static void
phosh_idle_dbus_object_proxy_get_property (GObject      *gobject,
  guint         prop_id,
  GValue       *value,
  GParamSpec   *pspec)
{
  PhoshIdleDBusObjectProxy *object = PHOSH_IDLE_DBUS_OBJECT_PROXY (gobject);
  GDBusInterface *interface;

  switch (prop_id)
    {
    case 1:
      interface = g_dbus_object_get_interface (G_DBUS_OBJECT (object), "org.gnome.Mutter.IdleMonitor");
      g_value_take_object (value, interface);
      break;

    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
      break;
  }
}

static void
phosh_idle_dbus_object_proxy_class_init (PhoshIdleDBusObjectProxyClass *klass)
{
  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

  gobject_class->set_property = phosh_idle_dbus_object_proxy_set_property;
  gobject_class->get_property = phosh_idle_dbus_object_proxy_get_property;

  g_object_class_override_property (gobject_class, 1, "idle-monitor");
}

/**
 * phosh_idle_dbus_object_proxy_new:
 * @connection: A #GDBusConnection.
 * @object_path: An object path.
 *
 * Creates a new proxy object.
 *
 * Returns: (transfer full): The proxy object.
 */
PhoshIdleDBusObjectProxy *
phosh_idle_dbus_object_proxy_new (GDBusConnection *connection,
  const gchar *object_path)
{
  g_return_val_if_fail (G_IS_DBUS_CONNECTION (connection), NULL);
  g_return_val_if_fail (g_variant_is_object_path (object_path), NULL);
  return PHOSH_IDLE_DBUS_OBJECT_PROXY (g_object_new (PHOSH_IDLE_DBUS_TYPE_OBJECT_PROXY, "g-connection", connection, "g-object-path", object_path, NULL));
}

/**
 * PhoshIdleDBusObjectSkeleton:
 *
 * The #PhoshIdleDBusObjectSkeleton structure contains only private data and should only be accessed using the provided API.
 */

/**
 * PhoshIdleDBusObjectSkeletonClass:
 * @parent_class: The parent class.
 *
 * Class structure for #PhoshIdleDBusObjectSkeleton.
 */

static void
phosh_idle_dbus_object_skeleton__phosh_idle_dbus_object_iface_init (PhoshIdleDBusObjectIface *iface G_GNUC_UNUSED)
{
}


static void
phosh_idle_dbus_object_skeleton__g_dbus_object_iface_init (GDBusObjectIface *iface)
{
  iface->interface_added = phosh_idle_dbus_object_notify;
  iface->interface_removed = phosh_idle_dbus_object_notify;
}

G_DEFINE_TYPE_WITH_CODE (PhoshIdleDBusObjectSkeleton, phosh_idle_dbus_object_skeleton, G_TYPE_DBUS_OBJECT_SKELETON,
                         G_IMPLEMENT_INTERFACE (PHOSH_IDLE_DBUS_TYPE_OBJECT, phosh_idle_dbus_object_skeleton__phosh_idle_dbus_object_iface_init)
                         G_IMPLEMENT_INTERFACE (G_TYPE_DBUS_OBJECT, phosh_idle_dbus_object_skeleton__g_dbus_object_iface_init))

static void
phosh_idle_dbus_object_skeleton_init (PhoshIdleDBusObjectSkeleton *object G_GNUC_UNUSED)
{
}

static void
phosh_idle_dbus_object_skeleton_set_property (GObject      *gobject,
  guint         prop_id,
  const GValue *value,
  GParamSpec   *pspec)
{
  PhoshIdleDBusObjectSkeleton *object = PHOSH_IDLE_DBUS_OBJECT_SKELETON (gobject);
  GDBusInterfaceSkeleton *interface;

  switch (prop_id)
    {
    case 1:
      interface = g_value_get_object (value);
      if (interface != NULL)
        {
          g_warn_if_fail (PHOSH_IDLE_DBUS_IS_IDLE_MONITOR (interface));
          g_dbus_object_skeleton_add_interface (G_DBUS_OBJECT_SKELETON (object), interface);
        }
      else
        {
          g_dbus_object_skeleton_remove_interface_by_name (G_DBUS_OBJECT_SKELETON (object), "org.gnome.Mutter.IdleMonitor");
        }
      break;

    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
      break;
  }
}

static void
phosh_idle_dbus_object_skeleton_get_property (GObject      *gobject,
  guint         prop_id,
  GValue       *value,
  GParamSpec   *pspec)
{
  PhoshIdleDBusObjectSkeleton *object = PHOSH_IDLE_DBUS_OBJECT_SKELETON (gobject);
  GDBusInterface *interface;

  switch (prop_id)
    {
    case 1:
      interface = g_dbus_object_get_interface (G_DBUS_OBJECT (object), "org.gnome.Mutter.IdleMonitor");
      g_value_take_object (value, interface);
      break;

    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
      break;
  }
}

static void
phosh_idle_dbus_object_skeleton_class_init (PhoshIdleDBusObjectSkeletonClass *klass)
{
  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

  gobject_class->set_property = phosh_idle_dbus_object_skeleton_set_property;
  gobject_class->get_property = phosh_idle_dbus_object_skeleton_get_property;

  g_object_class_override_property (gobject_class, 1, "idle-monitor");
}

/**
 * phosh_idle_dbus_object_skeleton_new:
 * @object_path: An object path.
 *
 * Creates a new skeleton object.
 *
 * Returns: (transfer full): The skeleton object.
 */
PhoshIdleDBusObjectSkeleton *
phosh_idle_dbus_object_skeleton_new (const gchar *object_path)
{
  g_return_val_if_fail (g_variant_is_object_path (object_path), NULL);
  return PHOSH_IDLE_DBUS_OBJECT_SKELETON (g_object_new (PHOSH_IDLE_DBUS_TYPE_OBJECT_SKELETON, "g-object-path", object_path, NULL));
}

/**
 * phosh_idle_dbus_object_skeleton_set_idle_monitor:
 * @object: A #PhoshIdleDBusObjectSkeleton.
 * @interface_: (nullable): A #PhoshIdleDBusIdleMonitor or %NULL to clear the interface.
 *
 * Sets the #PhoshIdleDBusIdleMonitor instance for the D-Bus interface <link linkend="gdbus-interface-org-gnome-Mutter-IdleMonitor.top_of_page">org.gnome.Mutter.IdleMonitor</link> on @object.
 */
void phosh_idle_dbus_object_skeleton_set_idle_monitor (PhoshIdleDBusObjectSkeleton *object, PhoshIdleDBusIdleMonitor *interface_)
{
  g_object_set (G_OBJECT (object), "idle-monitor", interface_, NULL);
}


/* ------------------------------------------------------------------------
 * Code for ObjectManager client
 * ------------------------------------------------------------------------
 */

/**
 * SECTION:PhoshIdleDBusObjectManagerClient
 * @title: PhoshIdleDBusObjectManagerClient
 * @short_description: Generated GDBusObjectManagerClient type
 *
 * This section contains a #GDBusObjectManagerClient that uses phosh_idle_dbus_object_manager_client_get_proxy_type() as the #GDBusProxyTypeFunc.
 */

/**
 * PhoshIdleDBusObjectManagerClient:
 *
 * The #PhoshIdleDBusObjectManagerClient structure contains only private data and should only be accessed using the provided API.
 */

/**
 * PhoshIdleDBusObjectManagerClientClass:
 * @parent_class: The parent class.
 *
 * Class structure for #PhoshIdleDBusObjectManagerClient.
 */

G_DEFINE_TYPE (PhoshIdleDBusObjectManagerClient, phosh_idle_dbus_object_manager_client, G_TYPE_DBUS_OBJECT_MANAGER_CLIENT)

static void
phosh_idle_dbus_object_manager_client_init (PhoshIdleDBusObjectManagerClient *manager G_GNUC_UNUSED)
{
}

static void
phosh_idle_dbus_object_manager_client_class_init (PhoshIdleDBusObjectManagerClientClass *klass G_GNUC_UNUSED)
{
}

/**
 * phosh_idle_dbus_object_manager_client_get_proxy_type:
 * @manager: A #GDBusObjectManagerClient.
 * @object_path: The object path of the remote object (unused).
 * @interface_name: (nullable): Interface name of the remote object or %NULL to get the object proxy #GType.
 * @user_data: User data (unused).
 *
 * A #GDBusProxyTypeFunc that maps @interface_name to the generated #GDBusObjectProxy derived and #GDBusProxy derived types.
 *
 * Returns: A #GDBusProxy derived #GType if @interface_name is not %NULL, otherwise the #GType for #PhoshIdleDBusObjectProxy.
 */
GType
phosh_idle_dbus_object_manager_client_get_proxy_type (GDBusObjectManagerClient *manager G_GNUC_UNUSED, const gchar *object_path G_GNUC_UNUSED, const gchar *interface_name, gpointer user_data G_GNUC_UNUSED)
{
  static gsize once_init_value = 0;
  static GHashTable *lookup_hash;
  GType ret;

  if (interface_name == NULL)
    return PHOSH_IDLE_DBUS_TYPE_OBJECT_PROXY;
  if (g_once_init_enter (&once_init_value))
    {
      lookup_hash = g_hash_table_new (g_str_hash, g_str_equal);
      g_hash_table_insert (lookup_hash, (gpointer) "org.gnome.Mutter.IdleMonitor", (gpointer) (guintptr) (PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR_PROXY));
      g_once_init_leave (&once_init_value, 1);
    }
  ret = (GType) (guintptr) (g_hash_table_lookup (lookup_hash, interface_name));
  if (ret == (GType) 0)
    ret = G_TYPE_DBUS_PROXY;
  return ret;
}

/**
 * phosh_idle_dbus_object_manager_client_new:
 * @connection: A #GDBusConnection.
 * @flags: Flags from the #GDBusObjectManagerClientFlags enumeration.
 * @name: (nullable): A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously creates #GDBusObjectManagerClient using phosh_idle_dbus_object_manager_client_get_proxy_type() as the #GDBusProxyTypeFunc. See g_dbus_object_manager_client_new() for more details.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call phosh_idle_dbus_object_manager_client_new_finish() to get the result of the operation.
 *
 * See phosh_idle_dbus_object_manager_client_new_sync() for the synchronous, blocking version of this constructor.
 */
void
phosh_idle_dbus_object_manager_client_new (
    GDBusConnection        *connection,
    GDBusObjectManagerClientFlags  flags,
    const gchar            *name,
    const gchar            *object_path,
    GCancellable           *cancellable,
    GAsyncReadyCallback     callback,
    gpointer                user_data)
{
  g_async_initable_new_async (PHOSH_IDLE_DBUS_TYPE_OBJECT_MANAGER_CLIENT, G_PRIORITY_DEFAULT, cancellable, callback, user_data, "flags", flags, "name", name, "connection", connection, "object-path", object_path, "get-proxy-type-func", phosh_idle_dbus_object_manager_client_get_proxy_type, NULL);
}

/**
 * phosh_idle_dbus_object_manager_client_new_finish:
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to phosh_idle_dbus_object_manager_client_new().
 * @error: Return location for error or %NULL
 *
 * Finishes an operation started with phosh_idle_dbus_object_manager_client_new().
 *
 * Returns: (transfer full) (type PhoshIdleDBusObjectManagerClient): The constructed object manager client or %NULL if @error is set.
 */
GDBusObjectManager *
phosh_idle_dbus_object_manager_client_new_finish (
    GAsyncResult        *res,
    GError             **error)
{
  GObject *ret;
  GObject *source_object;
  source_object = g_async_result_get_source_object (res);
  ret = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), res, error);
  g_object_unref (source_object);
  if (ret != NULL)
    return G_DBUS_OBJECT_MANAGER (ret);
  else
    return NULL;
}

/**
 * phosh_idle_dbus_object_manager_client_new_sync:
 * @connection: A #GDBusConnection.
 * @flags: Flags from the #GDBusObjectManagerClientFlags enumeration.
 * @name: (nullable): A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL
 *
 * Synchronously creates #GDBusObjectManagerClient using phosh_idle_dbus_object_manager_client_get_proxy_type() as the #GDBusProxyTypeFunc. See g_dbus_object_manager_client_new_sync() for more details.
 *
 * The calling thread is blocked until a reply is received.
 *
 * See phosh_idle_dbus_object_manager_client_new() for the asynchronous version of this constructor.
 *
 * Returns: (transfer full) (type PhoshIdleDBusObjectManagerClient): The constructed object manager client or %NULL if @error is set.
 */
GDBusObjectManager *
phosh_idle_dbus_object_manager_client_new_sync (
    GDBusConnection        *connection,
    GDBusObjectManagerClientFlags  flags,
    const gchar            *name,
    const gchar            *object_path,
    GCancellable           *cancellable,
    GError                **error)
{
  GInitable *ret;
  ret = g_initable_new (PHOSH_IDLE_DBUS_TYPE_OBJECT_MANAGER_CLIENT, cancellable, error, "flags", flags, "name", name, "connection", connection, "object-path", object_path, "get-proxy-type-func", phosh_idle_dbus_object_manager_client_get_proxy_type, NULL);
  if (ret != NULL)
    return G_DBUS_OBJECT_MANAGER (ret);
  else
    return NULL;
}


/**
 * phosh_idle_dbus_object_manager_client_new_for_bus:
 * @bus_type: A #GBusType.
 * @flags: Flags from the #GDBusObjectManagerClientFlags enumeration.
 * @name: A bus name (well-known or unique).
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: User data to pass to @callback.
 *
 * Like phosh_idle_dbus_object_manager_client_new() but takes a #GBusType instead of a #GDBusConnection.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call phosh_idle_dbus_object_manager_client_new_for_bus_finish() to get the result of the operation.
 *
 * See phosh_idle_dbus_object_manager_client_new_for_bus_sync() for the synchronous, blocking version of this constructor.
 */
void
phosh_idle_dbus_object_manager_client_new_for_bus (
    GBusType                bus_type,
    GDBusObjectManagerClientFlags  flags,
    const gchar            *name,
    const gchar            *object_path,
    GCancellable           *cancellable,
    GAsyncReadyCallback     callback,
    gpointer                user_data)
{
  g_async_initable_new_async (PHOSH_IDLE_DBUS_TYPE_OBJECT_MANAGER_CLIENT, G_PRIORITY_DEFAULT, cancellable, callback, user_data, "flags", flags, "name", name, "bus-type", bus_type, "object-path", object_path, "get-proxy-type-func", phosh_idle_dbus_object_manager_client_get_proxy_type, NULL);
}

/**
 * phosh_idle_dbus_object_manager_client_new_for_bus_finish:
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to phosh_idle_dbus_object_manager_client_new_for_bus().
 * @error: Return location for error or %NULL
 *
 * Finishes an operation started with phosh_idle_dbus_object_manager_client_new_for_bus().
 *
 * Returns: (transfer full) (type PhoshIdleDBusObjectManagerClient): The constructed object manager client or %NULL if @error is set.
 */
GDBusObjectManager *
phosh_idle_dbus_object_manager_client_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error)
{
  GObject *ret;
  GObject *source_object;
  source_object = g_async_result_get_source_object (res);
  ret = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), res, error);
  g_object_unref (source_object);
  if (ret != NULL)
    return G_DBUS_OBJECT_MANAGER (ret);
  else
    return NULL;
}

/**
 * phosh_idle_dbus_object_manager_client_new_for_bus_sync:
 * @bus_type: A #GBusType.
 * @flags: Flags from the #GDBusObjectManagerClientFlags enumeration.
 * @name: A bus name (well-known or unique).
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL
 *
 * Like phosh_idle_dbus_object_manager_client_new_sync() but takes a #GBusType instead of a #GDBusConnection.
 *
 * The calling thread is blocked until a reply is received.
 *
 * See phosh_idle_dbus_object_manager_client_new_for_bus() for the asynchronous version of this constructor.
 *
 * Returns: (transfer full) (type PhoshIdleDBusObjectManagerClient): The constructed object manager client or %NULL if @error is set.
 */
GDBusObjectManager *
phosh_idle_dbus_object_manager_client_new_for_bus_sync (
    GBusType                bus_type,
    GDBusObjectManagerClientFlags  flags,
    const gchar            *name,
    const gchar            *object_path,
    GCancellable           *cancellable,
    GError                **error)
{
  GInitable *ret;
  ret = g_initable_new (PHOSH_IDLE_DBUS_TYPE_OBJECT_MANAGER_CLIENT, cancellable, error, "flags", flags, "name", name, "bus-type", bus_type, "object-path", object_path, "get-proxy-type-func", phosh_idle_dbus_object_manager_client_get_proxy_type, NULL);
  if (ret != NULL)
    return G_DBUS_OBJECT_MANAGER (ret);
  else
    return NULL;
}


