/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __GSD_RFKILL_DBUS_H__
#define __GSD_RFKILL_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.SettingsDaemon.Rfkill */

#define PHOSH_DBUS_TYPE_RFKILL (phosh_dbus_rfkill_get_type ())
#define PHOSH_DBUS_RFKILL(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_RFKILL, PhoshDBusRfkill))
#define PHOSH_DBUS_IS_RFKILL(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_RFKILL))
#define PHOSH_DBUS_RFKILL_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_RFKILL, PhoshDBusRfkillIface))

struct _PhoshDBusRfkill;
typedef struct _PhoshDBusRfkill PhoshDBusRfkill;
typedef struct _PhoshDBusRfkillIface PhoshDBusRfkillIface;

struct _PhoshDBusRfkillIface
{
  GTypeInterface parent_iface;

  gboolean  (*get_airplane_mode) (PhoshDBusRfkill *object);

  gboolean  (*get_bluetooth_airplane_mode) (PhoshDBusRfkill *object);

  gboolean  (*get_bluetooth_hardware_airplane_mode) (PhoshDBusRfkill *object);

  gboolean  (*get_bluetooth_has_airplane_mode) (PhoshDBusRfkill *object);

  gboolean  (*get_hardware_airplane_mode) (PhoshDBusRfkill *object);

  gboolean  (*get_should_show_airplane_mode) (PhoshDBusRfkill *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusRfkill, g_object_unref)
#endif

GType phosh_dbus_rfkill_get_type (void);

GDBusInterfaceInfo *phosh_dbus_rfkill_interface_info (void);
guint phosh_dbus_rfkill_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus property accessors: */
gboolean phosh_dbus_rfkill_get_airplane_mode (PhoshDBusRfkill *object);
void phosh_dbus_rfkill_set_airplane_mode (PhoshDBusRfkill *object, gboolean value);

gboolean phosh_dbus_rfkill_get_hardware_airplane_mode (PhoshDBusRfkill *object);
void phosh_dbus_rfkill_set_hardware_airplane_mode (PhoshDBusRfkill *object, gboolean value);

gboolean phosh_dbus_rfkill_get_bluetooth_airplane_mode (PhoshDBusRfkill *object);
void phosh_dbus_rfkill_set_bluetooth_airplane_mode (PhoshDBusRfkill *object, gboolean value);

gboolean phosh_dbus_rfkill_get_bluetooth_has_airplane_mode (PhoshDBusRfkill *object);
void phosh_dbus_rfkill_set_bluetooth_has_airplane_mode (PhoshDBusRfkill *object, gboolean value);

gboolean phosh_dbus_rfkill_get_bluetooth_hardware_airplane_mode (PhoshDBusRfkill *object);
void phosh_dbus_rfkill_set_bluetooth_hardware_airplane_mode (PhoshDBusRfkill *object, gboolean value);

gboolean phosh_dbus_rfkill_get_should_show_airplane_mode (PhoshDBusRfkill *object);
void phosh_dbus_rfkill_set_should_show_airplane_mode (PhoshDBusRfkill *object, gboolean value);


/* ---- */

#define PHOSH_DBUS_TYPE_RFKILL_PROXY (phosh_dbus_rfkill_proxy_get_type ())
#define PHOSH_DBUS_RFKILL_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_RFKILL_PROXY, PhoshDBusRfkillProxy))
#define PHOSH_DBUS_RFKILL_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_RFKILL_PROXY, PhoshDBusRfkillProxyClass))
#define PHOSH_DBUS_RFKILL_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_RFKILL_PROXY, PhoshDBusRfkillProxyClass))
#define PHOSH_DBUS_IS_RFKILL_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_RFKILL_PROXY))
#define PHOSH_DBUS_IS_RFKILL_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_RFKILL_PROXY))

typedef struct _PhoshDBusRfkillProxy PhoshDBusRfkillProxy;
typedef struct _PhoshDBusRfkillProxyClass PhoshDBusRfkillProxyClass;
typedef struct _PhoshDBusRfkillProxyPrivate PhoshDBusRfkillProxyPrivate;

struct _PhoshDBusRfkillProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusRfkillProxyPrivate *priv;
};

struct _PhoshDBusRfkillProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_rfkill_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusRfkillProxy, g_object_unref)
#endif

void phosh_dbus_rfkill_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusRfkill *phosh_dbus_rfkill_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusRfkill *phosh_dbus_rfkill_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_rfkill_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusRfkill *phosh_dbus_rfkill_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusRfkill *phosh_dbus_rfkill_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_RFKILL_SKELETON (phosh_dbus_rfkill_skeleton_get_type ())
#define PHOSH_DBUS_RFKILL_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_RFKILL_SKELETON, PhoshDBusRfkillSkeleton))
#define PHOSH_DBUS_RFKILL_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_RFKILL_SKELETON, PhoshDBusRfkillSkeletonClass))
#define PHOSH_DBUS_RFKILL_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_RFKILL_SKELETON, PhoshDBusRfkillSkeletonClass))
#define PHOSH_DBUS_IS_RFKILL_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_RFKILL_SKELETON))
#define PHOSH_DBUS_IS_RFKILL_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_RFKILL_SKELETON))

typedef struct _PhoshDBusRfkillSkeleton PhoshDBusRfkillSkeleton;
typedef struct _PhoshDBusRfkillSkeletonClass PhoshDBusRfkillSkeletonClass;
typedef struct _PhoshDBusRfkillSkeletonPrivate PhoshDBusRfkillSkeletonPrivate;

struct _PhoshDBusRfkillSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusRfkillSkeletonPrivate *priv;
};

struct _PhoshDBusRfkillSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_rfkill_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusRfkillSkeleton, g_object_unref)
#endif

PhoshDBusRfkill *phosh_dbus_rfkill_skeleton_new (void);


G_END_DECLS

#endif /* __GSD_RFKILL_DBUS_H__ */
