/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __PHOSH_WWAN_OFONO_DBUS_H__
#define __PHOSH_WWAN_OFONO_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.ofono.SimManager */

#define PHOSH_DBUS_TYPE_OFONO_SIM_MANAGER (phosh_dbus_ofono_sim_manager_get_type ())
#define PHOSH_DBUS_OFONO_SIM_MANAGER(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_OFONO_SIM_MANAGER, PhoshDBusOfonoSimManager))
#define PHOSH_DBUS_IS_OFONO_SIM_MANAGER(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_OFONO_SIM_MANAGER))
#define PHOSH_DBUS_OFONO_SIM_MANAGER_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_OFONO_SIM_MANAGER, PhoshDBusOfonoSimManagerIface))

struct _PhoshDBusOfonoSimManager;
typedef struct _PhoshDBusOfonoSimManager PhoshDBusOfonoSimManager;
typedef struct _PhoshDBusOfonoSimManagerIface PhoshDBusOfonoSimManagerIface;

struct _PhoshDBusOfonoSimManagerIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_get_properties) (
    PhoshDBusOfonoSimManager *object,
    GDBusMethodInvocation *invocation);

  void (*property_changed) (
    PhoshDBusOfonoSimManager *object,
    const gchar *arg_unnamed_arg0,
    GVariant *arg_unnamed_arg1);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusOfonoSimManager, g_object_unref)
#endif

GType phosh_dbus_ofono_sim_manager_get_type (void);

GDBusInterfaceInfo *phosh_dbus_ofono_sim_manager_interface_info (void);
guint phosh_dbus_ofono_sim_manager_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void phosh_dbus_ofono_sim_manager_complete_get_properties (
    PhoshDBusOfonoSimManager *object,
    GDBusMethodInvocation *invocation,
    GVariant *properties);



/* D-Bus signal emissions functions: */
void phosh_dbus_ofono_sim_manager_emit_property_changed (
    PhoshDBusOfonoSimManager *object,
    const gchar *arg_unnamed_arg0,
    GVariant *arg_unnamed_arg1);



/* D-Bus method calls: */
void phosh_dbus_ofono_sim_manager_call_get_properties (
    PhoshDBusOfonoSimManager *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_ofono_sim_manager_call_get_properties_finish (
    PhoshDBusOfonoSimManager *proxy,
    GVariant ** out_properties,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_ofono_sim_manager_call_get_properties_sync (
    PhoshDBusOfonoSimManager *proxy,
    GVariant ** out_properties,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define PHOSH_DBUS_TYPE_OFONO_SIM_MANAGER_PROXY (phosh_dbus_ofono_sim_manager_proxy_get_type ())
#define PHOSH_DBUS_OFONO_SIM_MANAGER_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_OFONO_SIM_MANAGER_PROXY, PhoshDBusOfonoSimManagerProxy))
#define PHOSH_DBUS_OFONO_SIM_MANAGER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_OFONO_SIM_MANAGER_PROXY, PhoshDBusOfonoSimManagerProxyClass))
#define PHOSH_DBUS_OFONO_SIM_MANAGER_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_OFONO_SIM_MANAGER_PROXY, PhoshDBusOfonoSimManagerProxyClass))
#define PHOSH_DBUS_IS_OFONO_SIM_MANAGER_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_OFONO_SIM_MANAGER_PROXY))
#define PHOSH_DBUS_IS_OFONO_SIM_MANAGER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_OFONO_SIM_MANAGER_PROXY))

typedef struct _PhoshDBusOfonoSimManagerProxy PhoshDBusOfonoSimManagerProxy;
typedef struct _PhoshDBusOfonoSimManagerProxyClass PhoshDBusOfonoSimManagerProxyClass;
typedef struct _PhoshDBusOfonoSimManagerProxyPrivate PhoshDBusOfonoSimManagerProxyPrivate;

struct _PhoshDBusOfonoSimManagerProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusOfonoSimManagerProxyPrivate *priv;
};

struct _PhoshDBusOfonoSimManagerProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_ofono_sim_manager_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusOfonoSimManagerProxy, g_object_unref)
#endif

void phosh_dbus_ofono_sim_manager_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusOfonoSimManager *phosh_dbus_ofono_sim_manager_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusOfonoSimManager *phosh_dbus_ofono_sim_manager_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_ofono_sim_manager_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusOfonoSimManager *phosh_dbus_ofono_sim_manager_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusOfonoSimManager *phosh_dbus_ofono_sim_manager_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_OFONO_SIM_MANAGER_SKELETON (phosh_dbus_ofono_sim_manager_skeleton_get_type ())
#define PHOSH_DBUS_OFONO_SIM_MANAGER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_OFONO_SIM_MANAGER_SKELETON, PhoshDBusOfonoSimManagerSkeleton))
#define PHOSH_DBUS_OFONO_SIM_MANAGER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_OFONO_SIM_MANAGER_SKELETON, PhoshDBusOfonoSimManagerSkeletonClass))
#define PHOSH_DBUS_OFONO_SIM_MANAGER_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_OFONO_SIM_MANAGER_SKELETON, PhoshDBusOfonoSimManagerSkeletonClass))
#define PHOSH_DBUS_IS_OFONO_SIM_MANAGER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_OFONO_SIM_MANAGER_SKELETON))
#define PHOSH_DBUS_IS_OFONO_SIM_MANAGER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_OFONO_SIM_MANAGER_SKELETON))

typedef struct _PhoshDBusOfonoSimManagerSkeleton PhoshDBusOfonoSimManagerSkeleton;
typedef struct _PhoshDBusOfonoSimManagerSkeletonClass PhoshDBusOfonoSimManagerSkeletonClass;
typedef struct _PhoshDBusOfonoSimManagerSkeletonPrivate PhoshDBusOfonoSimManagerSkeletonPrivate;

struct _PhoshDBusOfonoSimManagerSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusOfonoSimManagerSkeletonPrivate *priv;
};

struct _PhoshDBusOfonoSimManagerSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_ofono_sim_manager_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusOfonoSimManagerSkeleton, g_object_unref)
#endif

PhoshDBusOfonoSimManager *phosh_dbus_ofono_sim_manager_skeleton_new (void);


/* ------------------------------------------------------------------------ */
/* Declarations for org.ofono.NetworkRegistration */

#define PHOSH_DBUS_TYPE_OFONO_NETWORK_REGISTRATION (phosh_dbus_ofono_network_registration_get_type ())
#define PHOSH_DBUS_OFONO_NETWORK_REGISTRATION(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_OFONO_NETWORK_REGISTRATION, PhoshDBusOfonoNetworkRegistration))
#define PHOSH_DBUS_IS_OFONO_NETWORK_REGISTRATION(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_OFONO_NETWORK_REGISTRATION))
#define PHOSH_DBUS_OFONO_NETWORK_REGISTRATION_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_OFONO_NETWORK_REGISTRATION, PhoshDBusOfonoNetworkRegistrationIface))

struct _PhoshDBusOfonoNetworkRegistration;
typedef struct _PhoshDBusOfonoNetworkRegistration PhoshDBusOfonoNetworkRegistration;
typedef struct _PhoshDBusOfonoNetworkRegistrationIface PhoshDBusOfonoNetworkRegistrationIface;

struct _PhoshDBusOfonoNetworkRegistrationIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_get_properties) (
    PhoshDBusOfonoNetworkRegistration *object,
    GDBusMethodInvocation *invocation);

  void (*property_changed) (
    PhoshDBusOfonoNetworkRegistration *object,
    const gchar *arg_unnamed_arg0,
    GVariant *arg_unnamed_arg1);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusOfonoNetworkRegistration, g_object_unref)
#endif

GType phosh_dbus_ofono_network_registration_get_type (void);

GDBusInterfaceInfo *phosh_dbus_ofono_network_registration_interface_info (void);
guint phosh_dbus_ofono_network_registration_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void phosh_dbus_ofono_network_registration_complete_get_properties (
    PhoshDBusOfonoNetworkRegistration *object,
    GDBusMethodInvocation *invocation,
    GVariant *properties);



/* D-Bus signal emissions functions: */
void phosh_dbus_ofono_network_registration_emit_property_changed (
    PhoshDBusOfonoNetworkRegistration *object,
    const gchar *arg_unnamed_arg0,
    GVariant *arg_unnamed_arg1);



/* D-Bus method calls: */
void phosh_dbus_ofono_network_registration_call_get_properties (
    PhoshDBusOfonoNetworkRegistration *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_ofono_network_registration_call_get_properties_finish (
    PhoshDBusOfonoNetworkRegistration *proxy,
    GVariant ** out_properties,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_ofono_network_registration_call_get_properties_sync (
    PhoshDBusOfonoNetworkRegistration *proxy,
    GVariant ** out_properties,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define PHOSH_DBUS_TYPE_OFONO_NETWORK_REGISTRATION_PROXY (phosh_dbus_ofono_network_registration_proxy_get_type ())
#define PHOSH_DBUS_OFONO_NETWORK_REGISTRATION_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_OFONO_NETWORK_REGISTRATION_PROXY, PhoshDBusOfonoNetworkRegistrationProxy))
#define PHOSH_DBUS_OFONO_NETWORK_REGISTRATION_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_OFONO_NETWORK_REGISTRATION_PROXY, PhoshDBusOfonoNetworkRegistrationProxyClass))
#define PHOSH_DBUS_OFONO_NETWORK_REGISTRATION_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_OFONO_NETWORK_REGISTRATION_PROXY, PhoshDBusOfonoNetworkRegistrationProxyClass))
#define PHOSH_DBUS_IS_OFONO_NETWORK_REGISTRATION_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_OFONO_NETWORK_REGISTRATION_PROXY))
#define PHOSH_DBUS_IS_OFONO_NETWORK_REGISTRATION_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_OFONO_NETWORK_REGISTRATION_PROXY))

typedef struct _PhoshDBusOfonoNetworkRegistrationProxy PhoshDBusOfonoNetworkRegistrationProxy;
typedef struct _PhoshDBusOfonoNetworkRegistrationProxyClass PhoshDBusOfonoNetworkRegistrationProxyClass;
typedef struct _PhoshDBusOfonoNetworkRegistrationProxyPrivate PhoshDBusOfonoNetworkRegistrationProxyPrivate;

struct _PhoshDBusOfonoNetworkRegistrationProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusOfonoNetworkRegistrationProxyPrivate *priv;
};

struct _PhoshDBusOfonoNetworkRegistrationProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_ofono_network_registration_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusOfonoNetworkRegistrationProxy, g_object_unref)
#endif

void phosh_dbus_ofono_network_registration_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusOfonoNetworkRegistration *phosh_dbus_ofono_network_registration_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusOfonoNetworkRegistration *phosh_dbus_ofono_network_registration_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_ofono_network_registration_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusOfonoNetworkRegistration *phosh_dbus_ofono_network_registration_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusOfonoNetworkRegistration *phosh_dbus_ofono_network_registration_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_OFONO_NETWORK_REGISTRATION_SKELETON (phosh_dbus_ofono_network_registration_skeleton_get_type ())
#define PHOSH_DBUS_OFONO_NETWORK_REGISTRATION_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_OFONO_NETWORK_REGISTRATION_SKELETON, PhoshDBusOfonoNetworkRegistrationSkeleton))
#define PHOSH_DBUS_OFONO_NETWORK_REGISTRATION_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_OFONO_NETWORK_REGISTRATION_SKELETON, PhoshDBusOfonoNetworkRegistrationSkeletonClass))
#define PHOSH_DBUS_OFONO_NETWORK_REGISTRATION_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_OFONO_NETWORK_REGISTRATION_SKELETON, PhoshDBusOfonoNetworkRegistrationSkeletonClass))
#define PHOSH_DBUS_IS_OFONO_NETWORK_REGISTRATION_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_OFONO_NETWORK_REGISTRATION_SKELETON))
#define PHOSH_DBUS_IS_OFONO_NETWORK_REGISTRATION_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_OFONO_NETWORK_REGISTRATION_SKELETON))

typedef struct _PhoshDBusOfonoNetworkRegistrationSkeleton PhoshDBusOfonoNetworkRegistrationSkeleton;
typedef struct _PhoshDBusOfonoNetworkRegistrationSkeletonClass PhoshDBusOfonoNetworkRegistrationSkeletonClass;
typedef struct _PhoshDBusOfonoNetworkRegistrationSkeletonPrivate PhoshDBusOfonoNetworkRegistrationSkeletonPrivate;

struct _PhoshDBusOfonoNetworkRegistrationSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusOfonoNetworkRegistrationSkeletonPrivate *priv;
};

struct _PhoshDBusOfonoNetworkRegistrationSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_ofono_network_registration_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusOfonoNetworkRegistrationSkeleton, g_object_unref)
#endif

PhoshDBusOfonoNetworkRegistration *phosh_dbus_ofono_network_registration_skeleton_new (void);


/* ------------------------------------------------------------------------ */
/* Declarations for org.ofono.Manager */

#define PHOSH_DBUS_TYPE_OFONO_MANAGER (phosh_dbus_ofono_manager_get_type ())
#define PHOSH_DBUS_OFONO_MANAGER(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_OFONO_MANAGER, PhoshDBusOfonoManager))
#define PHOSH_DBUS_IS_OFONO_MANAGER(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_OFONO_MANAGER))
#define PHOSH_DBUS_OFONO_MANAGER_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_OFONO_MANAGER, PhoshDBusOfonoManagerIface))

struct _PhoshDBusOfonoManager;
typedef struct _PhoshDBusOfonoManager PhoshDBusOfonoManager;
typedef struct _PhoshDBusOfonoManagerIface PhoshDBusOfonoManagerIface;

struct _PhoshDBusOfonoManagerIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_get_modems) (
    PhoshDBusOfonoManager *object,
    GDBusMethodInvocation *invocation);

  void (*modem_added) (
    PhoshDBusOfonoManager *object,
    const gchar *arg_unnamed_arg0,
    GVariant *arg_unnamed_arg1);

  void (*modem_removed) (
    PhoshDBusOfonoManager *object,
    const gchar *arg_unnamed_arg0);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusOfonoManager, g_object_unref)
#endif

GType phosh_dbus_ofono_manager_get_type (void);

GDBusInterfaceInfo *phosh_dbus_ofono_manager_interface_info (void);
guint phosh_dbus_ofono_manager_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void phosh_dbus_ofono_manager_complete_get_modems (
    PhoshDBusOfonoManager *object,
    GDBusMethodInvocation *invocation,
    GVariant *unnamed_arg0);



/* D-Bus signal emissions functions: */
void phosh_dbus_ofono_manager_emit_modem_added (
    PhoshDBusOfonoManager *object,
    const gchar *arg_unnamed_arg0,
    GVariant *arg_unnamed_arg1);

void phosh_dbus_ofono_manager_emit_modem_removed (
    PhoshDBusOfonoManager *object,
    const gchar *arg_unnamed_arg0);



/* D-Bus method calls: */
void phosh_dbus_ofono_manager_call_get_modems (
    PhoshDBusOfonoManager *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_ofono_manager_call_get_modems_finish (
    PhoshDBusOfonoManager *proxy,
    GVariant ** out_unnamed_arg0,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_ofono_manager_call_get_modems_sync (
    PhoshDBusOfonoManager *proxy,
    GVariant ** out_unnamed_arg0,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define PHOSH_DBUS_TYPE_OFONO_MANAGER_PROXY (phosh_dbus_ofono_manager_proxy_get_type ())
#define PHOSH_DBUS_OFONO_MANAGER_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_OFONO_MANAGER_PROXY, PhoshDBusOfonoManagerProxy))
#define PHOSH_DBUS_OFONO_MANAGER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_OFONO_MANAGER_PROXY, PhoshDBusOfonoManagerProxyClass))
#define PHOSH_DBUS_OFONO_MANAGER_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_OFONO_MANAGER_PROXY, PhoshDBusOfonoManagerProxyClass))
#define PHOSH_DBUS_IS_OFONO_MANAGER_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_OFONO_MANAGER_PROXY))
#define PHOSH_DBUS_IS_OFONO_MANAGER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_OFONO_MANAGER_PROXY))

typedef struct _PhoshDBusOfonoManagerProxy PhoshDBusOfonoManagerProxy;
typedef struct _PhoshDBusOfonoManagerProxyClass PhoshDBusOfonoManagerProxyClass;
typedef struct _PhoshDBusOfonoManagerProxyPrivate PhoshDBusOfonoManagerProxyPrivate;

struct _PhoshDBusOfonoManagerProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusOfonoManagerProxyPrivate *priv;
};

struct _PhoshDBusOfonoManagerProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_ofono_manager_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusOfonoManagerProxy, g_object_unref)
#endif

void phosh_dbus_ofono_manager_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusOfonoManager *phosh_dbus_ofono_manager_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusOfonoManager *phosh_dbus_ofono_manager_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_ofono_manager_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusOfonoManager *phosh_dbus_ofono_manager_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusOfonoManager *phosh_dbus_ofono_manager_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_OFONO_MANAGER_SKELETON (phosh_dbus_ofono_manager_skeleton_get_type ())
#define PHOSH_DBUS_OFONO_MANAGER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_OFONO_MANAGER_SKELETON, PhoshDBusOfonoManagerSkeleton))
#define PHOSH_DBUS_OFONO_MANAGER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_OFONO_MANAGER_SKELETON, PhoshDBusOfonoManagerSkeletonClass))
#define PHOSH_DBUS_OFONO_MANAGER_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_OFONO_MANAGER_SKELETON, PhoshDBusOfonoManagerSkeletonClass))
#define PHOSH_DBUS_IS_OFONO_MANAGER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_OFONO_MANAGER_SKELETON))
#define PHOSH_DBUS_IS_OFONO_MANAGER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_OFONO_MANAGER_SKELETON))

typedef struct _PhoshDBusOfonoManagerSkeleton PhoshDBusOfonoManagerSkeleton;
typedef struct _PhoshDBusOfonoManagerSkeletonClass PhoshDBusOfonoManagerSkeletonClass;
typedef struct _PhoshDBusOfonoManagerSkeletonPrivate PhoshDBusOfonoManagerSkeletonPrivate;

struct _PhoshDBusOfonoManagerSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusOfonoManagerSkeletonPrivate *priv;
};

struct _PhoshDBusOfonoManagerSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_ofono_manager_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusOfonoManagerSkeleton, g_object_unref)
#endif

PhoshDBusOfonoManager *phosh_dbus_ofono_manager_skeleton_new (void);


G_END_DECLS

#endif /* __PHOSH_WWAN_OFONO_DBUS_H__ */
