/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __GNOME_SESSION_PRESENCE_DBUS_H__
#define __GNOME_SESSION_PRESENCE_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.SessionManager.Presence */

#define PHOSH_DBUS_TYPE_SESSION_MANAGER_PRESENCE (phosh_dbus_session_manager_presence_get_type ())
#define PHOSH_DBUS_SESSION_MANAGER_PRESENCE(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_SESSION_MANAGER_PRESENCE, PhoshDBusSessionManagerPresence))
#define PHOSH_DBUS_IS_SESSION_MANAGER_PRESENCE(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_SESSION_MANAGER_PRESENCE))
#define PHOSH_DBUS_SESSION_MANAGER_PRESENCE_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_SESSION_MANAGER_PRESENCE, PhoshDBusSessionManagerPresenceIface))

struct _PhoshDBusSessionManagerPresence;
typedef struct _PhoshDBusSessionManagerPresence PhoshDBusSessionManagerPresence;
typedef struct _PhoshDBusSessionManagerPresenceIface PhoshDBusSessionManagerPresenceIface;

struct _PhoshDBusSessionManagerPresenceIface
{
  GTypeInterface parent_iface;


  guint  (*get_status) (PhoshDBusSessionManagerPresence *object);

  void (*status_changed) (
    PhoshDBusSessionManagerPresence *object,
    guint arg_status);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusSessionManagerPresence, g_object_unref)
#endif

GType phosh_dbus_session_manager_presence_get_type (void);

GDBusInterfaceInfo *phosh_dbus_session_manager_presence_interface_info (void);
guint phosh_dbus_session_manager_presence_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus signal emissions functions: */
void phosh_dbus_session_manager_presence_emit_status_changed (
    PhoshDBusSessionManagerPresence *object,
    guint arg_status);



/* D-Bus property accessors: */
guint phosh_dbus_session_manager_presence_get_status (PhoshDBusSessionManagerPresence *object);
void phosh_dbus_session_manager_presence_set_status (PhoshDBusSessionManagerPresence *object, guint value);


/* ---- */

#define PHOSH_DBUS_TYPE_SESSION_MANAGER_PRESENCE_PROXY (phosh_dbus_session_manager_presence_proxy_get_type ())
#define PHOSH_DBUS_SESSION_MANAGER_PRESENCE_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_SESSION_MANAGER_PRESENCE_PROXY, PhoshDBusSessionManagerPresenceProxy))
#define PHOSH_DBUS_SESSION_MANAGER_PRESENCE_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_SESSION_MANAGER_PRESENCE_PROXY, PhoshDBusSessionManagerPresenceProxyClass))
#define PHOSH_DBUS_SESSION_MANAGER_PRESENCE_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_SESSION_MANAGER_PRESENCE_PROXY, PhoshDBusSessionManagerPresenceProxyClass))
#define PHOSH_DBUS_IS_SESSION_MANAGER_PRESENCE_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_SESSION_MANAGER_PRESENCE_PROXY))
#define PHOSH_DBUS_IS_SESSION_MANAGER_PRESENCE_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_SESSION_MANAGER_PRESENCE_PROXY))

typedef struct _PhoshDBusSessionManagerPresenceProxy PhoshDBusSessionManagerPresenceProxy;
typedef struct _PhoshDBusSessionManagerPresenceProxyClass PhoshDBusSessionManagerPresenceProxyClass;
typedef struct _PhoshDBusSessionManagerPresenceProxyPrivate PhoshDBusSessionManagerPresenceProxyPrivate;

struct _PhoshDBusSessionManagerPresenceProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusSessionManagerPresenceProxyPrivate *priv;
};

struct _PhoshDBusSessionManagerPresenceProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_session_manager_presence_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusSessionManagerPresenceProxy, g_object_unref)
#endif

void phosh_dbus_session_manager_presence_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusSessionManagerPresence *phosh_dbus_session_manager_presence_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusSessionManagerPresence *phosh_dbus_session_manager_presence_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_session_manager_presence_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusSessionManagerPresence *phosh_dbus_session_manager_presence_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusSessionManagerPresence *phosh_dbus_session_manager_presence_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_SESSION_MANAGER_PRESENCE_SKELETON (phosh_dbus_session_manager_presence_skeleton_get_type ())
#define PHOSH_DBUS_SESSION_MANAGER_PRESENCE_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_SESSION_MANAGER_PRESENCE_SKELETON, PhoshDBusSessionManagerPresenceSkeleton))
#define PHOSH_DBUS_SESSION_MANAGER_PRESENCE_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_SESSION_MANAGER_PRESENCE_SKELETON, PhoshDBusSessionManagerPresenceSkeletonClass))
#define PHOSH_DBUS_SESSION_MANAGER_PRESENCE_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_SESSION_MANAGER_PRESENCE_SKELETON, PhoshDBusSessionManagerPresenceSkeletonClass))
#define PHOSH_DBUS_IS_SESSION_MANAGER_PRESENCE_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_SESSION_MANAGER_PRESENCE_SKELETON))
#define PHOSH_DBUS_IS_SESSION_MANAGER_PRESENCE_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_SESSION_MANAGER_PRESENCE_SKELETON))

typedef struct _PhoshDBusSessionManagerPresenceSkeleton PhoshDBusSessionManagerPresenceSkeleton;
typedef struct _PhoshDBusSessionManagerPresenceSkeletonClass PhoshDBusSessionManagerPresenceSkeletonClass;
typedef struct _PhoshDBusSessionManagerPresenceSkeletonPrivate PhoshDBusSessionManagerPresenceSkeletonPrivate;

struct _PhoshDBusSessionManagerPresenceSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusSessionManagerPresenceSkeletonPrivate *priv;
};

struct _PhoshDBusSessionManagerPresenceSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_session_manager_presence_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusSessionManagerPresenceSkeleton, g_object_unref)
#endif

PhoshDBusSessionManagerPresence *phosh_dbus_session_manager_presence_skeleton_new (void);


G_END_DECLS

#endif /* __GNOME_SESSION_PRESENCE_DBUS_H__ */
