/*
 * Copyright (C) 2025 Phosh.mobi e.V.
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 * Author: Guido Günther <agx@sigxcpu.org>
 */

#pragma once

#include "thumbnail.h"

G_BEGIN_DECLS

void phosh_thumbnail_set_ready (PhoshThumbnail *self, gboolean ready);

G_END_DECLS
