/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __GNOME_SESSION_DBUS_H__
#define __GNOME_SESSION_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.SessionManager */

#define PHOSH_DBUS_TYPE_SESSION_MANAGER (phosh_dbus_session_manager_get_type ())
#define PHOSH_DBUS_SESSION_MANAGER(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_SESSION_MANAGER, PhoshDBusSessionManager))
#define PHOSH_DBUS_IS_SESSION_MANAGER(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_SESSION_MANAGER))
#define PHOSH_DBUS_SESSION_MANAGER_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_SESSION_MANAGER, PhoshDBusSessionManagerIface))

struct _PhoshDBusSessionManager;
typedef struct _PhoshDBusSessionManager PhoshDBusSessionManager;
typedef struct _PhoshDBusSessionManagerIface PhoshDBusSessionManagerIface;

struct _PhoshDBusSessionManagerIface
{
  GTypeInterface parent_iface;



  gboolean (*handle_can_shutdown) (
    PhoshDBusSessionManager *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_inhibit) (
    PhoshDBusSessionManager *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_app_id,
    guint arg_toplevel_xid,
    const gchar *arg_reason,
    guint arg_flags);

  gboolean (*handle_is_inhibited) (
    PhoshDBusSessionManager *object,
    GDBusMethodInvocation *invocation,
    guint arg_unnamed_arg0);

  gboolean (*handle_logout) (
    PhoshDBusSessionManager *object,
    GDBusMethodInvocation *invocation,
    guint arg_mode);

  gboolean (*handle_reboot) (
    PhoshDBusSessionManager *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_register_client) (
    PhoshDBusSessionManager *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_app_id,
    const gchar *arg_client_startup_id);

  gboolean (*handle_shutdown) (
    PhoshDBusSessionManager *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_uninhibit) (
    PhoshDBusSessionManager *object,
    GDBusMethodInvocation *invocation,
    guint arg_inhibit_cookie);

  gboolean  (*get_session_is_active) (PhoshDBusSessionManager *object);

  void (*inhibitor_added) (
    PhoshDBusSessionManager *object,
    const gchar *arg_unnamed_arg0);

  void (*inhibitor_removed) (
    PhoshDBusSessionManager *object,
    const gchar *arg_unnamed_arg0);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusSessionManager, g_object_unref)
#endif

GType phosh_dbus_session_manager_get_type (void);

GDBusInterfaceInfo *phosh_dbus_session_manager_interface_info (void);
guint phosh_dbus_session_manager_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void phosh_dbus_session_manager_complete_logout (
    PhoshDBusSessionManager *object,
    GDBusMethodInvocation *invocation);

void phosh_dbus_session_manager_complete_shutdown (
    PhoshDBusSessionManager *object,
    GDBusMethodInvocation *invocation);

void phosh_dbus_session_manager_complete_reboot (
    PhoshDBusSessionManager *object,
    GDBusMethodInvocation *invocation);

void phosh_dbus_session_manager_complete_inhibit (
    PhoshDBusSessionManager *object,
    GDBusMethodInvocation *invocation,
    guint inhibit_cookie);

void phosh_dbus_session_manager_complete_uninhibit (
    PhoshDBusSessionManager *object,
    GDBusMethodInvocation *invocation);

void phosh_dbus_session_manager_complete_can_shutdown (
    PhoshDBusSessionManager *object,
    GDBusMethodInvocation *invocation,
    gboolean unnamed_arg0);

void phosh_dbus_session_manager_complete_is_inhibited (
    PhoshDBusSessionManager *object,
    GDBusMethodInvocation *invocation,
    gboolean unnamed_arg1);

void phosh_dbus_session_manager_complete_register_client (
    PhoshDBusSessionManager *object,
    GDBusMethodInvocation *invocation,
    const gchar *client_id);



/* D-Bus signal emissions functions: */
void phosh_dbus_session_manager_emit_inhibitor_added (
    PhoshDBusSessionManager *object,
    const gchar *arg_unnamed_arg0);

void phosh_dbus_session_manager_emit_inhibitor_removed (
    PhoshDBusSessionManager *object,
    const gchar *arg_unnamed_arg0);



/* D-Bus method calls: */
void phosh_dbus_session_manager_call_logout (
    PhoshDBusSessionManager *proxy,
    guint arg_mode,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_session_manager_call_logout_finish (
    PhoshDBusSessionManager *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_session_manager_call_logout_sync (
    PhoshDBusSessionManager *proxy,
    guint arg_mode,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_session_manager_call_shutdown (
    PhoshDBusSessionManager *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_session_manager_call_shutdown_finish (
    PhoshDBusSessionManager *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_session_manager_call_shutdown_sync (
    PhoshDBusSessionManager *proxy,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_session_manager_call_reboot (
    PhoshDBusSessionManager *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_session_manager_call_reboot_finish (
    PhoshDBusSessionManager *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_session_manager_call_reboot_sync (
    PhoshDBusSessionManager *proxy,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_session_manager_call_inhibit (
    PhoshDBusSessionManager *proxy,
    const gchar *arg_app_id,
    guint arg_toplevel_xid,
    const gchar *arg_reason,
    guint arg_flags,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_session_manager_call_inhibit_finish (
    PhoshDBusSessionManager *proxy,
    guint* out_inhibit_cookie,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_session_manager_call_inhibit_sync (
    PhoshDBusSessionManager *proxy,
    const gchar *arg_app_id,
    guint arg_toplevel_xid,
    const gchar *arg_reason,
    guint arg_flags,
    guint* out_inhibit_cookie,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_session_manager_call_uninhibit (
    PhoshDBusSessionManager *proxy,
    guint arg_inhibit_cookie,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_session_manager_call_uninhibit_finish (
    PhoshDBusSessionManager *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_session_manager_call_uninhibit_sync (
    PhoshDBusSessionManager *proxy,
    guint arg_inhibit_cookie,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_session_manager_call_can_shutdown (
    PhoshDBusSessionManager *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_session_manager_call_can_shutdown_finish (
    PhoshDBusSessionManager *proxy,
    gboolean* out_unnamed_arg0,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_session_manager_call_can_shutdown_sync (
    PhoshDBusSessionManager *proxy,
    gboolean* out_unnamed_arg0,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_session_manager_call_is_inhibited (
    PhoshDBusSessionManager *proxy,
    guint arg_unnamed_arg0,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_session_manager_call_is_inhibited_finish (
    PhoshDBusSessionManager *proxy,
    gboolean* out_unnamed_arg1,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_session_manager_call_is_inhibited_sync (
    PhoshDBusSessionManager *proxy,
    guint arg_unnamed_arg0,
    gboolean* out_unnamed_arg1,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_session_manager_call_register_client (
    PhoshDBusSessionManager *proxy,
    const gchar *arg_app_id,
    const gchar *arg_client_startup_id,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_session_manager_call_register_client_finish (
    PhoshDBusSessionManager *proxy,
    gchar ** out_client_id,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_session_manager_call_register_client_sync (
    PhoshDBusSessionManager *proxy,
    const gchar *arg_app_id,
    const gchar *arg_client_startup_id,
    gchar ** out_client_id,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
gboolean phosh_dbus_session_manager_get_session_is_active (PhoshDBusSessionManager *object);
void phosh_dbus_session_manager_set_session_is_active (PhoshDBusSessionManager *object, gboolean value);


/* ---- */

#define PHOSH_DBUS_TYPE_SESSION_MANAGER_PROXY (phosh_dbus_session_manager_proxy_get_type ())
#define PHOSH_DBUS_SESSION_MANAGER_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_SESSION_MANAGER_PROXY, PhoshDBusSessionManagerProxy))
#define PHOSH_DBUS_SESSION_MANAGER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_SESSION_MANAGER_PROXY, PhoshDBusSessionManagerProxyClass))
#define PHOSH_DBUS_SESSION_MANAGER_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_SESSION_MANAGER_PROXY, PhoshDBusSessionManagerProxyClass))
#define PHOSH_DBUS_IS_SESSION_MANAGER_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_SESSION_MANAGER_PROXY))
#define PHOSH_DBUS_IS_SESSION_MANAGER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_SESSION_MANAGER_PROXY))

typedef struct _PhoshDBusSessionManagerProxy PhoshDBusSessionManagerProxy;
typedef struct _PhoshDBusSessionManagerProxyClass PhoshDBusSessionManagerProxyClass;
typedef struct _PhoshDBusSessionManagerProxyPrivate PhoshDBusSessionManagerProxyPrivate;

struct _PhoshDBusSessionManagerProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusSessionManagerProxyPrivate *priv;
};

struct _PhoshDBusSessionManagerProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_session_manager_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusSessionManagerProxy, g_object_unref)
#endif

void phosh_dbus_session_manager_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusSessionManager *phosh_dbus_session_manager_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusSessionManager *phosh_dbus_session_manager_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_session_manager_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusSessionManager *phosh_dbus_session_manager_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusSessionManager *phosh_dbus_session_manager_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_SESSION_MANAGER_SKELETON (phosh_dbus_session_manager_skeleton_get_type ())
#define PHOSH_DBUS_SESSION_MANAGER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_SESSION_MANAGER_SKELETON, PhoshDBusSessionManagerSkeleton))
#define PHOSH_DBUS_SESSION_MANAGER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_SESSION_MANAGER_SKELETON, PhoshDBusSessionManagerSkeletonClass))
#define PHOSH_DBUS_SESSION_MANAGER_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_SESSION_MANAGER_SKELETON, PhoshDBusSessionManagerSkeletonClass))
#define PHOSH_DBUS_IS_SESSION_MANAGER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_SESSION_MANAGER_SKELETON))
#define PHOSH_DBUS_IS_SESSION_MANAGER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_SESSION_MANAGER_SKELETON))

typedef struct _PhoshDBusSessionManagerSkeleton PhoshDBusSessionManagerSkeleton;
typedef struct _PhoshDBusSessionManagerSkeletonClass PhoshDBusSessionManagerSkeletonClass;
typedef struct _PhoshDBusSessionManagerSkeletonPrivate PhoshDBusSessionManagerSkeletonPrivate;

struct _PhoshDBusSessionManagerSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusSessionManagerSkeletonPrivate *priv;
};

struct _PhoshDBusSessionManagerSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_session_manager_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusSessionManagerSkeleton, g_object_unref)
#endif

PhoshDBusSessionManager *phosh_dbus_session_manager_skeleton_new (void);


G_END_DECLS

#endif /* __GNOME_SESSION_DBUS_H__ */
