/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __PHOSH_PLUGIN_UPCOMING_EVENTS_PHOSH_CALENDAR_DBUS_H__
#define __PHOSH_PLUGIN_UPCOMING_EVENTS_PHOSH_CALENDAR_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for mobi.phosh.Shell.CalendarServer */

#define PHOSH_PLUGIN_DBUS_TYPE_CALENDAR_SERVER (phosh_plugin_dbus_calendar_server_get_type ())
#define PHOSH_PLUGIN_DBUS_CALENDAR_SERVER(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_PLUGIN_DBUS_TYPE_CALENDAR_SERVER, PhoshPluginDBusCalendarServer))
#define PHOSH_PLUGIN_DBUS_IS_CALENDAR_SERVER(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_PLUGIN_DBUS_TYPE_CALENDAR_SERVER))
#define PHOSH_PLUGIN_DBUS_CALENDAR_SERVER_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_PLUGIN_DBUS_TYPE_CALENDAR_SERVER, PhoshPluginDBusCalendarServerIface))

struct _PhoshPluginDBusCalendarServer;
typedef struct _PhoshPluginDBusCalendarServer PhoshPluginDBusCalendarServer;
typedef struct _PhoshPluginDBusCalendarServerIface PhoshPluginDBusCalendarServerIface;

struct _PhoshPluginDBusCalendarServerIface
{
  GTypeInterface parent_iface;



  gboolean (*handle_set_time_range) (
    PhoshPluginDBusCalendarServer *object,
    GDBusMethodInvocation *invocation,
    gint64 arg_since,
    gint64 arg_until,
    gboolean arg_force_reload);

  gboolean  (*get_has_calendars) (PhoshPluginDBusCalendarServer *object);

  void (*client_disappeared) (
    PhoshPluginDBusCalendarServer *object,
    const gchar *arg_source_uid);

  void (*events_added_or_updated) (
    PhoshPluginDBusCalendarServer *object,
    GVariant *arg_events);

  void (*events_removed) (
    PhoshPluginDBusCalendarServer *object,
    const gchar *const *arg_ids);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshPluginDBusCalendarServer, g_object_unref)
#endif

GType phosh_plugin_dbus_calendar_server_get_type (void);

GDBusInterfaceInfo *phosh_plugin_dbus_calendar_server_interface_info (void);
guint phosh_plugin_dbus_calendar_server_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void phosh_plugin_dbus_calendar_server_complete_set_time_range (
    PhoshPluginDBusCalendarServer *object,
    GDBusMethodInvocation *invocation);



/* D-Bus signal emissions functions: */
void phosh_plugin_dbus_calendar_server_emit_events_added_or_updated (
    PhoshPluginDBusCalendarServer *object,
    GVariant *arg_events);

void phosh_plugin_dbus_calendar_server_emit_events_removed (
    PhoshPluginDBusCalendarServer *object,
    const gchar *const *arg_ids);

void phosh_plugin_dbus_calendar_server_emit_client_disappeared (
    PhoshPluginDBusCalendarServer *object,
    const gchar *arg_source_uid);



/* D-Bus method calls: */
void phosh_plugin_dbus_calendar_server_call_set_time_range (
    PhoshPluginDBusCalendarServer *proxy,
    gint64 arg_since,
    gint64 arg_until,
    gboolean arg_force_reload,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_plugin_dbus_calendar_server_call_set_time_range_finish (
    PhoshPluginDBusCalendarServer *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_plugin_dbus_calendar_server_call_set_time_range_sync (
    PhoshPluginDBusCalendarServer *proxy,
    gint64 arg_since,
    gint64 arg_until,
    gboolean arg_force_reload,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
gboolean phosh_plugin_dbus_calendar_server_get_has_calendars (PhoshPluginDBusCalendarServer *object);
void phosh_plugin_dbus_calendar_server_set_has_calendars (PhoshPluginDBusCalendarServer *object, gboolean value);


/* ---- */

#define PHOSH_PLUGIN_DBUS_TYPE_CALENDAR_SERVER_PROXY (phosh_plugin_dbus_calendar_server_proxy_get_type ())
#define PHOSH_PLUGIN_DBUS_CALENDAR_SERVER_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_PLUGIN_DBUS_TYPE_CALENDAR_SERVER_PROXY, PhoshPluginDBusCalendarServerProxy))
#define PHOSH_PLUGIN_DBUS_CALENDAR_SERVER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_PLUGIN_DBUS_TYPE_CALENDAR_SERVER_PROXY, PhoshPluginDBusCalendarServerProxyClass))
#define PHOSH_PLUGIN_DBUS_CALENDAR_SERVER_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_PLUGIN_DBUS_TYPE_CALENDAR_SERVER_PROXY, PhoshPluginDBusCalendarServerProxyClass))
#define PHOSH_PLUGIN_DBUS_IS_CALENDAR_SERVER_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_PLUGIN_DBUS_TYPE_CALENDAR_SERVER_PROXY))
#define PHOSH_PLUGIN_DBUS_IS_CALENDAR_SERVER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_PLUGIN_DBUS_TYPE_CALENDAR_SERVER_PROXY))

typedef struct _PhoshPluginDBusCalendarServerProxy PhoshPluginDBusCalendarServerProxy;
typedef struct _PhoshPluginDBusCalendarServerProxyClass PhoshPluginDBusCalendarServerProxyClass;
typedef struct _PhoshPluginDBusCalendarServerProxyPrivate PhoshPluginDBusCalendarServerProxyPrivate;

struct _PhoshPluginDBusCalendarServerProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshPluginDBusCalendarServerProxyPrivate *priv;
};

struct _PhoshPluginDBusCalendarServerProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_plugin_dbus_calendar_server_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshPluginDBusCalendarServerProxy, g_object_unref)
#endif

void phosh_plugin_dbus_calendar_server_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshPluginDBusCalendarServer *phosh_plugin_dbus_calendar_server_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshPluginDBusCalendarServer *phosh_plugin_dbus_calendar_server_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_plugin_dbus_calendar_server_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshPluginDBusCalendarServer *phosh_plugin_dbus_calendar_server_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshPluginDBusCalendarServer *phosh_plugin_dbus_calendar_server_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_PLUGIN_DBUS_TYPE_CALENDAR_SERVER_SKELETON (phosh_plugin_dbus_calendar_server_skeleton_get_type ())
#define PHOSH_PLUGIN_DBUS_CALENDAR_SERVER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_PLUGIN_DBUS_TYPE_CALENDAR_SERVER_SKELETON, PhoshPluginDBusCalendarServerSkeleton))
#define PHOSH_PLUGIN_DBUS_CALENDAR_SERVER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_PLUGIN_DBUS_TYPE_CALENDAR_SERVER_SKELETON, PhoshPluginDBusCalendarServerSkeletonClass))
#define PHOSH_PLUGIN_DBUS_CALENDAR_SERVER_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_PLUGIN_DBUS_TYPE_CALENDAR_SERVER_SKELETON, PhoshPluginDBusCalendarServerSkeletonClass))
#define PHOSH_PLUGIN_DBUS_IS_CALENDAR_SERVER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_PLUGIN_DBUS_TYPE_CALENDAR_SERVER_SKELETON))
#define PHOSH_PLUGIN_DBUS_IS_CALENDAR_SERVER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_PLUGIN_DBUS_TYPE_CALENDAR_SERVER_SKELETON))

typedef struct _PhoshPluginDBusCalendarServerSkeleton PhoshPluginDBusCalendarServerSkeleton;
typedef struct _PhoshPluginDBusCalendarServerSkeletonClass PhoshPluginDBusCalendarServerSkeletonClass;
typedef struct _PhoshPluginDBusCalendarServerSkeletonPrivate PhoshPluginDBusCalendarServerSkeletonPrivate;

struct _PhoshPluginDBusCalendarServerSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshPluginDBusCalendarServerSkeletonPrivate *priv;
};

struct _PhoshPluginDBusCalendarServerSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_plugin_dbus_calendar_server_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshPluginDBusCalendarServerSkeleton, g_object_unref)
#endif

PhoshPluginDBusCalendarServer *phosh_plugin_dbus_calendar_server_skeleton_new (void);


G_END_DECLS

#endif /* __PHOSH_PLUGIN_UPCOMING_EVENTS_PHOSH_CALENDAR_DBUS_H__ */
