
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "cui-enums.h"
#include "cui-call.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "cui-call.h" */

GType
cui_call_state_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (CUI_CALL_STATE_UNKNOWN), "CUI_CALL_STATE_UNKNOWN", "unknown" },
        { C_ENUM (CUI_CALL_STATE_ACTIVE), "CUI_CALL_STATE_ACTIVE", "active" },
        { C_ENUM (CUI_CALL_STATE_HELD), "CUI_CALL_STATE_HELD", "held" },
        { C_ENUM (CUI_CALL_STATE_CALLING), "CUI_CALL_STATE_CALLING", "calling" },
        { C_ENUM (CUI_CALL_STATE_INCOMING), "CUI_CALL_STATE_INCOMING", "incoming" },
        { C_ENUM (CUI_CALL_STATE_DISCONNECTED), "CUI_CALL_STATE_DISCONNECTED", "disconnected" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("CuiCallState"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

