/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __ACCOUNTS_DBUS_H__
#define __ACCOUNTS_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.freedesktop.Accounts */

#define PHOSH_DBUS_TYPE_ACCOUNTS (phosh_dbus_accounts_get_type ())
#define PHOSH_DBUS_ACCOUNTS(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_ACCOUNTS, PhoshDBusAccounts))
#define PHOSH_DBUS_IS_ACCOUNTS(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_ACCOUNTS))
#define PHOSH_DBUS_ACCOUNTS_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_ACCOUNTS, PhoshDBusAccountsIface))

struct _PhoshDBusAccounts;
typedef struct _PhoshDBusAccounts PhoshDBusAccounts;
typedef struct _PhoshDBusAccountsIface PhoshDBusAccountsIface;

struct _PhoshDBusAccountsIface
{
  GTypeInterface parent_iface;

  gboolean (*handle_find_user_by_name) (
    PhoshDBusAccounts *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_name);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusAccounts, g_object_unref)
#endif

GType phosh_dbus_accounts_get_type (void);

GDBusInterfaceInfo *phosh_dbus_accounts_interface_info (void);
guint phosh_dbus_accounts_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void phosh_dbus_accounts_complete_find_user_by_name (
    PhoshDBusAccounts *object,
    GDBusMethodInvocation *invocation,
    const gchar *user);



/* D-Bus method calls: */
void phosh_dbus_accounts_call_find_user_by_name (
    PhoshDBusAccounts *proxy,
    const gchar *arg_name,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_accounts_call_find_user_by_name_finish (
    PhoshDBusAccounts *proxy,
    gchar ** out_user,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_accounts_call_find_user_by_name_sync (
    PhoshDBusAccounts *proxy,
    const gchar *arg_name,
    gchar ** out_user,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define PHOSH_DBUS_TYPE_ACCOUNTS_PROXY (phosh_dbus_accounts_proxy_get_type ())
#define PHOSH_DBUS_ACCOUNTS_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_ACCOUNTS_PROXY, PhoshDBusAccountsProxy))
#define PHOSH_DBUS_ACCOUNTS_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_ACCOUNTS_PROXY, PhoshDBusAccountsProxyClass))
#define PHOSH_DBUS_ACCOUNTS_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_ACCOUNTS_PROXY, PhoshDBusAccountsProxyClass))
#define PHOSH_DBUS_IS_ACCOUNTS_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_ACCOUNTS_PROXY))
#define PHOSH_DBUS_IS_ACCOUNTS_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_ACCOUNTS_PROXY))

typedef struct _PhoshDBusAccountsProxy PhoshDBusAccountsProxy;
typedef struct _PhoshDBusAccountsProxyClass PhoshDBusAccountsProxyClass;
typedef struct _PhoshDBusAccountsProxyPrivate PhoshDBusAccountsProxyPrivate;

struct _PhoshDBusAccountsProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusAccountsProxyPrivate *priv;
};

struct _PhoshDBusAccountsProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_accounts_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusAccountsProxy, g_object_unref)
#endif

void phosh_dbus_accounts_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusAccounts *phosh_dbus_accounts_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusAccounts *phosh_dbus_accounts_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_accounts_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusAccounts *phosh_dbus_accounts_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusAccounts *phosh_dbus_accounts_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_ACCOUNTS_SKELETON (phosh_dbus_accounts_skeleton_get_type ())
#define PHOSH_DBUS_ACCOUNTS_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_ACCOUNTS_SKELETON, PhoshDBusAccountsSkeleton))
#define PHOSH_DBUS_ACCOUNTS_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_ACCOUNTS_SKELETON, PhoshDBusAccountsSkeletonClass))
#define PHOSH_DBUS_ACCOUNTS_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_ACCOUNTS_SKELETON, PhoshDBusAccountsSkeletonClass))
#define PHOSH_DBUS_IS_ACCOUNTS_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_ACCOUNTS_SKELETON))
#define PHOSH_DBUS_IS_ACCOUNTS_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_ACCOUNTS_SKELETON))

typedef struct _PhoshDBusAccountsSkeleton PhoshDBusAccountsSkeleton;
typedef struct _PhoshDBusAccountsSkeletonClass PhoshDBusAccountsSkeletonClass;
typedef struct _PhoshDBusAccountsSkeletonPrivate PhoshDBusAccountsSkeletonPrivate;

struct _PhoshDBusAccountsSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusAccountsSkeletonPrivate *priv;
};

struct _PhoshDBusAccountsSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_accounts_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusAccountsSkeleton, g_object_unref)
#endif

PhoshDBusAccounts *phosh_dbus_accounts_skeleton_new (void);


G_END_DECLS

#endif /* __ACCOUNTS_DBUS_H__ */
