/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __NOTIFY_DBUS_H__
#define __NOTIFY_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.freedesktop.Notifications */

#define PHOSH_DBUS_TYPE_NOTIFICATIONS (phosh_dbus_notifications_get_type ())
#define PHOSH_DBUS_NOTIFICATIONS(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_NOTIFICATIONS, PhoshDBusNotifications))
#define PHOSH_DBUS_IS_NOTIFICATIONS(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_NOTIFICATIONS))
#define PHOSH_DBUS_NOTIFICATIONS_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_NOTIFICATIONS, PhoshDBusNotificationsIface))

struct _PhoshDBusNotifications;
typedef struct _PhoshDBusNotifications PhoshDBusNotifications;
typedef struct _PhoshDBusNotificationsIface PhoshDBusNotificationsIface;

struct _PhoshDBusNotificationsIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_close_notification) (
    PhoshDBusNotifications *object,
    GDBusMethodInvocation *invocation,
    guint arg_id);

  gboolean (*handle_get_capabilities) (
    PhoshDBusNotifications *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_get_server_information) (
    PhoshDBusNotifications *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_notify) (
    PhoshDBusNotifications *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_app_name,
    guint arg_replaces_id,
    const gchar *arg_app_icon,
    const gchar *arg_summary,
    const gchar *arg_body,
    const gchar *const *arg_actions,
    GVariant *arg_hints,
    gint arg_expire_timeout);

  void (*action_invoked) (
    PhoshDBusNotifications *object,
    guint arg_id,
    const gchar *arg_action_key);

  void (*notification_closed) (
    PhoshDBusNotifications *object,
    guint arg_id,
    guint arg_reason);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusNotifications, g_object_unref)
#endif

GType phosh_dbus_notifications_get_type (void);

GDBusInterfaceInfo *phosh_dbus_notifications_interface_info (void);
guint phosh_dbus_notifications_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void phosh_dbus_notifications_complete_get_capabilities (
    PhoshDBusNotifications *object,
    GDBusMethodInvocation *invocation,
    const gchar *const *capabilities);

void phosh_dbus_notifications_complete_notify (
    PhoshDBusNotifications *object,
    GDBusMethodInvocation *invocation,
    guint id);

void phosh_dbus_notifications_complete_close_notification (
    PhoshDBusNotifications *object,
    GDBusMethodInvocation *invocation);

void phosh_dbus_notifications_complete_get_server_information (
    PhoshDBusNotifications *object,
    GDBusMethodInvocation *invocation,
    const gchar *name,
    const gchar *vendor,
    const gchar *version,
    const gchar *spec_version);



/* D-Bus signal emissions functions: */
void phosh_dbus_notifications_emit_notification_closed (
    PhoshDBusNotifications *object,
    guint arg_id,
    guint arg_reason);

void phosh_dbus_notifications_emit_action_invoked (
    PhoshDBusNotifications *object,
    guint arg_id,
    const gchar *arg_action_key);



/* D-Bus method calls: */
void phosh_dbus_notifications_call_get_capabilities (
    PhoshDBusNotifications *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_notifications_call_get_capabilities_finish (
    PhoshDBusNotifications *proxy,
    gchar *** out_capabilities,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_notifications_call_get_capabilities_sync (
    PhoshDBusNotifications *proxy,
    gchar *** out_capabilities,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_notifications_call_notify (
    PhoshDBusNotifications *proxy,
    const gchar *arg_app_name,
    guint arg_replaces_id,
    const gchar *arg_app_icon,
    const gchar *arg_summary,
    const gchar *arg_body,
    const gchar *const *arg_actions,
    GVariant *arg_hints,
    gint arg_expire_timeout,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_notifications_call_notify_finish (
    PhoshDBusNotifications *proxy,
    guint* out_id,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_notifications_call_notify_sync (
    PhoshDBusNotifications *proxy,
    const gchar *arg_app_name,
    guint arg_replaces_id,
    const gchar *arg_app_icon,
    const gchar *arg_summary,
    const gchar *arg_body,
    const gchar *const *arg_actions,
    GVariant *arg_hints,
    gint arg_expire_timeout,
    guint* out_id,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_notifications_call_close_notification (
    PhoshDBusNotifications *proxy,
    guint arg_id,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_notifications_call_close_notification_finish (
    PhoshDBusNotifications *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_notifications_call_close_notification_sync (
    PhoshDBusNotifications *proxy,
    guint arg_id,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_notifications_call_get_server_information (
    PhoshDBusNotifications *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_notifications_call_get_server_information_finish (
    PhoshDBusNotifications *proxy,
    gchar ** out_name,
    gchar ** out_vendor,
    gchar ** out_version,
    gchar ** out_spec_version,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_notifications_call_get_server_information_sync (
    PhoshDBusNotifications *proxy,
    gchar ** out_name,
    gchar ** out_vendor,
    gchar ** out_version,
    gchar ** out_spec_version,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define PHOSH_DBUS_TYPE_NOTIFICATIONS_PROXY (phosh_dbus_notifications_proxy_get_type ())
#define PHOSH_DBUS_NOTIFICATIONS_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_NOTIFICATIONS_PROXY, PhoshDBusNotificationsProxy))
#define PHOSH_DBUS_NOTIFICATIONS_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_NOTIFICATIONS_PROXY, PhoshDBusNotificationsProxyClass))
#define PHOSH_DBUS_NOTIFICATIONS_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_NOTIFICATIONS_PROXY, PhoshDBusNotificationsProxyClass))
#define PHOSH_DBUS_IS_NOTIFICATIONS_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_NOTIFICATIONS_PROXY))
#define PHOSH_DBUS_IS_NOTIFICATIONS_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_NOTIFICATIONS_PROXY))

typedef struct _PhoshDBusNotificationsProxy PhoshDBusNotificationsProxy;
typedef struct _PhoshDBusNotificationsProxyClass PhoshDBusNotificationsProxyClass;
typedef struct _PhoshDBusNotificationsProxyPrivate PhoshDBusNotificationsProxyPrivate;

struct _PhoshDBusNotificationsProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusNotificationsProxyPrivate *priv;
};

struct _PhoshDBusNotificationsProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_notifications_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusNotificationsProxy, g_object_unref)
#endif

void phosh_dbus_notifications_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusNotifications *phosh_dbus_notifications_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusNotifications *phosh_dbus_notifications_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_notifications_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusNotifications *phosh_dbus_notifications_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusNotifications *phosh_dbus_notifications_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_NOTIFICATIONS_SKELETON (phosh_dbus_notifications_skeleton_get_type ())
#define PHOSH_DBUS_NOTIFICATIONS_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_NOTIFICATIONS_SKELETON, PhoshDBusNotificationsSkeleton))
#define PHOSH_DBUS_NOTIFICATIONS_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_NOTIFICATIONS_SKELETON, PhoshDBusNotificationsSkeletonClass))
#define PHOSH_DBUS_NOTIFICATIONS_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_NOTIFICATIONS_SKELETON, PhoshDBusNotificationsSkeletonClass))
#define PHOSH_DBUS_IS_NOTIFICATIONS_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_NOTIFICATIONS_SKELETON))
#define PHOSH_DBUS_IS_NOTIFICATIONS_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_NOTIFICATIONS_SKELETON))

typedef struct _PhoshDBusNotificationsSkeleton PhoshDBusNotificationsSkeleton;
typedef struct _PhoshDBusNotificationsSkeletonClass PhoshDBusNotificationsSkeletonClass;
typedef struct _PhoshDBusNotificationsSkeletonPrivate PhoshDBusNotificationsSkeletonPrivate;

struct _PhoshDBusNotificationsSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusNotificationsSkeletonPrivate *priv;
};

struct _PhoshDBusNotificationsSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_notifications_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusNotificationsSkeleton, g_object_unref)
#endif

PhoshDBusNotifications *phosh_dbus_notifications_skeleton_new (void);


G_END_DECLS

#endif /* __NOTIFY_DBUS_H__ */
