/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __HOSTNAME1_DBUS_H__
#define __HOSTNAME1_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.freedesktop.hostname1 */

#define PHOSH_DBUS_TYPE_HOSTNAME1 (phosh_dbus_hostname1_get_type ())
#define PHOSH_DBUS_HOSTNAME1(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_HOSTNAME1, PhoshDBusHostname1))
#define PHOSH_DBUS_IS_HOSTNAME1(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_HOSTNAME1))
#define PHOSH_DBUS_HOSTNAME1_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_HOSTNAME1, PhoshDBusHostname1Iface))

struct _PhoshDBusHostname1;
typedef struct _PhoshDBusHostname1 PhoshDBusHostname1;
typedef struct _PhoshDBusHostname1Iface PhoshDBusHostname1Iface;

struct _PhoshDBusHostname1Iface
{
  GTypeInterface parent_iface;

  const gchar * (*get_chassis) (PhoshDBusHostname1 *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusHostname1, g_object_unref)
#endif

GType phosh_dbus_hostname1_get_type (void);

GDBusInterfaceInfo *phosh_dbus_hostname1_interface_info (void);
guint phosh_dbus_hostname1_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus property accessors: */
const gchar *phosh_dbus_hostname1_get_chassis (PhoshDBusHostname1 *object);
gchar *phosh_dbus_hostname1_dup_chassis (PhoshDBusHostname1 *object);
void phosh_dbus_hostname1_set_chassis (PhoshDBusHostname1 *object, const gchar *value);


/* ---- */

#define PHOSH_DBUS_TYPE_HOSTNAME1_PROXY (phosh_dbus_hostname1_proxy_get_type ())
#define PHOSH_DBUS_HOSTNAME1_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_HOSTNAME1_PROXY, PhoshDBusHostname1Proxy))
#define PHOSH_DBUS_HOSTNAME1_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_HOSTNAME1_PROXY, PhoshDBusHostname1ProxyClass))
#define PHOSH_DBUS_HOSTNAME1_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_HOSTNAME1_PROXY, PhoshDBusHostname1ProxyClass))
#define PHOSH_DBUS_IS_HOSTNAME1_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_HOSTNAME1_PROXY))
#define PHOSH_DBUS_IS_HOSTNAME1_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_HOSTNAME1_PROXY))

typedef struct _PhoshDBusHostname1Proxy PhoshDBusHostname1Proxy;
typedef struct _PhoshDBusHostname1ProxyClass PhoshDBusHostname1ProxyClass;
typedef struct _PhoshDBusHostname1ProxyPrivate PhoshDBusHostname1ProxyPrivate;

struct _PhoshDBusHostname1Proxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusHostname1ProxyPrivate *priv;
};

struct _PhoshDBusHostname1ProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_hostname1_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusHostname1Proxy, g_object_unref)
#endif

void phosh_dbus_hostname1_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusHostname1 *phosh_dbus_hostname1_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusHostname1 *phosh_dbus_hostname1_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_hostname1_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusHostname1 *phosh_dbus_hostname1_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusHostname1 *phosh_dbus_hostname1_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_HOSTNAME1_SKELETON (phosh_dbus_hostname1_skeleton_get_type ())
#define PHOSH_DBUS_HOSTNAME1_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_HOSTNAME1_SKELETON, PhoshDBusHostname1Skeleton))
#define PHOSH_DBUS_HOSTNAME1_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_HOSTNAME1_SKELETON, PhoshDBusHostname1SkeletonClass))
#define PHOSH_DBUS_HOSTNAME1_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_HOSTNAME1_SKELETON, PhoshDBusHostname1SkeletonClass))
#define PHOSH_DBUS_IS_HOSTNAME1_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_HOSTNAME1_SKELETON))
#define PHOSH_DBUS_IS_HOSTNAME1_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_HOSTNAME1_SKELETON))

typedef struct _PhoshDBusHostname1Skeleton PhoshDBusHostname1Skeleton;
typedef struct _PhoshDBusHostname1SkeletonClass PhoshDBusHostname1SkeletonClass;
typedef struct _PhoshDBusHostname1SkeletonPrivate PhoshDBusHostname1SkeletonPrivate;

struct _PhoshDBusHostname1Skeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusHostname1SkeletonPrivate *priv;
};

struct _PhoshDBusHostname1SkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_hostname1_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusHostname1Skeleton, g_object_unref)
#endif

PhoshDBusHostname1 *phosh_dbus_hostname1_skeleton_new (void);


G_END_DECLS

#endif /* __HOSTNAME1_DBUS_H__ */
