
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#pragma once

#include <glib-object.h>

G_BEGIN_DECLS

/* enumerations from "phosh-settings-enums.h" */

GType phosh_app_filter_mode_flags_get_type (void);
#define PHOSH_TYPE_APP_FILTER_MODE_FLAGS (phosh_app_filter_mode_flags_get_type())

GType phosh_shell_layout_get_type (void);
#define PHOSH_TYPE_SHELL_LAYOUT (phosh_shell_layout_get_type())

GType phosh_layout_clock_position_get_type (void);
#define PHOSH_TYPE_LAYOUT_CLOCK_POSITION (phosh_layout_clock_position_get_type())

GType phosh_notify_screen_wakeup_flags_get_type (void);
#define PHOSH_TYPE_NOTIFY_SCREEN_WAKEUP_FLAGS (phosh_notify_screen_wakeup_flags_get_type())

GType phosh_wwan_backend_get_type (void);
#define PHOSH_TYPE_WWAN_BACKEND (phosh_wwan_backend_get_type())

/* enumerations from "animation.h" */

GType phosh_animation_type_get_type (void);
#define PHOSH_TYPE_ANIMATION_TYPE (phosh_animation_type_get_type())

/* enumerations from "app-grid-button.h" */

GType phosh_app_grid_button_mode_get_type (void);
#define PHOSH_TYPE_APP_GRID_BUTTON_MODE (phosh_app_grid_button_mode_get_type())

/* enumerations from "drag-surface.h" */

GType phosh_drag_surface_state_get_type (void);
#define PHOSH_TYPE_DRAG_SURFACE_STATE (phosh_drag_surface_state_get_type())

GType phosh_drag_surface_drag_mode_get_type (void);
#define PHOSH_TYPE_DRAG_SURFACE_DRAG_MODE (phosh_drag_surface_drag_mode_get_type())

/* enumerations from "gnome-shell-manager.h" */

GType phosh_shell_action_mode_get_type (void);
#define PHOSH_TYPE_SHELL_ACTION_MODE (phosh_shell_action_mode_get_type())

GType phosh_shell_key_binding_flags_get_type (void);
#define PHOSH_TYPE_SHELL_KEY_BINDING_FLAGS (phosh_shell_key_binding_flags_get_type())

/* enumerations from "home.h" */

GType phosh_home_state_get_type (void);
#define PHOSH_TYPE_HOME_STATE (phosh_home_state_get_type())

/* enumerations from "lockscreen.h" */

GType phosh_lockscreen_page_get_type (void);
#define PHOSH_TYPE_LOCKSCREEN_PAGE (phosh_lockscreen_page_get_type())

/* enumerations from "mode-manager.h" */

GType phosh_mode_device_type_get_type (void);
#define PHOSH_TYPE_MODE_DEVICE_TYPE (phosh_mode_device_type_get_type())

GType phosh_mode_hw_flags_get_type (void);
#define PHOSH_TYPE_MODE_HW_FLAGS (phosh_mode_hw_flags_get_type())

/* enumerations from "monitor.h" */

GType phosh_monitor_connector_type_get_type (void);
#define PHOSH_TYPE_MONITOR_CONNECTOR_TYPE (phosh_monitor_connector_type_get_type())

GType phosh_monitor_transform_get_type (void);
#define PHOSH_TYPE_MONITOR_TRANSFORM (phosh_monitor_transform_get_type())

GType phosh_monitor_power_save_mode_get_type (void);
#define PHOSH_TYPE_MONITOR_POWER_SAVE_MODE (phosh_monitor_power_save_mode_get_type())

/* enumerations from "notification.h" */

GType phosh_notification_urgency_get_type (void);
#define PHOSH_TYPE_NOTIFICATION_URGENCY (phosh_notification_urgency_get_type())

GType phosh_notification_reason_get_type (void);
#define PHOSH_TYPE_NOTIFICATION_REASON (phosh_notification_reason_get_type())

/* enumerations from "phosh-wayland.h" */

GType phosh_wayland_seat_capabilities_get_type (void);
#define PHOSH_TYPE_WAYLAND_SEAT_CAPABILITIES (phosh_wayland_seat_capabilities_get_type())

/* enumerations from "rotation-manager.h" */

GType phosh_rotation_manager_mode_get_type (void);
#define PHOSH_TYPE_ROTATION_MANAGER_MODE (phosh_rotation_manager_mode_get_type())

/* enumerations from "shell-priv.h" */

GType phosh_shell_state_flags_get_type (void);
#define PHOSH_TYPE_SHELL_STATE_FLAGS (phosh_shell_state_flags_get_type())

GType phosh_shell_debug_flags_get_type (void);
#define PHOSH_TYPE_SHELL_DEBUG_FLAGS (phosh_shell_debug_flags_get_type())

/* enumerations from "top-panel.h" */

GType phosh_top_panel_state_get_type (void);
#define PHOSH_TYPE_TOP_PANEL_STATE (phosh_top_panel_state_get_type())

G_END_DECLS

/* Generated data ends here */

