
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "phosh-enums.h"
#include "animation.h"
#include "app-grid-button.h"
#include "drag-surface.h"
#include "gnome-shell-manager.h"
#include "home.h"
#include "mode-manager.h"
#include "monitor/monitor.h"
#include "notifications/notification.h"
#include "notifications/notify-manager.h"
#include "phosh-wayland.h"
#include "rotation-manager.h"
#include "shell-priv.h"
#include "top-panel.h"
#include "lockscreen.h"
#include "phosh-settings-enums.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "phosh-settings-enums.h" */

GType
phosh_app_filter_mode_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (PHOSH_APP_FILTER_MODE_FLAGS_NONE), "PHOSH_APP_FILTER_MODE_FLAGS_NONE", "none" },
        { C_FLAGS (PHOSH_APP_FILTER_MODE_FLAGS_ADAPTIVE), "PHOSH_APP_FILTER_MODE_FLAGS_ADAPTIVE", "adaptive" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("PhoshAppFilterModeFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
phosh_shell_layout_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (PHOSH_SHELL_LAYOUT_NONE), "PHOSH_SHELL_LAYOUT_NONE", "none" },
        { C_ENUM (PHOSH_SHELL_LAYOUT_DEVICE), "PHOSH_SHELL_LAYOUT_DEVICE", "device" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("PhoshShellLayout"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
phosh_layout_clock_position_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (PHOSH_LAYOUT_CLOCK_POS_CENTER), "PHOSH_LAYOUT_CLOCK_POS_CENTER", "center" },
        { C_ENUM (PHOSH_LAYOUT_CLOCK_POS_LEFT), "PHOSH_LAYOUT_CLOCK_POS_LEFT", "left" },
        { C_ENUM (PHOSH_LAYOUT_CLOCK_POS_RIGHT), "PHOSH_LAYOUT_CLOCK_POS_RIGHT", "right" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("PhoshLayoutClockPosition"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
phosh_notify_screen_wakeup_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (PHOSH_NOTIFY_SCREEN_WAKEUP_FLAG_ANY), "PHOSH_NOTIFY_SCREEN_WAKEUP_FLAG_ANY", "any" },
        { C_FLAGS (PHOSH_NOTIFY_SCREEN_WAKEUP_FLAG_URGENCY), "PHOSH_NOTIFY_SCREEN_WAKEUP_FLAG_URGENCY", "urgency" },
        { C_FLAGS (PHOSH_NOTIFY_SCREEN_WAKEUP_FLAG_CATEGORY), "PHOSH_NOTIFY_SCREEN_WAKEUP_FLAG_CATEGORY", "category" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("PhoshNotifyScreenWakeupFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
phosh_wwan_backend_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (PHOSH_WWAN_BACKEND_MM), "PHOSH_WWAN_BACKEND_MM", "modemmanager" },
        { C_ENUM (PHOSH_WWAN_BACKEND_OFONO), "PHOSH_WWAN_BACKEND_OFONO", "ofono" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("PhoshWWanBackend"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "animation.h" */

GType
phosh_animation_type_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (PHOSH_ANIMATION_TYPE_EASE_OUT_CUBIC), "PHOSH_ANIMATION_TYPE_EASE_OUT_CUBIC", "out-cubic" },
        { C_ENUM (PHOSH_ANIMATION_TYPE_EASE_IN_QUINTIC), "PHOSH_ANIMATION_TYPE_EASE_IN_QUINTIC", "in-quintic" },
        { C_ENUM (PHOSH_ANIMATION_TYPE_EASE_OUT_QUINTIC), "PHOSH_ANIMATION_TYPE_EASE_OUT_QUINTIC", "out-quintic" },
        { C_ENUM (PHOSH_ANIMATION_TYPE_EASE_OUT_BOUNCE), "PHOSH_ANIMATION_TYPE_EASE_OUT_BOUNCE", "out-bounce" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("PhoshAnimationType"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "app-grid-button.h" */

GType
phosh_app_grid_button_mode_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (PHOSH_APP_GRID_BUTTON_LAUNCHER), "PHOSH_APP_GRID_BUTTON_LAUNCHER", "launcher" },
        { C_ENUM (PHOSH_APP_GRID_BUTTON_FAVORITES), "PHOSH_APP_GRID_BUTTON_FAVORITES", "favorites" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("PhoshAppGridButtonMode"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "drag-surface.h" */

GType
phosh_drag_surface_state_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (PHOSH_DRAG_SURFACE_STATE_FOLDED), "PHOSH_DRAG_SURFACE_STATE_FOLDED", "folded" },
        { C_ENUM (PHOSH_DRAG_SURFACE_STATE_UNFOLDED), "PHOSH_DRAG_SURFACE_STATE_UNFOLDED", "unfolded" },
        { C_ENUM (PHOSH_DRAG_SURFACE_STATE_DRAGGED), "PHOSH_DRAG_SURFACE_STATE_DRAGGED", "dragged" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("PhoshDragSurfaceState"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
phosh_drag_surface_drag_mode_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (PHOSH_DRAG_SURFACE_DRAG_MODE_FULL), "PHOSH_DRAG_SURFACE_DRAG_MODE_FULL", "full" },
        { C_ENUM (PHOSH_DRAG_SURFACE_DRAG_MODE_HANDLE), "PHOSH_DRAG_SURFACE_DRAG_MODE_HANDLE", "handle" },
        { C_ENUM (PHOSH_DRAG_SURFACE_DRAG_MODE_NONE), "PHOSH_DRAG_SURFACE_DRAG_MODE_NONE", "none" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("PhoshDragSurfaceDragMode"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "gnome-shell-manager.h" */

GType
phosh_shell_action_mode_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (PHOSH_SHELL_ACTION_MODE_NONE), "PHOSH_SHELL_ACTION_MODE_NONE", "none" },
        { C_FLAGS (PHOSH_SHELL_ACTION_MODE_NORMAL), "PHOSH_SHELL_ACTION_MODE_NORMAL", "normal" },
        { C_FLAGS (PHOSH_SHELL_ACTION_MODE_OVERVIEW), "PHOSH_SHELL_ACTION_MODE_OVERVIEW", "overview" },
        { C_FLAGS (PHOSH_SHELL_ACTION_MODE_LOCK_SCREEN), "PHOSH_SHELL_ACTION_MODE_LOCK_SCREEN", "lock-screen" },
        { C_FLAGS (PHOSH_SHELL_ACTION_MODE_UNLOCK_SCREEN), "PHOSH_SHELL_ACTION_MODE_UNLOCK_SCREEN", "unlock-screen" },
        { C_FLAGS (PHOSH_SHELL_ACTION_MODE_LOGIN_SCREEN), "PHOSH_SHELL_ACTION_MODE_LOGIN_SCREEN", "login-screen" },
        { C_FLAGS (PHOSH_SHELL_ACTION_MODE_SYSTEM_MODAL), "PHOSH_SHELL_ACTION_MODE_SYSTEM_MODAL", "system-modal" },
        { C_FLAGS (PHOSH_SHELL_ACTION_MODE_LOOKING_GLASS), "PHOSH_SHELL_ACTION_MODE_LOOKING_GLASS", "looking-glass" },
        { C_FLAGS (PHOSH_SHELL_ACTION_MODE_POPUP), "PHOSH_SHELL_ACTION_MODE_POPUP", "popup" },
        { C_FLAGS (PHOSH_SHELL_ACTION_MODE_ALL), "PHOSH_SHELL_ACTION_MODE_ALL", "all" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("PhoshShellActionMode"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
phosh_shell_key_binding_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (PHOSH_SHELL_KEY_BINDING_NONE), "PHOSH_SHELL_KEY_BINDING_NONE", "none" },
        { C_FLAGS (PHOSH_SHELL_KEY_BINDING_PER_WINDOW), "PHOSH_SHELL_KEY_BINDING_PER_WINDOW", "per-window" },
        { C_FLAGS (PHOSH_SHELL_KEY_BINDING_BUILTIN), "PHOSH_SHELL_KEY_BINDING_BUILTIN", "builtin" },
        { C_FLAGS (PHOSH_SHELL_KEY_BINDING_IS_REVERSED), "PHOSH_SHELL_KEY_BINDING_IS_REVERSED", "is-reversed" },
        { C_FLAGS (PHOSH_SHELL_KEY_BINDING_NON_MASKABLE), "PHOSH_SHELL_KEY_BINDING_NON_MASKABLE", "non-maskable" },
        { C_FLAGS (PHOSH_SHELL_KEY_BINDING_NO_AUTO_GRAB), "PHOSH_SHELL_KEY_BINDING_NO_AUTO_GRAB", "no-auto-grab" },
        { C_FLAGS (PHOSH_SHELL_KEY_BINDING_IGNORE_AUTOREPEAT), "PHOSH_SHELL_KEY_BINDING_IGNORE_AUTOREPEAT", "ignore-autorepeat" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("PhoshShellKeyBindingFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "home.h" */

GType
phosh_home_state_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (PHOSH_HOME_STATE_FOLDED), "PHOSH_HOME_STATE_FOLDED", "folded" },
        { C_ENUM (PHOSH_HOME_STATE_UNFOLDED), "PHOSH_HOME_STATE_UNFOLDED", "unfolded" },
        { C_ENUM (PHOSH_HOME_STATE_TRANSITION), "PHOSH_HOME_STATE_TRANSITION", "transition" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("PhoshHomeState"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "lockscreen.h" */

GType
phosh_lockscreen_page_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (PHOSH_LOCKSCREEN_PAGE_INFO), "PHOSH_LOCKSCREEN_PAGE_INFO", "info" },
        { C_ENUM (PHOSH_LOCKSCREEN_PAGE_EXTRA), "PHOSH_LOCKSCREEN_PAGE_EXTRA", "extra" },
        { C_ENUM (PHOSH_LOCKSCREEN_PAGE_UNLOCK), "PHOSH_LOCKSCREEN_PAGE_UNLOCK", "unlock" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("PhoshLockscreenPage"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "mode-manager.h" */

GType
phosh_mode_device_type_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (PHOSH_MODE_DEVICE_TYPE_UNKNOWN), "PHOSH_MODE_DEVICE_TYPE_UNKNOWN", "unknown" },
        { C_ENUM (PHOSH_MODE_DEVICE_TYPE_PHONE), "PHOSH_MODE_DEVICE_TYPE_PHONE", "phone" },
        { C_ENUM (PHOSH_MODE_DEVICE_TYPE_LAPTOP), "PHOSH_MODE_DEVICE_TYPE_LAPTOP", "laptop" },
        { C_ENUM (PHOSH_MODE_DEVICE_TYPE_DESKTOP), "PHOSH_MODE_DEVICE_TYPE_DESKTOP", "desktop" },
        { C_ENUM (PHOSH_MODE_DEVICE_TYPE_TABLET), "PHOSH_MODE_DEVICE_TYPE_TABLET", "tablet" },
        { C_ENUM (PHOSH_MODE_DEVICE_TYPE_CONVERTIBLE), "PHOSH_MODE_DEVICE_TYPE_CONVERTIBLE", "convertible" },
        { C_ENUM (PHOSH_MODE_DEVICE_TYPE_EMBEDDED), "PHOSH_MODE_DEVICE_TYPE_EMBEDDED", "embedded" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("PhoshModeDeviceType"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
phosh_mode_hw_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (PHOSH_MODE_HW_NONE), "PHOSH_MODE_HW_NONE", "none" },
        { C_FLAGS (PHOSH_MODE_HW_EXT_DISPLAY), "PHOSH_MODE_HW_EXT_DISPLAY", "ext-display" },
        { C_FLAGS (PHOSH_MODE_HW_KEYBOARD), "PHOSH_MODE_HW_KEYBOARD", "keyboard" },
        { C_FLAGS (PHOSH_MODE_HW_POINTER), "PHOSH_MODE_HW_POINTER", "pointer" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("PhoshModeHwFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "monitor.h" */

GType
phosh_monitor_connector_type_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (PHOSH_MONITOR_CONNECTOR_TYPE_Unknown), "PHOSH_MONITOR_CONNECTOR_TYPE_Unknown", "unknown" },
        { C_ENUM (PHOSH_MONITOR_CONNECTOR_TYPE_VGA), "PHOSH_MONITOR_CONNECTOR_TYPE_VGA", "vga" },
        { C_ENUM (PHOSH_MONITOR_CONNECTOR_TYPE_DVII), "PHOSH_MONITOR_CONNECTOR_TYPE_DVII", "dvii" },
        { C_ENUM (PHOSH_MONITOR_CONNECTOR_TYPE_DVID), "PHOSH_MONITOR_CONNECTOR_TYPE_DVID", "dvid" },
        { C_ENUM (PHOSH_MONITOR_CONNECTOR_TYPE_DVIA), "PHOSH_MONITOR_CONNECTOR_TYPE_DVIA", "dvia" },
        { C_ENUM (PHOSH_MONITOR_CONNECTOR_TYPE_Composite), "PHOSH_MONITOR_CONNECTOR_TYPE_Composite", "composite" },
        { C_ENUM (PHOSH_MONITOR_CONNECTOR_TYPE_SVIDEO), "PHOSH_MONITOR_CONNECTOR_TYPE_SVIDEO", "svideo" },
        { C_ENUM (PHOSH_MONITOR_CONNECTOR_TYPE_LVDS), "PHOSH_MONITOR_CONNECTOR_TYPE_LVDS", "lvds" },
        { C_ENUM (PHOSH_MONITOR_CONNECTOR_TYPE_Component), "PHOSH_MONITOR_CONNECTOR_TYPE_Component", "component" },
        { C_ENUM (PHOSH_MONITOR_CONNECTOR_TYPE_9PinDIN), "PHOSH_MONITOR_CONNECTOR_TYPE_9PinDIN", "9pindin" },
        { C_ENUM (PHOSH_MONITOR_CONNECTOR_TYPE_DisplayPort), "PHOSH_MONITOR_CONNECTOR_TYPE_DisplayPort", "displayport" },
        { C_ENUM (PHOSH_MONITOR_CONNECTOR_TYPE_HDMIA), "PHOSH_MONITOR_CONNECTOR_TYPE_HDMIA", "hdmia" },
        { C_ENUM (PHOSH_MONITOR_CONNECTOR_TYPE_HDMIB), "PHOSH_MONITOR_CONNECTOR_TYPE_HDMIB", "hdmib" },
        { C_ENUM (PHOSH_MONITOR_CONNECTOR_TYPE_TV), "PHOSH_MONITOR_CONNECTOR_TYPE_TV", "tv" },
        { C_ENUM (PHOSH_MONITOR_CONNECTOR_TYPE_eDP), "PHOSH_MONITOR_CONNECTOR_TYPE_eDP", "edp" },
        { C_ENUM (PHOSH_MONITOR_CONNECTOR_TYPE_VIRTUAL), "PHOSH_MONITOR_CONNECTOR_TYPE_VIRTUAL", "virtual" },
        { C_ENUM (PHOSH_MONITOR_CONNECTOR_TYPE_DSI), "PHOSH_MONITOR_CONNECTOR_TYPE_DSI", "dsi" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("PhoshMonitorConnectorType"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
phosh_monitor_transform_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (PHOSH_MONITOR_TRANSFORM_NORMAL), "PHOSH_MONITOR_TRANSFORM_NORMAL", "normal" },
        { C_ENUM (PHOSH_MONITOR_TRANSFORM_90), "PHOSH_MONITOR_TRANSFORM_90", "90" },
        { C_ENUM (PHOSH_MONITOR_TRANSFORM_180), "PHOSH_MONITOR_TRANSFORM_180", "180" },
        { C_ENUM (PHOSH_MONITOR_TRANSFORM_270), "PHOSH_MONITOR_TRANSFORM_270", "270" },
        { C_ENUM (PHOSH_MONITOR_TRANSFORM_FLIPPED), "PHOSH_MONITOR_TRANSFORM_FLIPPED", "flipped" },
        { C_ENUM (PHOSH_MONITOR_TRANSFORM_FLIPPED_90), "PHOSH_MONITOR_TRANSFORM_FLIPPED_90", "flipped-90" },
        { C_ENUM (PHOSH_MONITOR_TRANSFORM_FLIPPED_180), "PHOSH_MONITOR_TRANSFORM_FLIPPED_180", "flipped-180" },
        { C_ENUM (PHOSH_MONITOR_TRANSFORM_FLIPPED_270), "PHOSH_MONITOR_TRANSFORM_FLIPPED_270", "flipped-270" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("PhoshMonitorTransform"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
phosh_monitor_power_save_mode_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (PHOSH_MONITOR_POWER_SAVE_MODE_OFF), "PHOSH_MONITOR_POWER_SAVE_MODE_OFF", "off" },
        { C_ENUM (PHOSH_MONITOR_POWER_SAVE_MODE_ON), "PHOSH_MONITOR_POWER_SAVE_MODE_ON", "on" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("PhoshMonitorPowerSaveMode"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "notification.h" */

GType
phosh_notification_urgency_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (PHOSH_NOTIFICATION_URGENCY_LOW), "PHOSH_NOTIFICATION_URGENCY_LOW", "low" },
        { C_ENUM (PHOSH_NOTIFICATION_URGENCY_NORMAL), "PHOSH_NOTIFICATION_URGENCY_NORMAL", "normal" },
        { C_ENUM (PHOSH_NOTIFICATION_URGENCY_CRITICAL), "PHOSH_NOTIFICATION_URGENCY_CRITICAL", "critical" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("PhoshNotificationUrgency"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
phosh_notification_reason_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (PHOSH_NOTIFICATION_REASON_EXPIRED), "PHOSH_NOTIFICATION_REASON_EXPIRED", "expired" },
        { C_ENUM (PHOSH_NOTIFICATION_REASON_DISMISSED), "PHOSH_NOTIFICATION_REASON_DISMISSED", "dismissed" },
        { C_ENUM (PHOSH_NOTIFICATION_REASON_CLOSED), "PHOSH_NOTIFICATION_REASON_CLOSED", "closed" },
        { C_ENUM (PHOSH_NOTIFICATION_REASON_UNDEFINED), "PHOSH_NOTIFICATION_REASON_UNDEFINED", "undefined" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("PhoshNotificationReason"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "phosh-wayland.h" */

GType
phosh_wayland_seat_capabilities_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (PHOSH_WAYLAND_SEAT_CAPABILITY_NONE), "PHOSH_WAYLAND_SEAT_CAPABILITY_NONE", "none" },
        { C_FLAGS (PHOSH_WAYLAND_SEAT_CAPABILITY_POINTER), "PHOSH_WAYLAND_SEAT_CAPABILITY_POINTER", "pointer" },
        { C_FLAGS (PHOSH_WAYLAND_SEAT_CAPABILITY_KEYBOARD), "PHOSH_WAYLAND_SEAT_CAPABILITY_KEYBOARD", "keyboard" },
        { C_FLAGS (PHOSH_WAYLAND_SEAT_CAPABILITY_TOUCH), "PHOSH_WAYLAND_SEAT_CAPABILITY_TOUCH", "touch" },
        { C_FLAGS (PHOSH_WAYLAND_SEAT_CAPABILITY_TABLET_MODE_SWITCH), "PHOSH_WAYLAND_SEAT_CAPABILITY_TABLET_MODE_SWITCH", "tablet-mode-switch" },
        { C_FLAGS (PHOSH_WAYLAND_SEAT_CAPABILITY_LID_SWITCH), "PHOSH_WAYLAND_SEAT_CAPABILITY_LID_SWITCH", "lid-switch" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("PhoshWaylandSeatCapabilities"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "rotation-manager.h" */

GType
phosh_rotation_manager_mode_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (PHOSH_ROTATION_MANAGER_MODE_OFF), "PHOSH_ROTATION_MANAGER_MODE_OFF", "off" },
        { C_ENUM (PHOSH_ROTATION_MANAGER_MODE_SENSOR), "PHOSH_ROTATION_MANAGER_MODE_SENSOR", "sensor" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("PhoshRotationManagerMode"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "shell-priv.h" */

GType
phosh_shell_state_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (PHOSH_STATE_NONE), "PHOSH_STATE_NONE", "none" },
        { C_FLAGS (PHOSH_STATE_MODAL_SYSTEM_PROMPT), "PHOSH_STATE_MODAL_SYSTEM_PROMPT", "modal-system-prompt" },
        { C_FLAGS (PHOSH_STATE_BLANKED), "PHOSH_STATE_BLANKED", "blanked" },
        { C_FLAGS (PHOSH_STATE_LOCKED), "PHOSH_STATE_LOCKED", "locked" },
        { C_FLAGS (PHOSH_STATE_SETTINGS), "PHOSH_STATE_SETTINGS", "settings" },
        { C_FLAGS (PHOSH_STATE_OVERVIEW), "PHOSH_STATE_OVERVIEW", "overview" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("PhoshShellStateFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
phosh_shell_debug_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (PHOSH_SHELL_DEBUG_FLAG_NONE), "PHOSH_SHELL_DEBUG_FLAG_NONE", "flag-none" },
        { C_FLAGS (PHOSH_SHELL_DEBUG_FLAG_ALWAYS_SPLASH), "PHOSH_SHELL_DEBUG_FLAG_ALWAYS_SPLASH", "flag-always-splash" },
        { C_FLAGS (PHOSH_SHELL_DEBUG_FLAG_FAKE_BUILTIN), "PHOSH_SHELL_DEBUG_FLAG_FAKE_BUILTIN", "flag-fake-builtin" },
        { C_FLAGS (PHOSH_SHELL_DEBUG_BACKLIGHT_NON_LINEAR), "PHOSH_SHELL_DEBUG_BACKLIGHT_NON_LINEAR", "backlight-non-linear" },
        { C_FLAGS (PHOSH_SHELL_DEBUG_APP_ACTIVATION), "PHOSH_SHELL_DEBUG_APP_ACTIVATION", "app-activation" },
        { C_FLAGS (PHOSH_SHELL_DEBUG_THUMBNAIL_OVERLAY), "PHOSH_SHELL_DEBUG_THUMBNAIL_OVERLAY", "thumbnail-overlay" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("PhoshShellDebugFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "top-panel.h" */

GType
phosh_top_panel_state_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (PHOSH_TOP_PANEL_STATE_FOLDED), "PHOSH_TOP_PANEL_STATE_FOLDED", "folded" },
        { C_ENUM (PHOSH_TOP_PANEL_STATE_UNFOLDED), "PHOSH_TOP_PANEL_STATE_UNFOLDED", "unfolded" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("PhoshTopPanelState"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

