/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __MPRIS_DBUS_H__
#define __MPRIS_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.mpris.MediaPlayer2 */

#define PHOSH_DBUS_TYPE_MEDIA_PLAYER2 (phosh_dbus_media_player2_get_type ())
#define PHOSH_DBUS_MEDIA_PLAYER2(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_MEDIA_PLAYER2, PhoshDBusMediaPlayer2))
#define PHOSH_DBUS_IS_MEDIA_PLAYER2(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_MEDIA_PLAYER2))
#define PHOSH_DBUS_MEDIA_PLAYER2_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_MEDIA_PLAYER2, PhoshDBusMediaPlayer2Iface))

struct _PhoshDBusMediaPlayer2;
typedef struct _PhoshDBusMediaPlayer2 PhoshDBusMediaPlayer2;
typedef struct _PhoshDBusMediaPlayer2Iface PhoshDBusMediaPlayer2Iface;

struct _PhoshDBusMediaPlayer2Iface
{
  GTypeInterface parent_iface;


  gboolean (*handle_raise) (
    PhoshDBusMediaPlayer2 *object,
    GDBusMethodInvocation *invocation);

  gboolean  (*get_can_raise) (PhoshDBusMediaPlayer2 *object);

  const gchar * (*get_desktop_entry) (PhoshDBusMediaPlayer2 *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusMediaPlayer2, g_object_unref)
#endif

GType phosh_dbus_media_player2_get_type (void);

GDBusInterfaceInfo *phosh_dbus_media_player2_interface_info (void);
guint phosh_dbus_media_player2_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void phosh_dbus_media_player2_complete_raise (
    PhoshDBusMediaPlayer2 *object,
    GDBusMethodInvocation *invocation);



/* D-Bus method calls: */
void phosh_dbus_media_player2_call_raise (
    PhoshDBusMediaPlayer2 *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_media_player2_call_raise_finish (
    PhoshDBusMediaPlayer2 *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_media_player2_call_raise_sync (
    PhoshDBusMediaPlayer2 *proxy,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
gboolean phosh_dbus_media_player2_get_can_raise (PhoshDBusMediaPlayer2 *object);
void phosh_dbus_media_player2_set_can_raise (PhoshDBusMediaPlayer2 *object, gboolean value);

const gchar *phosh_dbus_media_player2_get_desktop_entry (PhoshDBusMediaPlayer2 *object);
gchar *phosh_dbus_media_player2_dup_desktop_entry (PhoshDBusMediaPlayer2 *object);
void phosh_dbus_media_player2_set_desktop_entry (PhoshDBusMediaPlayer2 *object, const gchar *value);


/* ---- */

#define PHOSH_DBUS_TYPE_MEDIA_PLAYER2_PROXY (phosh_dbus_media_player2_proxy_get_type ())
#define PHOSH_DBUS_MEDIA_PLAYER2_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_MEDIA_PLAYER2_PROXY, PhoshDBusMediaPlayer2Proxy))
#define PHOSH_DBUS_MEDIA_PLAYER2_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_MEDIA_PLAYER2_PROXY, PhoshDBusMediaPlayer2ProxyClass))
#define PHOSH_DBUS_MEDIA_PLAYER2_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_MEDIA_PLAYER2_PROXY, PhoshDBusMediaPlayer2ProxyClass))
#define PHOSH_DBUS_IS_MEDIA_PLAYER2_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_MEDIA_PLAYER2_PROXY))
#define PHOSH_DBUS_IS_MEDIA_PLAYER2_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_MEDIA_PLAYER2_PROXY))

typedef struct _PhoshDBusMediaPlayer2Proxy PhoshDBusMediaPlayer2Proxy;
typedef struct _PhoshDBusMediaPlayer2ProxyClass PhoshDBusMediaPlayer2ProxyClass;
typedef struct _PhoshDBusMediaPlayer2ProxyPrivate PhoshDBusMediaPlayer2ProxyPrivate;

struct _PhoshDBusMediaPlayer2Proxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusMediaPlayer2ProxyPrivate *priv;
};

struct _PhoshDBusMediaPlayer2ProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_media_player2_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusMediaPlayer2Proxy, g_object_unref)
#endif

void phosh_dbus_media_player2_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusMediaPlayer2 *phosh_dbus_media_player2_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusMediaPlayer2 *phosh_dbus_media_player2_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_media_player2_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusMediaPlayer2 *phosh_dbus_media_player2_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusMediaPlayer2 *phosh_dbus_media_player2_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_MEDIA_PLAYER2_SKELETON (phosh_dbus_media_player2_skeleton_get_type ())
#define PHOSH_DBUS_MEDIA_PLAYER2_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_MEDIA_PLAYER2_SKELETON, PhoshDBusMediaPlayer2Skeleton))
#define PHOSH_DBUS_MEDIA_PLAYER2_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_MEDIA_PLAYER2_SKELETON, PhoshDBusMediaPlayer2SkeletonClass))
#define PHOSH_DBUS_MEDIA_PLAYER2_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_MEDIA_PLAYER2_SKELETON, PhoshDBusMediaPlayer2SkeletonClass))
#define PHOSH_DBUS_IS_MEDIA_PLAYER2_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_MEDIA_PLAYER2_SKELETON))
#define PHOSH_DBUS_IS_MEDIA_PLAYER2_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_MEDIA_PLAYER2_SKELETON))

typedef struct _PhoshDBusMediaPlayer2Skeleton PhoshDBusMediaPlayer2Skeleton;
typedef struct _PhoshDBusMediaPlayer2SkeletonClass PhoshDBusMediaPlayer2SkeletonClass;
typedef struct _PhoshDBusMediaPlayer2SkeletonPrivate PhoshDBusMediaPlayer2SkeletonPrivate;

struct _PhoshDBusMediaPlayer2Skeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusMediaPlayer2SkeletonPrivate *priv;
};

struct _PhoshDBusMediaPlayer2SkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_media_player2_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusMediaPlayer2Skeleton, g_object_unref)
#endif

PhoshDBusMediaPlayer2 *phosh_dbus_media_player2_skeleton_new (void);


/* ------------------------------------------------------------------------ */
/* Declarations for org.mpris.MediaPlayer2.Player */

#define PHOSH_DBUS_TYPE_MEDIA_PLAYER2_PLAYER (phosh_dbus_media_player2_player_get_type ())
#define PHOSH_DBUS_MEDIA_PLAYER2_PLAYER(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_MEDIA_PLAYER2_PLAYER, PhoshDBusMediaPlayer2Player))
#define PHOSH_DBUS_IS_MEDIA_PLAYER2_PLAYER(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_MEDIA_PLAYER2_PLAYER))
#define PHOSH_DBUS_MEDIA_PLAYER2_PLAYER_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_MEDIA_PLAYER2_PLAYER, PhoshDBusMediaPlayer2PlayerIface))

struct _PhoshDBusMediaPlayer2Player;
typedef struct _PhoshDBusMediaPlayer2Player PhoshDBusMediaPlayer2Player;
typedef struct _PhoshDBusMediaPlayer2PlayerIface PhoshDBusMediaPlayer2PlayerIface;

struct _PhoshDBusMediaPlayer2PlayerIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_next) (
    PhoshDBusMediaPlayer2Player *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_play_pause) (
    PhoshDBusMediaPlayer2Player *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_previous) (
    PhoshDBusMediaPlayer2Player *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_seek) (
    PhoshDBusMediaPlayer2Player *object,
    GDBusMethodInvocation *invocation,
    gint64 arg_Offset);

  gboolean  (*get_can_go_next) (PhoshDBusMediaPlayer2Player *object);

  gboolean  (*get_can_go_previous) (PhoshDBusMediaPlayer2Player *object);

  gboolean  (*get_can_play) (PhoshDBusMediaPlayer2Player *object);

  gboolean  (*get_can_seek) (PhoshDBusMediaPlayer2Player *object);

  GVariant * (*get_metadata) (PhoshDBusMediaPlayer2Player *object);

  const gchar * (*get_playback_status) (PhoshDBusMediaPlayer2Player *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusMediaPlayer2Player, g_object_unref)
#endif

GType phosh_dbus_media_player2_player_get_type (void);

GDBusInterfaceInfo *phosh_dbus_media_player2_player_interface_info (void);
guint phosh_dbus_media_player2_player_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void phosh_dbus_media_player2_player_complete_play_pause (
    PhoshDBusMediaPlayer2Player *object,
    GDBusMethodInvocation *invocation);

void phosh_dbus_media_player2_player_complete_next (
    PhoshDBusMediaPlayer2Player *object,
    GDBusMethodInvocation *invocation);

void phosh_dbus_media_player2_player_complete_previous (
    PhoshDBusMediaPlayer2Player *object,
    GDBusMethodInvocation *invocation);

void phosh_dbus_media_player2_player_complete_seek (
    PhoshDBusMediaPlayer2Player *object,
    GDBusMethodInvocation *invocation);



/* D-Bus method calls: */
void phosh_dbus_media_player2_player_call_play_pause (
    PhoshDBusMediaPlayer2Player *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_media_player2_player_call_play_pause_finish (
    PhoshDBusMediaPlayer2Player *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_media_player2_player_call_play_pause_sync (
    PhoshDBusMediaPlayer2Player *proxy,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_media_player2_player_call_next (
    PhoshDBusMediaPlayer2Player *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_media_player2_player_call_next_finish (
    PhoshDBusMediaPlayer2Player *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_media_player2_player_call_next_sync (
    PhoshDBusMediaPlayer2Player *proxy,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_media_player2_player_call_previous (
    PhoshDBusMediaPlayer2Player *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_media_player2_player_call_previous_finish (
    PhoshDBusMediaPlayer2Player *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_media_player2_player_call_previous_sync (
    PhoshDBusMediaPlayer2Player *proxy,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_media_player2_player_call_seek (
    PhoshDBusMediaPlayer2Player *proxy,
    gint64 arg_Offset,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_media_player2_player_call_seek_finish (
    PhoshDBusMediaPlayer2Player *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_media_player2_player_call_seek_sync (
    PhoshDBusMediaPlayer2Player *proxy,
    gint64 arg_Offset,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
gboolean phosh_dbus_media_player2_player_get_can_go_next (PhoshDBusMediaPlayer2Player *object);
void phosh_dbus_media_player2_player_set_can_go_next (PhoshDBusMediaPlayer2Player *object, gboolean value);

gboolean phosh_dbus_media_player2_player_get_can_go_previous (PhoshDBusMediaPlayer2Player *object);
void phosh_dbus_media_player2_player_set_can_go_previous (PhoshDBusMediaPlayer2Player *object, gboolean value);

gboolean phosh_dbus_media_player2_player_get_can_play (PhoshDBusMediaPlayer2Player *object);
void phosh_dbus_media_player2_player_set_can_play (PhoshDBusMediaPlayer2Player *object, gboolean value);

gboolean phosh_dbus_media_player2_player_get_can_seek (PhoshDBusMediaPlayer2Player *object);
void phosh_dbus_media_player2_player_set_can_seek (PhoshDBusMediaPlayer2Player *object, gboolean value);

GVariant *phosh_dbus_media_player2_player_get_metadata (PhoshDBusMediaPlayer2Player *object);
GVariant *phosh_dbus_media_player2_player_dup_metadata (PhoshDBusMediaPlayer2Player *object);
void phosh_dbus_media_player2_player_set_metadata (PhoshDBusMediaPlayer2Player *object, GVariant *value);

const gchar *phosh_dbus_media_player2_player_get_playback_status (PhoshDBusMediaPlayer2Player *object);
gchar *phosh_dbus_media_player2_player_dup_playback_status (PhoshDBusMediaPlayer2Player *object);
void phosh_dbus_media_player2_player_set_playback_status (PhoshDBusMediaPlayer2Player *object, const gchar *value);


/* ---- */

#define PHOSH_DBUS_TYPE_MEDIA_PLAYER2_PLAYER_PROXY (phosh_dbus_media_player2_player_proxy_get_type ())
#define PHOSH_DBUS_MEDIA_PLAYER2_PLAYER_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_MEDIA_PLAYER2_PLAYER_PROXY, PhoshDBusMediaPlayer2PlayerProxy))
#define PHOSH_DBUS_MEDIA_PLAYER2_PLAYER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_MEDIA_PLAYER2_PLAYER_PROXY, PhoshDBusMediaPlayer2PlayerProxyClass))
#define PHOSH_DBUS_MEDIA_PLAYER2_PLAYER_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_MEDIA_PLAYER2_PLAYER_PROXY, PhoshDBusMediaPlayer2PlayerProxyClass))
#define PHOSH_DBUS_IS_MEDIA_PLAYER2_PLAYER_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_MEDIA_PLAYER2_PLAYER_PROXY))
#define PHOSH_DBUS_IS_MEDIA_PLAYER2_PLAYER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_MEDIA_PLAYER2_PLAYER_PROXY))

typedef struct _PhoshDBusMediaPlayer2PlayerProxy PhoshDBusMediaPlayer2PlayerProxy;
typedef struct _PhoshDBusMediaPlayer2PlayerProxyClass PhoshDBusMediaPlayer2PlayerProxyClass;
typedef struct _PhoshDBusMediaPlayer2PlayerProxyPrivate PhoshDBusMediaPlayer2PlayerProxyPrivate;

struct _PhoshDBusMediaPlayer2PlayerProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusMediaPlayer2PlayerProxyPrivate *priv;
};

struct _PhoshDBusMediaPlayer2PlayerProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_media_player2_player_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusMediaPlayer2PlayerProxy, g_object_unref)
#endif

void phosh_dbus_media_player2_player_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusMediaPlayer2Player *phosh_dbus_media_player2_player_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusMediaPlayer2Player *phosh_dbus_media_player2_player_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_media_player2_player_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusMediaPlayer2Player *phosh_dbus_media_player2_player_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusMediaPlayer2Player *phosh_dbus_media_player2_player_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_MEDIA_PLAYER2_PLAYER_SKELETON (phosh_dbus_media_player2_player_skeleton_get_type ())
#define PHOSH_DBUS_MEDIA_PLAYER2_PLAYER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_MEDIA_PLAYER2_PLAYER_SKELETON, PhoshDBusMediaPlayer2PlayerSkeleton))
#define PHOSH_DBUS_MEDIA_PLAYER2_PLAYER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_MEDIA_PLAYER2_PLAYER_SKELETON, PhoshDBusMediaPlayer2PlayerSkeletonClass))
#define PHOSH_DBUS_MEDIA_PLAYER2_PLAYER_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_MEDIA_PLAYER2_PLAYER_SKELETON, PhoshDBusMediaPlayer2PlayerSkeletonClass))
#define PHOSH_DBUS_IS_MEDIA_PLAYER2_PLAYER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_MEDIA_PLAYER2_PLAYER_SKELETON))
#define PHOSH_DBUS_IS_MEDIA_PLAYER2_PLAYER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_MEDIA_PLAYER2_PLAYER_SKELETON))

typedef struct _PhoshDBusMediaPlayer2PlayerSkeleton PhoshDBusMediaPlayer2PlayerSkeleton;
typedef struct _PhoshDBusMediaPlayer2PlayerSkeletonClass PhoshDBusMediaPlayer2PlayerSkeletonClass;
typedef struct _PhoshDBusMediaPlayer2PlayerSkeletonPrivate PhoshDBusMediaPlayer2PlayerSkeletonPrivate;

struct _PhoshDBusMediaPlayer2PlayerSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusMediaPlayer2PlayerSkeletonPrivate *priv;
};

struct _PhoshDBusMediaPlayer2PlayerSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_media_player2_player_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusMediaPlayer2PlayerSkeleton, g_object_unref)
#endif

PhoshDBusMediaPlayer2Player *phosh_dbus_media_player2_player_skeleton_new (void);


G_END_DECLS

#endif /* __MPRIS_DBUS_H__ */
