/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __PORTAL_DBUS_H__
#define __PORTAL_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.freedesktop.impl.portal.Request */

#define PHOSH_DBUS_TYPE_IMPL_PORTAL_REQUEST (phosh_dbus_impl_portal_request_get_type ())
#define PHOSH_DBUS_IMPL_PORTAL_REQUEST(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_IMPL_PORTAL_REQUEST, PhoshDBusImplPortalRequest))
#define PHOSH_DBUS_IS_IMPL_PORTAL_REQUEST(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_IMPL_PORTAL_REQUEST))
#define PHOSH_DBUS_IMPL_PORTAL_REQUEST_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_IMPL_PORTAL_REQUEST, PhoshDBusImplPortalRequestIface))

struct _PhoshDBusImplPortalRequest;
typedef struct _PhoshDBusImplPortalRequest PhoshDBusImplPortalRequest;
typedef struct _PhoshDBusImplPortalRequestIface PhoshDBusImplPortalRequestIface;

struct _PhoshDBusImplPortalRequestIface
{
  GTypeInterface parent_iface;

  gboolean (*handle_close) (
    PhoshDBusImplPortalRequest *object,
    GDBusMethodInvocation *invocation);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusImplPortalRequest, g_object_unref)
#endif

GType phosh_dbus_impl_portal_request_get_type (void);

GDBusInterfaceInfo *phosh_dbus_impl_portal_request_interface_info (void);
guint phosh_dbus_impl_portal_request_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void phosh_dbus_impl_portal_request_complete_close (
    PhoshDBusImplPortalRequest *object,
    GDBusMethodInvocation *invocation);



/* D-Bus method calls: */
void phosh_dbus_impl_portal_request_call_close (
    PhoshDBusImplPortalRequest *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_impl_portal_request_call_close_finish (
    PhoshDBusImplPortalRequest *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_impl_portal_request_call_close_sync (
    PhoshDBusImplPortalRequest *proxy,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define PHOSH_DBUS_TYPE_IMPL_PORTAL_REQUEST_PROXY (phosh_dbus_impl_portal_request_proxy_get_type ())
#define PHOSH_DBUS_IMPL_PORTAL_REQUEST_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_IMPL_PORTAL_REQUEST_PROXY, PhoshDBusImplPortalRequestProxy))
#define PHOSH_DBUS_IMPL_PORTAL_REQUEST_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_IMPL_PORTAL_REQUEST_PROXY, PhoshDBusImplPortalRequestProxyClass))
#define PHOSH_DBUS_IMPL_PORTAL_REQUEST_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_IMPL_PORTAL_REQUEST_PROXY, PhoshDBusImplPortalRequestProxyClass))
#define PHOSH_DBUS_IS_IMPL_PORTAL_REQUEST_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_IMPL_PORTAL_REQUEST_PROXY))
#define PHOSH_DBUS_IS_IMPL_PORTAL_REQUEST_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_IMPL_PORTAL_REQUEST_PROXY))

typedef struct _PhoshDBusImplPortalRequestProxy PhoshDBusImplPortalRequestProxy;
typedef struct _PhoshDBusImplPortalRequestProxyClass PhoshDBusImplPortalRequestProxyClass;
typedef struct _PhoshDBusImplPortalRequestProxyPrivate PhoshDBusImplPortalRequestProxyPrivate;

struct _PhoshDBusImplPortalRequestProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusImplPortalRequestProxyPrivate *priv;
};

struct _PhoshDBusImplPortalRequestProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_impl_portal_request_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusImplPortalRequestProxy, g_object_unref)
#endif

void phosh_dbus_impl_portal_request_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusImplPortalRequest *phosh_dbus_impl_portal_request_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusImplPortalRequest *phosh_dbus_impl_portal_request_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_impl_portal_request_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusImplPortalRequest *phosh_dbus_impl_portal_request_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusImplPortalRequest *phosh_dbus_impl_portal_request_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_IMPL_PORTAL_REQUEST_SKELETON (phosh_dbus_impl_portal_request_skeleton_get_type ())
#define PHOSH_DBUS_IMPL_PORTAL_REQUEST_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_IMPL_PORTAL_REQUEST_SKELETON, PhoshDBusImplPortalRequestSkeleton))
#define PHOSH_DBUS_IMPL_PORTAL_REQUEST_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_IMPL_PORTAL_REQUEST_SKELETON, PhoshDBusImplPortalRequestSkeletonClass))
#define PHOSH_DBUS_IMPL_PORTAL_REQUEST_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_IMPL_PORTAL_REQUEST_SKELETON, PhoshDBusImplPortalRequestSkeletonClass))
#define PHOSH_DBUS_IS_IMPL_PORTAL_REQUEST_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_IMPL_PORTAL_REQUEST_SKELETON))
#define PHOSH_DBUS_IS_IMPL_PORTAL_REQUEST_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_IMPL_PORTAL_REQUEST_SKELETON))

typedef struct _PhoshDBusImplPortalRequestSkeleton PhoshDBusImplPortalRequestSkeleton;
typedef struct _PhoshDBusImplPortalRequestSkeletonClass PhoshDBusImplPortalRequestSkeletonClass;
typedef struct _PhoshDBusImplPortalRequestSkeletonPrivate PhoshDBusImplPortalRequestSkeletonPrivate;

struct _PhoshDBusImplPortalRequestSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusImplPortalRequestSkeletonPrivate *priv;
};

struct _PhoshDBusImplPortalRequestSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_impl_portal_request_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusImplPortalRequestSkeleton, g_object_unref)
#endif

PhoshDBusImplPortalRequest *phosh_dbus_impl_portal_request_skeleton_new (void);


/* ------------------------------------------------------------------------ */
/* Declarations for org.freedesktop.impl.portal.Access */

#define PHOSH_DBUS_TYPE_IMPL_PORTAL_ACCESS (phosh_dbus_impl_portal_access_get_type ())
#define PHOSH_DBUS_IMPL_PORTAL_ACCESS(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_IMPL_PORTAL_ACCESS, PhoshDBusImplPortalAccess))
#define PHOSH_DBUS_IS_IMPL_PORTAL_ACCESS(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_IMPL_PORTAL_ACCESS))
#define PHOSH_DBUS_IMPL_PORTAL_ACCESS_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_IMPL_PORTAL_ACCESS, PhoshDBusImplPortalAccessIface))

struct _PhoshDBusImplPortalAccess;
typedef struct _PhoshDBusImplPortalAccess PhoshDBusImplPortalAccess;
typedef struct _PhoshDBusImplPortalAccessIface PhoshDBusImplPortalAccessIface;

struct _PhoshDBusImplPortalAccessIface
{
  GTypeInterface parent_iface;

  gboolean (*handle_access_dialog) (
    PhoshDBusImplPortalAccess *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_handle,
    const gchar *arg_app_id,
    const gchar *arg_parent_window,
    const gchar *arg_title,
    const gchar *arg_subtitle,
    const gchar *arg_body,
    GVariant *arg_options);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusImplPortalAccess, g_object_unref)
#endif

GType phosh_dbus_impl_portal_access_get_type (void);

GDBusInterfaceInfo *phosh_dbus_impl_portal_access_interface_info (void);
guint phosh_dbus_impl_portal_access_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void phosh_dbus_impl_portal_access_complete_access_dialog (
    PhoshDBusImplPortalAccess *object,
    GDBusMethodInvocation *invocation,
    guint response,
    GVariant *results);



/* D-Bus method calls: */
void phosh_dbus_impl_portal_access_call_access_dialog (
    PhoshDBusImplPortalAccess *proxy,
    const gchar *arg_handle,
    const gchar *arg_app_id,
    const gchar *arg_parent_window,
    const gchar *arg_title,
    const gchar *arg_subtitle,
    const gchar *arg_body,
    GVariant *arg_options,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_impl_portal_access_call_access_dialog_finish (
    PhoshDBusImplPortalAccess *proxy,
    guint* out_response,
    GVariant ** out_results,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_impl_portal_access_call_access_dialog_sync (
    PhoshDBusImplPortalAccess *proxy,
    const gchar *arg_handle,
    const gchar *arg_app_id,
    const gchar *arg_parent_window,
    const gchar *arg_title,
    const gchar *arg_subtitle,
    const gchar *arg_body,
    GVariant *arg_options,
    guint* out_response,
    GVariant ** out_results,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define PHOSH_DBUS_TYPE_IMPL_PORTAL_ACCESS_PROXY (phosh_dbus_impl_portal_access_proxy_get_type ())
#define PHOSH_DBUS_IMPL_PORTAL_ACCESS_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_IMPL_PORTAL_ACCESS_PROXY, PhoshDBusImplPortalAccessProxy))
#define PHOSH_DBUS_IMPL_PORTAL_ACCESS_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_IMPL_PORTAL_ACCESS_PROXY, PhoshDBusImplPortalAccessProxyClass))
#define PHOSH_DBUS_IMPL_PORTAL_ACCESS_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_IMPL_PORTAL_ACCESS_PROXY, PhoshDBusImplPortalAccessProxyClass))
#define PHOSH_DBUS_IS_IMPL_PORTAL_ACCESS_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_IMPL_PORTAL_ACCESS_PROXY))
#define PHOSH_DBUS_IS_IMPL_PORTAL_ACCESS_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_IMPL_PORTAL_ACCESS_PROXY))

typedef struct _PhoshDBusImplPortalAccessProxy PhoshDBusImplPortalAccessProxy;
typedef struct _PhoshDBusImplPortalAccessProxyClass PhoshDBusImplPortalAccessProxyClass;
typedef struct _PhoshDBusImplPortalAccessProxyPrivate PhoshDBusImplPortalAccessProxyPrivate;

struct _PhoshDBusImplPortalAccessProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusImplPortalAccessProxyPrivate *priv;
};

struct _PhoshDBusImplPortalAccessProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_impl_portal_access_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusImplPortalAccessProxy, g_object_unref)
#endif

void phosh_dbus_impl_portal_access_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusImplPortalAccess *phosh_dbus_impl_portal_access_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusImplPortalAccess *phosh_dbus_impl_portal_access_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_impl_portal_access_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusImplPortalAccess *phosh_dbus_impl_portal_access_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusImplPortalAccess *phosh_dbus_impl_portal_access_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_IMPL_PORTAL_ACCESS_SKELETON (phosh_dbus_impl_portal_access_skeleton_get_type ())
#define PHOSH_DBUS_IMPL_PORTAL_ACCESS_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_IMPL_PORTAL_ACCESS_SKELETON, PhoshDBusImplPortalAccessSkeleton))
#define PHOSH_DBUS_IMPL_PORTAL_ACCESS_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_IMPL_PORTAL_ACCESS_SKELETON, PhoshDBusImplPortalAccessSkeletonClass))
#define PHOSH_DBUS_IMPL_PORTAL_ACCESS_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_IMPL_PORTAL_ACCESS_SKELETON, PhoshDBusImplPortalAccessSkeletonClass))
#define PHOSH_DBUS_IS_IMPL_PORTAL_ACCESS_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_IMPL_PORTAL_ACCESS_SKELETON))
#define PHOSH_DBUS_IS_IMPL_PORTAL_ACCESS_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_IMPL_PORTAL_ACCESS_SKELETON))

typedef struct _PhoshDBusImplPortalAccessSkeleton PhoshDBusImplPortalAccessSkeleton;
typedef struct _PhoshDBusImplPortalAccessSkeletonClass PhoshDBusImplPortalAccessSkeletonClass;
typedef struct _PhoshDBusImplPortalAccessSkeletonPrivate PhoshDBusImplPortalAccessSkeletonPrivate;

struct _PhoshDBusImplPortalAccessSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusImplPortalAccessSkeletonPrivate *priv;
};

struct _PhoshDBusImplPortalAccessSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_impl_portal_access_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusImplPortalAccessSkeleton, g_object_unref)
#endif

PhoshDBusImplPortalAccess *phosh_dbus_impl_portal_access_skeleton_new (void);


G_END_DECLS

#endif /* __PORTAL_DBUS_H__ */
