/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __PHOSH_DBUS_DEBUG_CONTROL_H__
#define __PHOSH_DBUS_DEBUG_CONTROL_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for mobi.phosh.Shell.DebugControl */

#define PHOSH_DBUS_TYPE_DEBUG_CONTROL (phosh_dbus_debug_control_get_type ())
#define PHOSH_DBUS_DEBUG_CONTROL(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_DEBUG_CONTROL, PhoshDBusDebugControl))
#define PHOSH_DBUS_IS_DEBUG_CONTROL(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_DEBUG_CONTROL))
#define PHOSH_DBUS_DEBUG_CONTROL_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_DEBUG_CONTROL, PhoshDBusDebugControlIface))

struct _PhoshDBusDebugControl;
typedef struct _PhoshDBusDebugControl PhoshDBusDebugControl;
typedef struct _PhoshDBusDebugControlIface PhoshDBusDebugControlIface;

struct _PhoshDBusDebugControlIface
{
  GTypeInterface parent_iface;

  const gchar *const * (*get_log_domains) (PhoshDBusDebugControl *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusDebugControl, g_object_unref)
#endif

GType phosh_dbus_debug_control_get_type (void);

GDBusInterfaceInfo *phosh_dbus_debug_control_interface_info (void);
guint phosh_dbus_debug_control_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus property accessors: */
const gchar *const *phosh_dbus_debug_control_get_log_domains (PhoshDBusDebugControl *object);
gchar **phosh_dbus_debug_control_dup_log_domains (PhoshDBusDebugControl *object);
void phosh_dbus_debug_control_set_log_domains (PhoshDBusDebugControl *object, const gchar *const *value);


/* ---- */

#define PHOSH_DBUS_TYPE_DEBUG_CONTROL_PROXY (phosh_dbus_debug_control_proxy_get_type ())
#define PHOSH_DBUS_DEBUG_CONTROL_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_DEBUG_CONTROL_PROXY, PhoshDBusDebugControlProxy))
#define PHOSH_DBUS_DEBUG_CONTROL_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_DEBUG_CONTROL_PROXY, PhoshDBusDebugControlProxyClass))
#define PHOSH_DBUS_DEBUG_CONTROL_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_DEBUG_CONTROL_PROXY, PhoshDBusDebugControlProxyClass))
#define PHOSH_DBUS_IS_DEBUG_CONTROL_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_DEBUG_CONTROL_PROXY))
#define PHOSH_DBUS_IS_DEBUG_CONTROL_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_DEBUG_CONTROL_PROXY))

typedef struct _PhoshDBusDebugControlProxy PhoshDBusDebugControlProxy;
typedef struct _PhoshDBusDebugControlProxyClass PhoshDBusDebugControlProxyClass;
typedef struct _PhoshDBusDebugControlProxyPrivate PhoshDBusDebugControlProxyPrivate;

struct _PhoshDBusDebugControlProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusDebugControlProxyPrivate *priv;
};

struct _PhoshDBusDebugControlProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_debug_control_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusDebugControlProxy, g_object_unref)
#endif

void phosh_dbus_debug_control_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusDebugControl *phosh_dbus_debug_control_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusDebugControl *phosh_dbus_debug_control_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_debug_control_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusDebugControl *phosh_dbus_debug_control_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusDebugControl *phosh_dbus_debug_control_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_DEBUG_CONTROL_SKELETON (phosh_dbus_debug_control_skeleton_get_type ())
#define PHOSH_DBUS_DEBUG_CONTROL_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_DEBUG_CONTROL_SKELETON, PhoshDBusDebugControlSkeleton))
#define PHOSH_DBUS_DEBUG_CONTROL_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_DEBUG_CONTROL_SKELETON, PhoshDBusDebugControlSkeletonClass))
#define PHOSH_DBUS_DEBUG_CONTROL_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_DEBUG_CONTROL_SKELETON, PhoshDBusDebugControlSkeletonClass))
#define PHOSH_DBUS_IS_DEBUG_CONTROL_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_DEBUG_CONTROL_SKELETON))
#define PHOSH_DBUS_IS_DEBUG_CONTROL_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_DEBUG_CONTROL_SKELETON))

typedef struct _PhoshDBusDebugControlSkeleton PhoshDBusDebugControlSkeleton;
typedef struct _PhoshDBusDebugControlSkeletonClass PhoshDBusDebugControlSkeletonClass;
typedef struct _PhoshDBusDebugControlSkeletonPrivate PhoshDBusDebugControlSkeletonPrivate;

struct _PhoshDBusDebugControlSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusDebugControlSkeletonPrivate *priv;
};

struct _PhoshDBusDebugControlSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_debug_control_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusDebugControlSkeleton, g_object_unref)
#endif

PhoshDBusDebugControl *phosh_dbus_debug_control_skeleton_new (void);


G_END_DECLS

#endif /* __PHOSH_DBUS_DEBUG_CONTROL_H__ */
