/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __SYNCBUS_H__
#define __SYNCBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for mobi.phosh.syncbus.Folder */

#define SYNCBUS_IMPL_TYPE_FOLDER (syncbus_impl_folder_get_type ())
#define SYNCBUS_IMPL_FOLDER(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), SYNCBUS_IMPL_TYPE_FOLDER, SyncbusImplFolder))
#define SYNCBUS_IMPL_IS_FOLDER(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), SYNCBUS_IMPL_TYPE_FOLDER))
#define SYNCBUS_IMPL_FOLDER_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), SYNCBUS_IMPL_TYPE_FOLDER, SyncbusImplFolderIface))

struct _SyncbusImplFolder;
typedef struct _SyncbusImplFolder SyncbusImplFolder;
typedef struct _SyncbusImplFolderIface SyncbusImplFolderIface;

struct _SyncbusImplFolderIface
{
  GTypeInterface parent_iface;

  guchar  (*get_completion) (SyncbusImplFolder *object);

  const gchar * (*get_id) (SyncbusImplFolder *object);

  const gchar * (*get_label) (SyncbusImplFolder *object);

  const gchar * (*get_path) (SyncbusImplFolder *object);

  gboolean  (*get_paused) (SyncbusImplFolder *object);

  const gchar * (*get_state) (SyncbusImplFolder *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (SyncbusImplFolder, g_object_unref)
#endif

GType syncbus_impl_folder_get_type (void);

GDBusInterfaceInfo *syncbus_impl_folder_interface_info (void);
guint syncbus_impl_folder_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus property accessors: */
const gchar *syncbus_impl_folder_get_id (SyncbusImplFolder *object);
gchar *syncbus_impl_folder_dup_id (SyncbusImplFolder *object);
void syncbus_impl_folder_set_id (SyncbusImplFolder *object, const gchar *value);

const gchar *syncbus_impl_folder_get_label (SyncbusImplFolder *object);
gchar *syncbus_impl_folder_dup_label (SyncbusImplFolder *object);
void syncbus_impl_folder_set_label (SyncbusImplFolder *object, const gchar *value);

const gchar *syncbus_impl_folder_get_path (SyncbusImplFolder *object);
gchar *syncbus_impl_folder_dup_path (SyncbusImplFolder *object);
void syncbus_impl_folder_set_path (SyncbusImplFolder *object, const gchar *value);

guchar syncbus_impl_folder_get_completion (SyncbusImplFolder *object);
void syncbus_impl_folder_set_completion (SyncbusImplFolder *object, guchar value);

const gchar *syncbus_impl_folder_get_state (SyncbusImplFolder *object);
gchar *syncbus_impl_folder_dup_state (SyncbusImplFolder *object);
void syncbus_impl_folder_set_state (SyncbusImplFolder *object, const gchar *value);

gboolean syncbus_impl_folder_get_paused (SyncbusImplFolder *object);
void syncbus_impl_folder_set_paused (SyncbusImplFolder *object, gboolean value);


/* ---- */

#define SYNCBUS_IMPL_TYPE_FOLDER_PROXY (syncbus_impl_folder_proxy_get_type ())
#define SYNCBUS_IMPL_FOLDER_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), SYNCBUS_IMPL_TYPE_FOLDER_PROXY, SyncbusImplFolderProxy))
#define SYNCBUS_IMPL_FOLDER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), SYNCBUS_IMPL_TYPE_FOLDER_PROXY, SyncbusImplFolderProxyClass))
#define SYNCBUS_IMPL_FOLDER_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), SYNCBUS_IMPL_TYPE_FOLDER_PROXY, SyncbusImplFolderProxyClass))
#define SYNCBUS_IMPL_IS_FOLDER_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), SYNCBUS_IMPL_TYPE_FOLDER_PROXY))
#define SYNCBUS_IMPL_IS_FOLDER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), SYNCBUS_IMPL_TYPE_FOLDER_PROXY))

typedef struct _SyncbusImplFolderProxy SyncbusImplFolderProxy;
typedef struct _SyncbusImplFolderProxyClass SyncbusImplFolderProxyClass;
typedef struct _SyncbusImplFolderProxyPrivate SyncbusImplFolderProxyPrivate;

struct _SyncbusImplFolderProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  SyncbusImplFolderProxyPrivate *priv;
};

struct _SyncbusImplFolderProxyClass
{
  GDBusProxyClass parent_class;
};

GType syncbus_impl_folder_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (SyncbusImplFolderProxy, g_object_unref)
#endif

void syncbus_impl_folder_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
SyncbusImplFolder *syncbus_impl_folder_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
SyncbusImplFolder *syncbus_impl_folder_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void syncbus_impl_folder_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
SyncbusImplFolder *syncbus_impl_folder_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
SyncbusImplFolder *syncbus_impl_folder_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define SYNCBUS_IMPL_TYPE_FOLDER_SKELETON (syncbus_impl_folder_skeleton_get_type ())
#define SYNCBUS_IMPL_FOLDER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), SYNCBUS_IMPL_TYPE_FOLDER_SKELETON, SyncbusImplFolderSkeleton))
#define SYNCBUS_IMPL_FOLDER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), SYNCBUS_IMPL_TYPE_FOLDER_SKELETON, SyncbusImplFolderSkeletonClass))
#define SYNCBUS_IMPL_FOLDER_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), SYNCBUS_IMPL_TYPE_FOLDER_SKELETON, SyncbusImplFolderSkeletonClass))
#define SYNCBUS_IMPL_IS_FOLDER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), SYNCBUS_IMPL_TYPE_FOLDER_SKELETON))
#define SYNCBUS_IMPL_IS_FOLDER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), SYNCBUS_IMPL_TYPE_FOLDER_SKELETON))

typedef struct _SyncbusImplFolderSkeleton SyncbusImplFolderSkeleton;
typedef struct _SyncbusImplFolderSkeletonClass SyncbusImplFolderSkeletonClass;
typedef struct _SyncbusImplFolderSkeletonPrivate SyncbusImplFolderSkeletonPrivate;

struct _SyncbusImplFolderSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  SyncbusImplFolderSkeletonPrivate *priv;
};

struct _SyncbusImplFolderSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType syncbus_impl_folder_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (SyncbusImplFolderSkeleton, g_object_unref)
#endif

SyncbusImplFolder *syncbus_impl_folder_skeleton_new (void);


/* ------------------------------------------------------------------------ */
/* Declarations for mobi.phosh.syncbus.Manager */

#define SYNCBUS_IMPL_TYPE_MANAGER (syncbus_impl_manager_get_type ())
#define SYNCBUS_IMPL_MANAGER(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), SYNCBUS_IMPL_TYPE_MANAGER, SyncbusImplManager))
#define SYNCBUS_IMPL_IS_MANAGER(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), SYNCBUS_IMPL_TYPE_MANAGER))
#define SYNCBUS_IMPL_MANAGER_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), SYNCBUS_IMPL_TYPE_MANAGER, SyncbusImplManagerIface))

struct _SyncbusImplManager;
typedef struct _SyncbusImplManager SyncbusImplManager;
typedef struct _SyncbusImplManagerIface SyncbusImplManagerIface;

struct _SyncbusImplManagerIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_start) (
    SyncbusImplManager *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_stop) (
    SyncbusImplManager *object,
    GDBusMethodInvocation *invocation);

  gboolean  (*get_enabled) (SyncbusImplManager *object);

  const gchar * (*get_error) (SyncbusImplManager *object);

  const gchar * (*get_url) (SyncbusImplManager *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (SyncbusImplManager, g_object_unref)
#endif

GType syncbus_impl_manager_get_type (void);

GDBusInterfaceInfo *syncbus_impl_manager_interface_info (void);
guint syncbus_impl_manager_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void syncbus_impl_manager_complete_start (
    SyncbusImplManager *object,
    GDBusMethodInvocation *invocation);

void syncbus_impl_manager_complete_stop (
    SyncbusImplManager *object,
    GDBusMethodInvocation *invocation);



/* D-Bus method calls: */
void syncbus_impl_manager_call_start (
    SyncbusImplManager *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean syncbus_impl_manager_call_start_finish (
    SyncbusImplManager *proxy,
    GAsyncResult *res,
    GError **error);

gboolean syncbus_impl_manager_call_start_sync (
    SyncbusImplManager *proxy,
    GCancellable *cancellable,
    GError **error);

void syncbus_impl_manager_call_stop (
    SyncbusImplManager *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean syncbus_impl_manager_call_stop_finish (
    SyncbusImplManager *proxy,
    GAsyncResult *res,
    GError **error);

gboolean syncbus_impl_manager_call_stop_sync (
    SyncbusImplManager *proxy,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
gboolean syncbus_impl_manager_get_enabled (SyncbusImplManager *object);
void syncbus_impl_manager_set_enabled (SyncbusImplManager *object, gboolean value);

const gchar *syncbus_impl_manager_get_error (SyncbusImplManager *object);
gchar *syncbus_impl_manager_dup_error (SyncbusImplManager *object);
void syncbus_impl_manager_set_error (SyncbusImplManager *object, const gchar *value);

const gchar *syncbus_impl_manager_get_url (SyncbusImplManager *object);
gchar *syncbus_impl_manager_dup_url (SyncbusImplManager *object);
void syncbus_impl_manager_set_url (SyncbusImplManager *object, const gchar *value);


/* ---- */

#define SYNCBUS_IMPL_TYPE_MANAGER_PROXY (syncbus_impl_manager_proxy_get_type ())
#define SYNCBUS_IMPL_MANAGER_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), SYNCBUS_IMPL_TYPE_MANAGER_PROXY, SyncbusImplManagerProxy))
#define SYNCBUS_IMPL_MANAGER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), SYNCBUS_IMPL_TYPE_MANAGER_PROXY, SyncbusImplManagerProxyClass))
#define SYNCBUS_IMPL_MANAGER_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), SYNCBUS_IMPL_TYPE_MANAGER_PROXY, SyncbusImplManagerProxyClass))
#define SYNCBUS_IMPL_IS_MANAGER_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), SYNCBUS_IMPL_TYPE_MANAGER_PROXY))
#define SYNCBUS_IMPL_IS_MANAGER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), SYNCBUS_IMPL_TYPE_MANAGER_PROXY))

typedef struct _SyncbusImplManagerProxy SyncbusImplManagerProxy;
typedef struct _SyncbusImplManagerProxyClass SyncbusImplManagerProxyClass;
typedef struct _SyncbusImplManagerProxyPrivate SyncbusImplManagerProxyPrivate;

struct _SyncbusImplManagerProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  SyncbusImplManagerProxyPrivate *priv;
};

struct _SyncbusImplManagerProxyClass
{
  GDBusProxyClass parent_class;
};

GType syncbus_impl_manager_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (SyncbusImplManagerProxy, g_object_unref)
#endif

void syncbus_impl_manager_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
SyncbusImplManager *syncbus_impl_manager_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
SyncbusImplManager *syncbus_impl_manager_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void syncbus_impl_manager_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
SyncbusImplManager *syncbus_impl_manager_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
SyncbusImplManager *syncbus_impl_manager_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define SYNCBUS_IMPL_TYPE_MANAGER_SKELETON (syncbus_impl_manager_skeleton_get_type ())
#define SYNCBUS_IMPL_MANAGER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), SYNCBUS_IMPL_TYPE_MANAGER_SKELETON, SyncbusImplManagerSkeleton))
#define SYNCBUS_IMPL_MANAGER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), SYNCBUS_IMPL_TYPE_MANAGER_SKELETON, SyncbusImplManagerSkeletonClass))
#define SYNCBUS_IMPL_MANAGER_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), SYNCBUS_IMPL_TYPE_MANAGER_SKELETON, SyncbusImplManagerSkeletonClass))
#define SYNCBUS_IMPL_IS_MANAGER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), SYNCBUS_IMPL_TYPE_MANAGER_SKELETON))
#define SYNCBUS_IMPL_IS_MANAGER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), SYNCBUS_IMPL_TYPE_MANAGER_SKELETON))

typedef struct _SyncbusImplManagerSkeleton SyncbusImplManagerSkeleton;
typedef struct _SyncbusImplManagerSkeletonClass SyncbusImplManagerSkeletonClass;
typedef struct _SyncbusImplManagerSkeletonPrivate SyncbusImplManagerSkeletonPrivate;

struct _SyncbusImplManagerSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  SyncbusImplManagerSkeletonPrivate *priv;
};

struct _SyncbusImplManagerSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType syncbus_impl_manager_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (SyncbusImplManagerSkeleton, g_object_unref)
#endif

SyncbusImplManager *syncbus_impl_manager_skeleton_new (void);


/* ---- */

#define SYNCBUS_IMPL_TYPE_OBJECT (syncbus_impl_object_get_type ())
#define SYNCBUS_IMPL_OBJECT(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), SYNCBUS_IMPL_TYPE_OBJECT, SyncbusImplObject))
#define SYNCBUS_IMPL_IS_OBJECT(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), SYNCBUS_IMPL_TYPE_OBJECT))
#define SYNCBUS_IMPL_OBJECT_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), SYNCBUS_IMPL_TYPE_OBJECT, SyncbusImplObject))

struct _SyncbusImplObject;
typedef struct _SyncbusImplObject SyncbusImplObject;
typedef struct _SyncbusImplObjectIface SyncbusImplObjectIface;

struct _SyncbusImplObjectIface
{
  GTypeInterface parent_iface;
};

GType syncbus_impl_object_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (SyncbusImplObject, g_object_unref)
#endif

SyncbusImplFolder *syncbus_impl_object_get_folder (SyncbusImplObject *object);
SyncbusImplManager *syncbus_impl_object_get_manager (SyncbusImplObject *object);
SyncbusImplFolder *syncbus_impl_object_peek_folder (SyncbusImplObject *object);
SyncbusImplManager *syncbus_impl_object_peek_manager (SyncbusImplObject *object);

#define SYNCBUS_IMPL_TYPE_OBJECT_PROXY (syncbus_impl_object_proxy_get_type ())
#define SYNCBUS_IMPL_OBJECT_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), SYNCBUS_IMPL_TYPE_OBJECT_PROXY, SyncbusImplObjectProxy))
#define SYNCBUS_IMPL_OBJECT_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), SYNCBUS_IMPL_TYPE_OBJECT_PROXY, SyncbusImplObjectProxyClass))
#define SYNCBUS_IMPL_OBJECT_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), SYNCBUS_IMPL_TYPE_OBJECT_PROXY, SyncbusImplObjectProxyClass))
#define SYNCBUS_IMPL_IS_OBJECT_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), SYNCBUS_IMPL_TYPE_OBJECT_PROXY))
#define SYNCBUS_IMPL_IS_OBJECT_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), SYNCBUS_IMPL_TYPE_OBJECT_PROXY))

typedef struct _SyncbusImplObjectProxy SyncbusImplObjectProxy;
typedef struct _SyncbusImplObjectProxyClass SyncbusImplObjectProxyClass;
typedef struct _SyncbusImplObjectProxyPrivate SyncbusImplObjectProxyPrivate;

struct _SyncbusImplObjectProxy
{
  /*< private >*/
  GDBusObjectProxy parent_instance;
  SyncbusImplObjectProxyPrivate *priv;
};

struct _SyncbusImplObjectProxyClass
{
  GDBusObjectProxyClass parent_class;
};

GType syncbus_impl_object_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (SyncbusImplObjectProxy, g_object_unref)
#endif

SyncbusImplObjectProxy *syncbus_impl_object_proxy_new (GDBusConnection *connection, const gchar *object_path);

#define SYNCBUS_IMPL_TYPE_OBJECT_SKELETON (syncbus_impl_object_skeleton_get_type ())
#define SYNCBUS_IMPL_OBJECT_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), SYNCBUS_IMPL_TYPE_OBJECT_SKELETON, SyncbusImplObjectSkeleton))
#define SYNCBUS_IMPL_OBJECT_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), SYNCBUS_IMPL_TYPE_OBJECT_SKELETON, SyncbusImplObjectSkeletonClass))
#define SYNCBUS_IMPL_OBJECT_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), SYNCBUS_IMPL_TYPE_OBJECT_SKELETON, SyncbusImplObjectSkeletonClass))
#define SYNCBUS_IMPL_IS_OBJECT_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), SYNCBUS_IMPL_TYPE_OBJECT_SKELETON))
#define SYNCBUS_IMPL_IS_OBJECT_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), SYNCBUS_IMPL_TYPE_OBJECT_SKELETON))

typedef struct _SyncbusImplObjectSkeleton SyncbusImplObjectSkeleton;
typedef struct _SyncbusImplObjectSkeletonClass SyncbusImplObjectSkeletonClass;
typedef struct _SyncbusImplObjectSkeletonPrivate SyncbusImplObjectSkeletonPrivate;

struct _SyncbusImplObjectSkeleton
{
  /*< private >*/
  GDBusObjectSkeleton parent_instance;
  SyncbusImplObjectSkeletonPrivate *priv;
};

struct _SyncbusImplObjectSkeletonClass
{
  GDBusObjectSkeletonClass parent_class;
};

GType syncbus_impl_object_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (SyncbusImplObjectSkeleton, g_object_unref)
#endif

SyncbusImplObjectSkeleton *syncbus_impl_object_skeleton_new (const gchar *object_path);
void syncbus_impl_object_skeleton_set_folder (SyncbusImplObjectSkeleton *object, SyncbusImplFolder *interface_);
void syncbus_impl_object_skeleton_set_manager (SyncbusImplObjectSkeleton *object, SyncbusImplManager *interface_);

/* ---- */

#define SYNCBUS_IMPL_TYPE_OBJECT_MANAGER_CLIENT (syncbus_impl_object_manager_client_get_type ())
#define SYNCBUS_IMPL_OBJECT_MANAGER_CLIENT(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), SYNCBUS_IMPL_TYPE_OBJECT_MANAGER_CLIENT, SyncbusImplObjectManagerClient))
#define SYNCBUS_IMPL_OBJECT_MANAGER_CLIENT_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), SYNCBUS_IMPL_TYPE_OBJECT_MANAGER_CLIENT, SyncbusImplObjectManagerClientClass))
#define SYNCBUS_IMPL_OBJECT_MANAGER_CLIENT_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), SYNCBUS_IMPL_TYPE_OBJECT_MANAGER_CLIENT, SyncbusImplObjectManagerClientClass))
#define SYNCBUS_IMPL_IS_OBJECT_MANAGER_CLIENT(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), SYNCBUS_IMPL_TYPE_OBJECT_MANAGER_CLIENT))
#define SYNCBUS_IMPL_IS_OBJECT_MANAGER_CLIENT_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), SYNCBUS_IMPL_TYPE_OBJECT_MANAGER_CLIENT))

typedef struct _SyncbusImplObjectManagerClient SyncbusImplObjectManagerClient;
typedef struct _SyncbusImplObjectManagerClientClass SyncbusImplObjectManagerClientClass;
typedef struct _SyncbusImplObjectManagerClientPrivate SyncbusImplObjectManagerClientPrivate;

struct _SyncbusImplObjectManagerClient
{
  /*< private >*/
  GDBusObjectManagerClient parent_instance;
  SyncbusImplObjectManagerClientPrivate *priv;
};

struct _SyncbusImplObjectManagerClientClass
{
  GDBusObjectManagerClientClass parent_class;
};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (SyncbusImplObjectManagerClient, g_object_unref)
#endif

GType syncbus_impl_object_manager_client_get_type (void);

GType syncbus_impl_object_manager_client_get_proxy_type (GDBusObjectManagerClient *manager, const gchar *object_path, const gchar *interface_name, gpointer user_data);

void syncbus_impl_object_manager_client_new (
    GDBusConnection        *connection,
    GDBusObjectManagerClientFlags  flags,
    const gchar            *name,
    const gchar            *object_path,
    GCancellable           *cancellable,
    GAsyncReadyCallback     callback,
    gpointer                user_data);
GDBusObjectManager *syncbus_impl_object_manager_client_new_finish (
    GAsyncResult        *res,
    GError             **error);
GDBusObjectManager *syncbus_impl_object_manager_client_new_sync (
    GDBusConnection        *connection,
    GDBusObjectManagerClientFlags  flags,
    const gchar            *name,
    const gchar            *object_path,
    GCancellable           *cancellable,
    GError                **error);

void syncbus_impl_object_manager_client_new_for_bus (
    GBusType                bus_type,
    GDBusObjectManagerClientFlags  flags,
    const gchar            *name,
    const gchar            *object_path,
    GCancellable           *cancellable,
    GAsyncReadyCallback     callback,
    gpointer                user_data);
GDBusObjectManager *syncbus_impl_object_manager_client_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
GDBusObjectManager *syncbus_impl_object_manager_client_new_for_bus_sync (
    GBusType                bus_type,
    GDBusObjectManagerClientFlags  flags,
    const gchar            *name,
    const gchar            *object_path,
    GCancellable           *cancellable,
    GError                **error);


G_END_DECLS

#endif /* __SYNCBUS_H__ */
