/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __IIO_SENSOR_PROXY_DBUS_H__
#define __IIO_SENSOR_PROXY_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for net.hadess.SensorProxy */

#define PHOSH_DBUS_TYPE_SENSOR_PROXY (phosh_dbus_sensor_proxy_get_type ())
#define PHOSH_DBUS_SENSOR_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_SENSOR_PROXY, PhoshDBusSensorProxy))
#define PHOSH_DBUS_IS_SENSOR_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_SENSOR_PROXY))
#define PHOSH_DBUS_SENSOR_PROXY_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_SENSOR_PROXY, PhoshDBusSensorProxyIface))

struct _PhoshDBusSensorProxy;
typedef struct _PhoshDBusSensorProxy PhoshDBusSensorProxy;
typedef struct _PhoshDBusSensorProxyIface PhoshDBusSensorProxyIface;

struct _PhoshDBusSensorProxyIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_claim_accelerometer) (
    PhoshDBusSensorProxy *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_claim_light) (
    PhoshDBusSensorProxy *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_claim_proximity) (
    PhoshDBusSensorProxy *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_release_accelerometer) (
    PhoshDBusSensorProxy *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_release_light) (
    PhoshDBusSensorProxy *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_release_proximity) (
    PhoshDBusSensorProxy *object,
    GDBusMethodInvocation *invocation);

  const gchar * (*get_accelerometer_orientation) (PhoshDBusSensorProxy *object);

  gboolean  (*get_has_accelerometer) (PhoshDBusSensorProxy *object);

  gboolean  (*get_has_ambient_light) (PhoshDBusSensorProxy *object);

  gboolean  (*get_has_proximity) (PhoshDBusSensorProxy *object);

  gdouble  (*get_light_level) (PhoshDBusSensorProxy *object);

  const gchar * (*get_light_level_unit) (PhoshDBusSensorProxy *object);

  gboolean  (*get_proximity_near) (PhoshDBusSensorProxy *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusSensorProxy, g_object_unref)
#endif

GType phosh_dbus_sensor_proxy_get_type (void);

GDBusInterfaceInfo *phosh_dbus_sensor_proxy_interface_info (void);
guint phosh_dbus_sensor_proxy_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void phosh_dbus_sensor_proxy_complete_claim_accelerometer (
    PhoshDBusSensorProxy *object,
    GDBusMethodInvocation *invocation);

void phosh_dbus_sensor_proxy_complete_release_accelerometer (
    PhoshDBusSensorProxy *object,
    GDBusMethodInvocation *invocation);

void phosh_dbus_sensor_proxy_complete_claim_light (
    PhoshDBusSensorProxy *object,
    GDBusMethodInvocation *invocation);

void phosh_dbus_sensor_proxy_complete_release_light (
    PhoshDBusSensorProxy *object,
    GDBusMethodInvocation *invocation);

void phosh_dbus_sensor_proxy_complete_claim_proximity (
    PhoshDBusSensorProxy *object,
    GDBusMethodInvocation *invocation);

void phosh_dbus_sensor_proxy_complete_release_proximity (
    PhoshDBusSensorProxy *object,
    GDBusMethodInvocation *invocation);



/* D-Bus method calls: */
void phosh_dbus_sensor_proxy_call_claim_accelerometer (
    PhoshDBusSensorProxy *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_sensor_proxy_call_claim_accelerometer_finish (
    PhoshDBusSensorProxy *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_sensor_proxy_call_claim_accelerometer_sync (
    PhoshDBusSensorProxy *proxy,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_sensor_proxy_call_release_accelerometer (
    PhoshDBusSensorProxy *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_sensor_proxy_call_release_accelerometer_finish (
    PhoshDBusSensorProxy *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_sensor_proxy_call_release_accelerometer_sync (
    PhoshDBusSensorProxy *proxy,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_sensor_proxy_call_claim_light (
    PhoshDBusSensorProxy *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_sensor_proxy_call_claim_light_finish (
    PhoshDBusSensorProxy *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_sensor_proxy_call_claim_light_sync (
    PhoshDBusSensorProxy *proxy,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_sensor_proxy_call_release_light (
    PhoshDBusSensorProxy *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_sensor_proxy_call_release_light_finish (
    PhoshDBusSensorProxy *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_sensor_proxy_call_release_light_sync (
    PhoshDBusSensorProxy *proxy,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_sensor_proxy_call_claim_proximity (
    PhoshDBusSensorProxy *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_sensor_proxy_call_claim_proximity_finish (
    PhoshDBusSensorProxy *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_sensor_proxy_call_claim_proximity_sync (
    PhoshDBusSensorProxy *proxy,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_sensor_proxy_call_release_proximity (
    PhoshDBusSensorProxy *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_sensor_proxy_call_release_proximity_finish (
    PhoshDBusSensorProxy *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_sensor_proxy_call_release_proximity_sync (
    PhoshDBusSensorProxy *proxy,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
gboolean phosh_dbus_sensor_proxy_get_has_accelerometer (PhoshDBusSensorProxy *object);
void phosh_dbus_sensor_proxy_set_has_accelerometer (PhoshDBusSensorProxy *object, gboolean value);

const gchar *phosh_dbus_sensor_proxy_get_accelerometer_orientation (PhoshDBusSensorProxy *object);
gchar *phosh_dbus_sensor_proxy_dup_accelerometer_orientation (PhoshDBusSensorProxy *object);
void phosh_dbus_sensor_proxy_set_accelerometer_orientation (PhoshDBusSensorProxy *object, const gchar *value);

gboolean phosh_dbus_sensor_proxy_get_has_ambient_light (PhoshDBusSensorProxy *object);
void phosh_dbus_sensor_proxy_set_has_ambient_light (PhoshDBusSensorProxy *object, gboolean value);

const gchar *phosh_dbus_sensor_proxy_get_light_level_unit (PhoshDBusSensorProxy *object);
gchar *phosh_dbus_sensor_proxy_dup_light_level_unit (PhoshDBusSensorProxy *object);
void phosh_dbus_sensor_proxy_set_light_level_unit (PhoshDBusSensorProxy *object, const gchar *value);

gdouble phosh_dbus_sensor_proxy_get_light_level (PhoshDBusSensorProxy *object);
void phosh_dbus_sensor_proxy_set_light_level (PhoshDBusSensorProxy *object, gdouble value);

gboolean phosh_dbus_sensor_proxy_get_has_proximity (PhoshDBusSensorProxy *object);
void phosh_dbus_sensor_proxy_set_has_proximity (PhoshDBusSensorProxy *object, gboolean value);

gboolean phosh_dbus_sensor_proxy_get_proximity_near (PhoshDBusSensorProxy *object);
void phosh_dbus_sensor_proxy_set_proximity_near (PhoshDBusSensorProxy *object, gboolean value);


/* ---- */

#define PHOSH_DBUS_TYPE_SENSOR_PROXY_PROXY (phosh_dbus_sensor_proxy_proxy_get_type ())
#define PHOSH_DBUS_SENSOR_PROXY_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_SENSOR_PROXY_PROXY, PhoshDBusSensorProxyProxy))
#define PHOSH_DBUS_SENSOR_PROXY_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_SENSOR_PROXY_PROXY, PhoshDBusSensorProxyProxyClass))
#define PHOSH_DBUS_SENSOR_PROXY_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_SENSOR_PROXY_PROXY, PhoshDBusSensorProxyProxyClass))
#define PHOSH_DBUS_IS_SENSOR_PROXY_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_SENSOR_PROXY_PROXY))
#define PHOSH_DBUS_IS_SENSOR_PROXY_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_SENSOR_PROXY_PROXY))

typedef struct _PhoshDBusSensorProxyProxy PhoshDBusSensorProxyProxy;
typedef struct _PhoshDBusSensorProxyProxyClass PhoshDBusSensorProxyProxyClass;
typedef struct _PhoshDBusSensorProxyProxyPrivate PhoshDBusSensorProxyProxyPrivate;

struct _PhoshDBusSensorProxyProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusSensorProxyProxyPrivate *priv;
};

struct _PhoshDBusSensorProxyProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_sensor_proxy_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusSensorProxyProxy, g_object_unref)
#endif

void phosh_dbus_sensor_proxy_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusSensorProxy *phosh_dbus_sensor_proxy_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusSensorProxy *phosh_dbus_sensor_proxy_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_sensor_proxy_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusSensorProxy *phosh_dbus_sensor_proxy_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusSensorProxy *phosh_dbus_sensor_proxy_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_SENSOR_PROXY_SKELETON (phosh_dbus_sensor_proxy_skeleton_get_type ())
#define PHOSH_DBUS_SENSOR_PROXY_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_SENSOR_PROXY_SKELETON, PhoshDBusSensorProxySkeleton))
#define PHOSH_DBUS_SENSOR_PROXY_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_SENSOR_PROXY_SKELETON, PhoshDBusSensorProxySkeletonClass))
#define PHOSH_DBUS_SENSOR_PROXY_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_SENSOR_PROXY_SKELETON, PhoshDBusSensorProxySkeletonClass))
#define PHOSH_DBUS_IS_SENSOR_PROXY_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_SENSOR_PROXY_SKELETON))
#define PHOSH_DBUS_IS_SENSOR_PROXY_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_SENSOR_PROXY_SKELETON))

typedef struct _PhoshDBusSensorProxySkeleton PhoshDBusSensorProxySkeleton;
typedef struct _PhoshDBusSensorProxySkeletonClass PhoshDBusSensorProxySkeletonClass;
typedef struct _PhoshDBusSensorProxySkeletonPrivate PhoshDBusSensorProxySkeletonPrivate;

struct _PhoshDBusSensorProxySkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusSensorProxySkeletonPrivate *priv;
};

struct _PhoshDBusSensorProxySkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_sensor_proxy_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusSensorProxySkeleton, g_object_unref)
#endif

PhoshDBusSensorProxy *phosh_dbus_sensor_proxy_skeleton_new (void);


/* ------------------------------------------------------------------------ */
/* Declarations for net.hadess.SensorProxy.Compass */

#define PHOSH_DBUS_TYPE_SENSOR_PROXY_COMPASS (phosh_dbus_sensor_proxy_compass_get_type ())
#define PHOSH_DBUS_SENSOR_PROXY_COMPASS(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_SENSOR_PROXY_COMPASS, PhoshDBusSensorProxyCompass))
#define PHOSH_DBUS_IS_SENSOR_PROXY_COMPASS(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_SENSOR_PROXY_COMPASS))
#define PHOSH_DBUS_SENSOR_PROXY_COMPASS_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_SENSOR_PROXY_COMPASS, PhoshDBusSensorProxyCompassIface))

struct _PhoshDBusSensorProxyCompass;
typedef struct _PhoshDBusSensorProxyCompass PhoshDBusSensorProxyCompass;
typedef struct _PhoshDBusSensorProxyCompassIface PhoshDBusSensorProxyCompassIface;

struct _PhoshDBusSensorProxyCompassIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_claim_compass) (
    PhoshDBusSensorProxyCompass *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_release_compass) (
    PhoshDBusSensorProxyCompass *object,
    GDBusMethodInvocation *invocation);

  gdouble  (*get_compass_heading) (PhoshDBusSensorProxyCompass *object);

  gboolean  (*get_has_compass) (PhoshDBusSensorProxyCompass *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusSensorProxyCompass, g_object_unref)
#endif

GType phosh_dbus_sensor_proxy_compass_get_type (void);

GDBusInterfaceInfo *phosh_dbus_sensor_proxy_compass_interface_info (void);
guint phosh_dbus_sensor_proxy_compass_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void phosh_dbus_sensor_proxy_compass_complete_claim_compass (
    PhoshDBusSensorProxyCompass *object,
    GDBusMethodInvocation *invocation);

void phosh_dbus_sensor_proxy_compass_complete_release_compass (
    PhoshDBusSensorProxyCompass *object,
    GDBusMethodInvocation *invocation);



/* D-Bus method calls: */
void phosh_dbus_sensor_proxy_compass_call_claim_compass (
    PhoshDBusSensorProxyCompass *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_sensor_proxy_compass_call_claim_compass_finish (
    PhoshDBusSensorProxyCompass *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_sensor_proxy_compass_call_claim_compass_sync (
    PhoshDBusSensorProxyCompass *proxy,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_sensor_proxy_compass_call_release_compass (
    PhoshDBusSensorProxyCompass *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_sensor_proxy_compass_call_release_compass_finish (
    PhoshDBusSensorProxyCompass *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_sensor_proxy_compass_call_release_compass_sync (
    PhoshDBusSensorProxyCompass *proxy,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
gboolean phosh_dbus_sensor_proxy_compass_get_has_compass (PhoshDBusSensorProxyCompass *object);
void phosh_dbus_sensor_proxy_compass_set_has_compass (PhoshDBusSensorProxyCompass *object, gboolean value);

gdouble phosh_dbus_sensor_proxy_compass_get_compass_heading (PhoshDBusSensorProxyCompass *object);
void phosh_dbus_sensor_proxy_compass_set_compass_heading (PhoshDBusSensorProxyCompass *object, gdouble value);


/* ---- */

#define PHOSH_DBUS_TYPE_SENSOR_PROXY_COMPASS_PROXY (phosh_dbus_sensor_proxy_compass_proxy_get_type ())
#define PHOSH_DBUS_SENSOR_PROXY_COMPASS_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_SENSOR_PROXY_COMPASS_PROXY, PhoshDBusSensorProxyCompassProxy))
#define PHOSH_DBUS_SENSOR_PROXY_COMPASS_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_SENSOR_PROXY_COMPASS_PROXY, PhoshDBusSensorProxyCompassProxyClass))
#define PHOSH_DBUS_SENSOR_PROXY_COMPASS_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_SENSOR_PROXY_COMPASS_PROXY, PhoshDBusSensorProxyCompassProxyClass))
#define PHOSH_DBUS_IS_SENSOR_PROXY_COMPASS_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_SENSOR_PROXY_COMPASS_PROXY))
#define PHOSH_DBUS_IS_SENSOR_PROXY_COMPASS_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_SENSOR_PROXY_COMPASS_PROXY))

typedef struct _PhoshDBusSensorProxyCompassProxy PhoshDBusSensorProxyCompassProxy;
typedef struct _PhoshDBusSensorProxyCompassProxyClass PhoshDBusSensorProxyCompassProxyClass;
typedef struct _PhoshDBusSensorProxyCompassProxyPrivate PhoshDBusSensorProxyCompassProxyPrivate;

struct _PhoshDBusSensorProxyCompassProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusSensorProxyCompassProxyPrivate *priv;
};

struct _PhoshDBusSensorProxyCompassProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_sensor_proxy_compass_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusSensorProxyCompassProxy, g_object_unref)
#endif

void phosh_dbus_sensor_proxy_compass_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusSensorProxyCompass *phosh_dbus_sensor_proxy_compass_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusSensorProxyCompass *phosh_dbus_sensor_proxy_compass_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_sensor_proxy_compass_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusSensorProxyCompass *phosh_dbus_sensor_proxy_compass_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusSensorProxyCompass *phosh_dbus_sensor_proxy_compass_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_SENSOR_PROXY_COMPASS_SKELETON (phosh_dbus_sensor_proxy_compass_skeleton_get_type ())
#define PHOSH_DBUS_SENSOR_PROXY_COMPASS_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_SENSOR_PROXY_COMPASS_SKELETON, PhoshDBusSensorProxyCompassSkeleton))
#define PHOSH_DBUS_SENSOR_PROXY_COMPASS_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_SENSOR_PROXY_COMPASS_SKELETON, PhoshDBusSensorProxyCompassSkeletonClass))
#define PHOSH_DBUS_SENSOR_PROXY_COMPASS_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_SENSOR_PROXY_COMPASS_SKELETON, PhoshDBusSensorProxyCompassSkeletonClass))
#define PHOSH_DBUS_IS_SENSOR_PROXY_COMPASS_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_SENSOR_PROXY_COMPASS_SKELETON))
#define PHOSH_DBUS_IS_SENSOR_PROXY_COMPASS_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_SENSOR_PROXY_COMPASS_SKELETON))

typedef struct _PhoshDBusSensorProxyCompassSkeleton PhoshDBusSensorProxyCompassSkeleton;
typedef struct _PhoshDBusSensorProxyCompassSkeletonClass PhoshDBusSensorProxyCompassSkeletonClass;
typedef struct _PhoshDBusSensorProxyCompassSkeletonPrivate PhoshDBusSensorProxyCompassSkeletonPrivate;

struct _PhoshDBusSensorProxyCompassSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusSensorProxyCompassSkeletonPrivate *priv;
};

struct _PhoshDBusSensorProxyCompassSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_sensor_proxy_compass_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusSensorProxyCompassSkeleton, g_object_unref)
#endif

PhoshDBusSensorProxyCompass *phosh_dbus_sensor_proxy_compass_skeleton_new (void);


G_END_DECLS

#endif /* __IIO_SENSOR_PROXY_DBUS_H__ */
