/* Generated by wayland-scanner 1.26.0 */

/*
 * Copyright © 2022-2024 Guido Günther
 *
 * Permission to use, copy, modify, distribute, and sell this
 * software and its documentation for any purpose is hereby granted
 * without fee, provided that the above copyright notice appear in
 * all copies and that both that copyright notice and this permission
 * notice appear in supporting documentation, and that the name of
 * the copyright holders not be used in advertising or publicity
 * pertaining to distribution of the software without specific,
 * written prior permission.  The copyright holders make no
 * representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied
 * warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 */

#include <stdbool.h>
#include <stdlib.h>
#include <stdint.h>
#include "wayland-util.h"

#ifndef __has_attribute
# define __has_attribute(x) 0  /* Compatibility with non-clang compilers. */
#endif

#if (__has_attribute(visibility) || defined(__GNUC__) && __GNUC__ >= 4)
#define WL_PRIVATE __attribute__ ((visibility("hidden")))
#else
#define WL_PRIVATE
#endif

extern const struct wl_interface zphoc_alpha_layer_surface_v1_interface;
extern const struct wl_interface zphoc_draggable_layer_surface_v1_interface;
extern const struct wl_interface zphoc_stacked_layer_surface_v1_interface;
extern const struct wl_interface zwlr_layer_surface_v1_interface;

static const struct wl_interface *phoc_layer_surface_effects_unstable_v1_types[] = {
	NULL,
	NULL,
	&zphoc_draggable_layer_surface_v1_interface,
	&zwlr_layer_surface_v1_interface,
	&zphoc_alpha_layer_surface_v1_interface,
	&zwlr_layer_surface_v1_interface,
	&zphoc_stacked_layer_surface_v1_interface,
	&zwlr_layer_surface_v1_interface,
	&zwlr_layer_surface_v1_interface,
	&zwlr_layer_surface_v1_interface,
};

static const struct wl_message zphoc_layer_shell_effects_v1_requests[] = {
	{ "get_draggable_layer_surface", "no", phoc_layer_surface_effects_unstable_v1_types + 2 },
	{ "destroy", "", phoc_layer_surface_effects_unstable_v1_types + 0 },
	{ "get_alpha_layer_surface", "2no", phoc_layer_surface_effects_unstable_v1_types + 4 },
	{ "get_stacked_layer_surface", "3no", phoc_layer_surface_effects_unstable_v1_types + 6 },
};

WL_PRIVATE const struct wl_interface zphoc_layer_shell_effects_v1_interface = {
	"zphoc_layer_shell_effects_v1", 3,
	4, zphoc_layer_shell_effects_v1_requests,
	0, NULL,
};

static const struct wl_message zphoc_draggable_layer_surface_v1_requests[] = {
	{ "set_margins", "ii", phoc_layer_surface_effects_unstable_v1_types + 0 },
	{ "set_threshold", "f", phoc_layer_surface_effects_unstable_v1_types + 0 },
	{ "set_exclusive", "u", phoc_layer_surface_effects_unstable_v1_types + 0 },
	{ "set_drag_mode", "u", phoc_layer_surface_effects_unstable_v1_types + 0 },
	{ "set_drag_handle", "u", phoc_layer_surface_effects_unstable_v1_types + 0 },
	{ "set_state", "u", phoc_layer_surface_effects_unstable_v1_types + 0 },
	{ "destroy", "", phoc_layer_surface_effects_unstable_v1_types + 0 },
};

static const struct wl_message zphoc_draggable_layer_surface_v1_events[] = {
	{ "dragged", "i", phoc_layer_surface_effects_unstable_v1_types + 0 },
	{ "drag_end", "u", phoc_layer_surface_effects_unstable_v1_types + 0 },
};

WL_PRIVATE const struct wl_interface zphoc_draggable_layer_surface_v1_interface = {
	"zphoc_draggable_layer_surface_v1", 2,
	7, zphoc_draggable_layer_surface_v1_requests,
	2, zphoc_draggable_layer_surface_v1_events,
};

static const struct wl_message zphoc_alpha_layer_surface_v1_requests[] = {
	{ "set_alpha", "f", phoc_layer_surface_effects_unstable_v1_types + 0 },
	{ "destroy", "", phoc_layer_surface_effects_unstable_v1_types + 0 },
};

WL_PRIVATE const struct wl_interface zphoc_alpha_layer_surface_v1_interface = {
	"zphoc_alpha_layer_surface_v1", 2,
	2, zphoc_alpha_layer_surface_v1_requests,
	0, NULL,
};

static const struct wl_message zphoc_stacked_layer_surface_v1_requests[] = {
	{ "stack_below", "o", phoc_layer_surface_effects_unstable_v1_types + 8 },
	{ "stack_above", "o", phoc_layer_surface_effects_unstable_v1_types + 9 },
	{ "destroy", "", phoc_layer_surface_effects_unstable_v1_types + 0 },
};

WL_PRIVATE const struct wl_interface zphoc_stacked_layer_surface_v1_interface = {
	"zphoc_stacked_layer_surface_v1", 3,
	3, zphoc_stacked_layer_surface_v1_requests,
	0, NULL,
};

