/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __PHOSH_SCREENSHOT_DBUS_H__
#define __PHOSH_SCREENSHOT_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.Shell.Screenshot */

#define PHOSH_DBUS_TYPE_SCREENSHOT (phosh_dbus_screenshot_get_type ())
#define PHOSH_DBUS_SCREENSHOT(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_SCREENSHOT, PhoshDBusScreenshot))
#define PHOSH_DBUS_IS_SCREENSHOT(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_SCREENSHOT))
#define PHOSH_DBUS_SCREENSHOT_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_SCREENSHOT, PhoshDBusScreenshotIface))

struct _PhoshDBusScreenshot;
typedef struct _PhoshDBusScreenshot PhoshDBusScreenshot;
typedef struct _PhoshDBusScreenshotIface PhoshDBusScreenshotIface;

struct _PhoshDBusScreenshotIface
{
  GTypeInterface parent_iface;

  gboolean (*handle_flash_area) (
    PhoshDBusScreenshot *object,
    GDBusMethodInvocation *invocation,
    gint arg_x,
    gint arg_y,
    gint arg_width,
    gint arg_height);

  gboolean (*handle_pick_color) (
    PhoshDBusScreenshot *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_screenshot) (
    PhoshDBusScreenshot *object,
    GDBusMethodInvocation *invocation,
    gboolean arg_include_cursor,
    gboolean arg_flash,
    const gchar *arg_filename);

  gboolean (*handle_screenshot_area) (
    PhoshDBusScreenshot *object,
    GDBusMethodInvocation *invocation,
    gint arg_x,
    gint arg_y,
    gint arg_width,
    gint arg_height,
    gboolean arg_flash,
    const gchar *arg_filename);

  gboolean (*handle_screenshot_window) (
    PhoshDBusScreenshot *object,
    GDBusMethodInvocation *invocation,
    gboolean arg_include_frame,
    gboolean arg_include_cursor,
    gboolean arg_flash,
    const gchar *arg_filename);

  gboolean (*handle_select_area) (
    PhoshDBusScreenshot *object,
    GDBusMethodInvocation *invocation);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusScreenshot, g_object_unref)
#endif

GType phosh_dbus_screenshot_get_type (void);

GDBusInterfaceInfo *phosh_dbus_screenshot_interface_info (void);
guint phosh_dbus_screenshot_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void phosh_dbus_screenshot_complete_screenshot (
    PhoshDBusScreenshot *object,
    GDBusMethodInvocation *invocation,
    gboolean success,
    const gchar *filename_used);

void phosh_dbus_screenshot_complete_screenshot_window (
    PhoshDBusScreenshot *object,
    GDBusMethodInvocation *invocation,
    gboolean success,
    const gchar *filename_used);

void phosh_dbus_screenshot_complete_screenshot_area (
    PhoshDBusScreenshot *object,
    GDBusMethodInvocation *invocation,
    gboolean success,
    const gchar *filename_used);

void phosh_dbus_screenshot_complete_pick_color (
    PhoshDBusScreenshot *object,
    GDBusMethodInvocation *invocation,
    GVariant *result);

void phosh_dbus_screenshot_complete_flash_area (
    PhoshDBusScreenshot *object,
    GDBusMethodInvocation *invocation);

void phosh_dbus_screenshot_complete_select_area (
    PhoshDBusScreenshot *object,
    GDBusMethodInvocation *invocation,
    gint x,
    gint y,
    gint width,
    gint height);



/* D-Bus method calls: */
void phosh_dbus_screenshot_call_screenshot (
    PhoshDBusScreenshot *proxy,
    gboolean arg_include_cursor,
    gboolean arg_flash,
    const gchar *arg_filename,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_screenshot_call_screenshot_finish (
    PhoshDBusScreenshot *proxy,
    gboolean* out_success,
    gchar ** out_filename_used,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_screenshot_call_screenshot_sync (
    PhoshDBusScreenshot *proxy,
    gboolean arg_include_cursor,
    gboolean arg_flash,
    const gchar *arg_filename,
    gboolean* out_success,
    gchar ** out_filename_used,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_screenshot_call_screenshot_window (
    PhoshDBusScreenshot *proxy,
    gboolean arg_include_frame,
    gboolean arg_include_cursor,
    gboolean arg_flash,
    const gchar *arg_filename,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_screenshot_call_screenshot_window_finish (
    PhoshDBusScreenshot *proxy,
    gboolean* out_success,
    gchar ** out_filename_used,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_screenshot_call_screenshot_window_sync (
    PhoshDBusScreenshot *proxy,
    gboolean arg_include_frame,
    gboolean arg_include_cursor,
    gboolean arg_flash,
    const gchar *arg_filename,
    gboolean* out_success,
    gchar ** out_filename_used,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_screenshot_call_screenshot_area (
    PhoshDBusScreenshot *proxy,
    gint arg_x,
    gint arg_y,
    gint arg_width,
    gint arg_height,
    gboolean arg_flash,
    const gchar *arg_filename,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_screenshot_call_screenshot_area_finish (
    PhoshDBusScreenshot *proxy,
    gboolean* out_success,
    gchar ** out_filename_used,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_screenshot_call_screenshot_area_sync (
    PhoshDBusScreenshot *proxy,
    gint arg_x,
    gint arg_y,
    gint arg_width,
    gint arg_height,
    gboolean arg_flash,
    const gchar *arg_filename,
    gboolean* out_success,
    gchar ** out_filename_used,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_screenshot_call_pick_color (
    PhoshDBusScreenshot *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_screenshot_call_pick_color_finish (
    PhoshDBusScreenshot *proxy,
    GVariant ** out_result,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_screenshot_call_pick_color_sync (
    PhoshDBusScreenshot *proxy,
    GVariant ** out_result,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_screenshot_call_flash_area (
    PhoshDBusScreenshot *proxy,
    gint arg_x,
    gint arg_y,
    gint arg_width,
    gint arg_height,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_screenshot_call_flash_area_finish (
    PhoshDBusScreenshot *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_screenshot_call_flash_area_sync (
    PhoshDBusScreenshot *proxy,
    gint arg_x,
    gint arg_y,
    gint arg_width,
    gint arg_height,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_screenshot_call_select_area (
    PhoshDBusScreenshot *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_screenshot_call_select_area_finish (
    PhoshDBusScreenshot *proxy,
    gint* out_x,
    gint* out_y,
    gint* out_width,
    gint* out_height,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_screenshot_call_select_area_sync (
    PhoshDBusScreenshot *proxy,
    gint* out_x,
    gint* out_y,
    gint* out_width,
    gint* out_height,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define PHOSH_DBUS_TYPE_SCREENSHOT_PROXY (phosh_dbus_screenshot_proxy_get_type ())
#define PHOSH_DBUS_SCREENSHOT_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_SCREENSHOT_PROXY, PhoshDBusScreenshotProxy))
#define PHOSH_DBUS_SCREENSHOT_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_SCREENSHOT_PROXY, PhoshDBusScreenshotProxyClass))
#define PHOSH_DBUS_SCREENSHOT_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_SCREENSHOT_PROXY, PhoshDBusScreenshotProxyClass))
#define PHOSH_DBUS_IS_SCREENSHOT_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_SCREENSHOT_PROXY))
#define PHOSH_DBUS_IS_SCREENSHOT_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_SCREENSHOT_PROXY))

typedef struct _PhoshDBusScreenshotProxy PhoshDBusScreenshotProxy;
typedef struct _PhoshDBusScreenshotProxyClass PhoshDBusScreenshotProxyClass;
typedef struct _PhoshDBusScreenshotProxyPrivate PhoshDBusScreenshotProxyPrivate;

struct _PhoshDBusScreenshotProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusScreenshotProxyPrivate *priv;
};

struct _PhoshDBusScreenshotProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_screenshot_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusScreenshotProxy, g_object_unref)
#endif

void phosh_dbus_screenshot_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusScreenshot *phosh_dbus_screenshot_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusScreenshot *phosh_dbus_screenshot_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_screenshot_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusScreenshot *phosh_dbus_screenshot_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusScreenshot *phosh_dbus_screenshot_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_SCREENSHOT_SKELETON (phosh_dbus_screenshot_skeleton_get_type ())
#define PHOSH_DBUS_SCREENSHOT_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_SCREENSHOT_SKELETON, PhoshDBusScreenshotSkeleton))
#define PHOSH_DBUS_SCREENSHOT_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_SCREENSHOT_SKELETON, PhoshDBusScreenshotSkeletonClass))
#define PHOSH_DBUS_SCREENSHOT_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_SCREENSHOT_SKELETON, PhoshDBusScreenshotSkeletonClass))
#define PHOSH_DBUS_IS_SCREENSHOT_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_SCREENSHOT_SKELETON))
#define PHOSH_DBUS_IS_SCREENSHOT_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_SCREENSHOT_SKELETON))

typedef struct _PhoshDBusScreenshotSkeleton PhoshDBusScreenshotSkeleton;
typedef struct _PhoshDBusScreenshotSkeletonClass PhoshDBusScreenshotSkeletonClass;
typedef struct _PhoshDBusScreenshotSkeletonPrivate PhoshDBusScreenshotSkeletonPrivate;

struct _PhoshDBusScreenshotSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusScreenshotSkeletonPrivate *priv;
};

struct _PhoshDBusScreenshotSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_screenshot_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusScreenshotSkeleton, g_object_unref)
#endif

PhoshDBusScreenshot *phosh_dbus_screenshot_skeleton_new (void);


G_END_DECLS

#endif /* __PHOSH_SCREENSHOT_DBUS_H__ */
