/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __PHOSH_DISPLAY_DBUS_H__
#define __PHOSH_DISPLAY_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.Mutter.DisplayConfig */

#define PHOSH_DBUS_TYPE_DISPLAY_CONFIG (phosh_dbus_display_config_get_type ())
#define PHOSH_DBUS_DISPLAY_CONFIG(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_DISPLAY_CONFIG, PhoshDBusDisplayConfig))
#define PHOSH_DBUS_IS_DISPLAY_CONFIG(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_DISPLAY_CONFIG))
#define PHOSH_DBUS_DISPLAY_CONFIG_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_DISPLAY_CONFIG, PhoshDBusDisplayConfigIface))

struct _PhoshDBusDisplayConfig;
typedef struct _PhoshDBusDisplayConfig PhoshDBusDisplayConfig;
typedef struct _PhoshDBusDisplayConfigIface PhoshDBusDisplayConfigIface;

struct _PhoshDBusDisplayConfigIface
{
  GTypeInterface parent_iface;



  gboolean (*handle_apply_configuration) (
    PhoshDBusDisplayConfig *object,
    GDBusMethodInvocation *invocation,
    guint arg_serial,
    gboolean arg_persistent,
    GVariant *arg_crtcs,
    GVariant *arg_outputs);

  gboolean (*handle_apply_monitors_config) (
    PhoshDBusDisplayConfig *object,
    GDBusMethodInvocation *invocation,
    guint arg_serial,
    guint arg_method,
    GVariant *arg_logical_monitors,
    GVariant *arg_properties);

  gboolean (*handle_change_backlight) (
    PhoshDBusDisplayConfig *object,
    GDBusMethodInvocation *invocation,
    guint arg_serial,
    guint arg_output,
    gint arg_value);

  gboolean (*handle_get_crtc_gamma) (
    PhoshDBusDisplayConfig *object,
    GDBusMethodInvocation *invocation,
    guint arg_serial,
    guint arg_crtc);

  gboolean (*handle_get_current_state) (
    PhoshDBusDisplayConfig *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_get_resources) (
    PhoshDBusDisplayConfig *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_set_crtc_gamma) (
    PhoshDBusDisplayConfig *object,
    GDBusMethodInvocation *invocation,
    guint arg_serial,
    guint arg_crtc,
    GVariant *arg_red,
    GVariant *arg_green,
    GVariant *arg_blue);

  gboolean (*handle_set_output_ctm) (
    PhoshDBusDisplayConfig *object,
    GDBusMethodInvocation *invocation,
    guint arg_serial,
    guint arg_output,
    GVariant *arg_ctm);

  gboolean  (*get_apply_monitors_config_allowed) (PhoshDBusDisplayConfig *object);

  GVariant * (*get_backlight) (PhoshDBusDisplayConfig *object);

  gboolean  (*get_night_light_supported) (PhoshDBusDisplayConfig *object);

  gboolean  (*get_panel_orientation_managed) (PhoshDBusDisplayConfig *object);

  gint  (*get_power_save_mode) (PhoshDBusDisplayConfig *object);

  void (*monitors_changed) (
    PhoshDBusDisplayConfig *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusDisplayConfig, g_object_unref)
#endif

GType phosh_dbus_display_config_get_type (void);

GDBusInterfaceInfo *phosh_dbus_display_config_interface_info (void);
guint phosh_dbus_display_config_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void phosh_dbus_display_config_complete_get_resources (
    PhoshDBusDisplayConfig *object,
    GDBusMethodInvocation *invocation,
    guint serial,
    GVariant *crtcs,
    GVariant *outputs,
    GVariant *modes,
    gint max_screen_width,
    gint max_screen_height);

void phosh_dbus_display_config_complete_apply_configuration (
    PhoshDBusDisplayConfig *object,
    GDBusMethodInvocation *invocation);

void phosh_dbus_display_config_complete_change_backlight (
    PhoshDBusDisplayConfig *object,
    GDBusMethodInvocation *invocation,
    gint new_value);

void phosh_dbus_display_config_complete_get_crtc_gamma (
    PhoshDBusDisplayConfig *object,
    GDBusMethodInvocation *invocation,
    GVariant *red,
    GVariant *green,
    GVariant *blue);

void phosh_dbus_display_config_complete_set_crtc_gamma (
    PhoshDBusDisplayConfig *object,
    GDBusMethodInvocation *invocation);

void phosh_dbus_display_config_complete_get_current_state (
    PhoshDBusDisplayConfig *object,
    GDBusMethodInvocation *invocation,
    guint serial,
    GVariant *monitors,
    GVariant *logical_monitors,
    GVariant *properties);

void phosh_dbus_display_config_complete_apply_monitors_config (
    PhoshDBusDisplayConfig *object,
    GDBusMethodInvocation *invocation);

void phosh_dbus_display_config_complete_set_output_ctm (
    PhoshDBusDisplayConfig *object,
    GDBusMethodInvocation *invocation);



/* D-Bus signal emissions functions: */
void phosh_dbus_display_config_emit_monitors_changed (
    PhoshDBusDisplayConfig *object);



/* D-Bus method calls: */
void phosh_dbus_display_config_call_get_resources (
    PhoshDBusDisplayConfig *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_display_config_call_get_resources_finish (
    PhoshDBusDisplayConfig *proxy,
    guint* out_serial,
    GVariant ** out_crtcs,
    GVariant ** out_outputs,
    GVariant ** out_modes,
    gint* out_max_screen_width,
    gint* out_max_screen_height,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_display_config_call_get_resources_sync (
    PhoshDBusDisplayConfig *proxy,
    guint* out_serial,
    GVariant ** out_crtcs,
    GVariant ** out_outputs,
    GVariant ** out_modes,
    gint* out_max_screen_width,
    gint* out_max_screen_height,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_display_config_call_apply_configuration (
    PhoshDBusDisplayConfig *proxy,
    guint arg_serial,
    gboolean arg_persistent,
    GVariant *arg_crtcs,
    GVariant *arg_outputs,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_display_config_call_apply_configuration_finish (
    PhoshDBusDisplayConfig *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_display_config_call_apply_configuration_sync (
    PhoshDBusDisplayConfig *proxy,
    guint arg_serial,
    gboolean arg_persistent,
    GVariant *arg_crtcs,
    GVariant *arg_outputs,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_display_config_call_change_backlight (
    PhoshDBusDisplayConfig *proxy,
    guint arg_serial,
    guint arg_output,
    gint arg_value,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_display_config_call_change_backlight_finish (
    PhoshDBusDisplayConfig *proxy,
    gint* out_new_value,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_display_config_call_change_backlight_sync (
    PhoshDBusDisplayConfig *proxy,
    guint arg_serial,
    guint arg_output,
    gint arg_value,
    gint* out_new_value,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_display_config_call_get_crtc_gamma (
    PhoshDBusDisplayConfig *proxy,
    guint arg_serial,
    guint arg_crtc,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_display_config_call_get_crtc_gamma_finish (
    PhoshDBusDisplayConfig *proxy,
    GVariant ** out_red,
    GVariant ** out_green,
    GVariant ** out_blue,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_display_config_call_get_crtc_gamma_sync (
    PhoshDBusDisplayConfig *proxy,
    guint arg_serial,
    guint arg_crtc,
    GVariant ** out_red,
    GVariant ** out_green,
    GVariant ** out_blue,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_display_config_call_set_crtc_gamma (
    PhoshDBusDisplayConfig *proxy,
    guint arg_serial,
    guint arg_crtc,
    GVariant *arg_red,
    GVariant *arg_green,
    GVariant *arg_blue,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_display_config_call_set_crtc_gamma_finish (
    PhoshDBusDisplayConfig *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_display_config_call_set_crtc_gamma_sync (
    PhoshDBusDisplayConfig *proxy,
    guint arg_serial,
    guint arg_crtc,
    GVariant *arg_red,
    GVariant *arg_green,
    GVariant *arg_blue,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_display_config_call_get_current_state (
    PhoshDBusDisplayConfig *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_display_config_call_get_current_state_finish (
    PhoshDBusDisplayConfig *proxy,
    guint* out_serial,
    GVariant ** out_monitors,
    GVariant ** out_logical_monitors,
    GVariant ** out_properties,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_display_config_call_get_current_state_sync (
    PhoshDBusDisplayConfig *proxy,
    guint* out_serial,
    GVariant ** out_monitors,
    GVariant ** out_logical_monitors,
    GVariant ** out_properties,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_display_config_call_apply_monitors_config (
    PhoshDBusDisplayConfig *proxy,
    guint arg_serial,
    guint arg_method,
    GVariant *arg_logical_monitors,
    GVariant *arg_properties,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_display_config_call_apply_monitors_config_finish (
    PhoshDBusDisplayConfig *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_display_config_call_apply_monitors_config_sync (
    PhoshDBusDisplayConfig *proxy,
    guint arg_serial,
    guint arg_method,
    GVariant *arg_logical_monitors,
    GVariant *arg_properties,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_display_config_call_set_output_ctm (
    PhoshDBusDisplayConfig *proxy,
    guint arg_serial,
    guint arg_output,
    GVariant *arg_ctm,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_display_config_call_set_output_ctm_finish (
    PhoshDBusDisplayConfig *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_display_config_call_set_output_ctm_sync (
    PhoshDBusDisplayConfig *proxy,
    guint arg_serial,
    guint arg_output,
    GVariant *arg_ctm,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
GVariant *phosh_dbus_display_config_get_backlight (PhoshDBusDisplayConfig *object);
GVariant *phosh_dbus_display_config_dup_backlight (PhoshDBusDisplayConfig *object);
void phosh_dbus_display_config_set_backlight (PhoshDBusDisplayConfig *object, GVariant *value);

gint phosh_dbus_display_config_get_power_save_mode (PhoshDBusDisplayConfig *object);
void phosh_dbus_display_config_set_power_save_mode (PhoshDBusDisplayConfig *object, gint value);

gboolean phosh_dbus_display_config_get_panel_orientation_managed (PhoshDBusDisplayConfig *object);
void phosh_dbus_display_config_set_panel_orientation_managed (PhoshDBusDisplayConfig *object, gboolean value);

gboolean phosh_dbus_display_config_get_apply_monitors_config_allowed (PhoshDBusDisplayConfig *object);
void phosh_dbus_display_config_set_apply_monitors_config_allowed (PhoshDBusDisplayConfig *object, gboolean value);

gboolean phosh_dbus_display_config_get_night_light_supported (PhoshDBusDisplayConfig *object);
void phosh_dbus_display_config_set_night_light_supported (PhoshDBusDisplayConfig *object, gboolean value);


/* ---- */

#define PHOSH_DBUS_TYPE_DISPLAY_CONFIG_PROXY (phosh_dbus_display_config_proxy_get_type ())
#define PHOSH_DBUS_DISPLAY_CONFIG_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_DISPLAY_CONFIG_PROXY, PhoshDBusDisplayConfigProxy))
#define PHOSH_DBUS_DISPLAY_CONFIG_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_DISPLAY_CONFIG_PROXY, PhoshDBusDisplayConfigProxyClass))
#define PHOSH_DBUS_DISPLAY_CONFIG_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_DISPLAY_CONFIG_PROXY, PhoshDBusDisplayConfigProxyClass))
#define PHOSH_DBUS_IS_DISPLAY_CONFIG_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_DISPLAY_CONFIG_PROXY))
#define PHOSH_DBUS_IS_DISPLAY_CONFIG_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_DISPLAY_CONFIG_PROXY))

typedef struct _PhoshDBusDisplayConfigProxy PhoshDBusDisplayConfigProxy;
typedef struct _PhoshDBusDisplayConfigProxyClass PhoshDBusDisplayConfigProxyClass;
typedef struct _PhoshDBusDisplayConfigProxyPrivate PhoshDBusDisplayConfigProxyPrivate;

struct _PhoshDBusDisplayConfigProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusDisplayConfigProxyPrivate *priv;
};

struct _PhoshDBusDisplayConfigProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_display_config_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusDisplayConfigProxy, g_object_unref)
#endif

void phosh_dbus_display_config_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusDisplayConfig *phosh_dbus_display_config_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusDisplayConfig *phosh_dbus_display_config_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_display_config_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusDisplayConfig *phosh_dbus_display_config_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusDisplayConfig *phosh_dbus_display_config_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_DISPLAY_CONFIG_SKELETON (phosh_dbus_display_config_skeleton_get_type ())
#define PHOSH_DBUS_DISPLAY_CONFIG_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_DISPLAY_CONFIG_SKELETON, PhoshDBusDisplayConfigSkeleton))
#define PHOSH_DBUS_DISPLAY_CONFIG_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_DISPLAY_CONFIG_SKELETON, PhoshDBusDisplayConfigSkeletonClass))
#define PHOSH_DBUS_DISPLAY_CONFIG_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_DISPLAY_CONFIG_SKELETON, PhoshDBusDisplayConfigSkeletonClass))
#define PHOSH_DBUS_IS_DISPLAY_CONFIG_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_DISPLAY_CONFIG_SKELETON))
#define PHOSH_DBUS_IS_DISPLAY_CONFIG_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_DISPLAY_CONFIG_SKELETON))

typedef struct _PhoshDBusDisplayConfigSkeleton PhoshDBusDisplayConfigSkeleton;
typedef struct _PhoshDBusDisplayConfigSkeletonClass PhoshDBusDisplayConfigSkeletonClass;
typedef struct _PhoshDBusDisplayConfigSkeletonPrivate PhoshDBusDisplayConfigSkeletonPrivate;

struct _PhoshDBusDisplayConfigSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusDisplayConfigSkeletonPrivate *priv;
};

struct _PhoshDBusDisplayConfigSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_display_config_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusDisplayConfigSkeleton, g_object_unref)
#endif

PhoshDBusDisplayConfig *phosh_dbus_display_config_skeleton_new (void);


G_END_DECLS

#endif /* __PHOSH_DISPLAY_DBUS_H__ */
