/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __PHOSH_OSK0_DBUS_H__
#define __PHOSH_OSK0_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for sm.puri.OSK0 */

#define PHOSH_DBUS_TYPE_OSK0 (phosh_dbus_osk0_get_type ())
#define PHOSH_DBUS_OSK0(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_OSK0, PhoshDBusOSK0))
#define PHOSH_DBUS_IS_OSK0(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_OSK0))
#define PHOSH_DBUS_OSK0_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_OSK0, PhoshDBusOSK0Iface))

struct _PhoshDBusOSK0;
typedef struct _PhoshDBusOSK0 PhoshDBusOSK0;
typedef struct _PhoshDBusOSK0Iface PhoshDBusOSK0Iface;

struct _PhoshDBusOSK0Iface
{
  GTypeInterface parent_iface;


  gboolean (*handle_set_visible) (
    PhoshDBusOSK0 *object,
    GDBusMethodInvocation *invocation,
    gboolean arg_visible);

  gboolean  (*get_visible) (PhoshDBusOSK0 *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusOSK0, g_object_unref)
#endif

GType phosh_dbus_osk0_get_type (void);

GDBusInterfaceInfo *phosh_dbus_osk0_interface_info (void);
guint phosh_dbus_osk0_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void phosh_dbus_osk0_complete_set_visible (
    PhoshDBusOSK0 *object,
    GDBusMethodInvocation *invocation);



/* D-Bus method calls: */
void phosh_dbus_osk0_call_set_visible (
    PhoshDBusOSK0 *proxy,
    gboolean arg_visible,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_osk0_call_set_visible_finish (
    PhoshDBusOSK0 *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_osk0_call_set_visible_sync (
    PhoshDBusOSK0 *proxy,
    gboolean arg_visible,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
gboolean phosh_dbus_osk0_get_visible (PhoshDBusOSK0 *object);
void phosh_dbus_osk0_set_visible (PhoshDBusOSK0 *object, gboolean value);


/* ---- */

#define PHOSH_DBUS_TYPE_OSK0_PROXY (phosh_dbus_osk0_proxy_get_type ())
#define PHOSH_DBUS_OSK0_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_OSK0_PROXY, PhoshDBusOSK0Proxy))
#define PHOSH_DBUS_OSK0_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_OSK0_PROXY, PhoshDBusOSK0ProxyClass))
#define PHOSH_DBUS_OSK0_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_OSK0_PROXY, PhoshDBusOSK0ProxyClass))
#define PHOSH_DBUS_IS_OSK0_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_OSK0_PROXY))
#define PHOSH_DBUS_IS_OSK0_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_OSK0_PROXY))

typedef struct _PhoshDBusOSK0Proxy PhoshDBusOSK0Proxy;
typedef struct _PhoshDBusOSK0ProxyClass PhoshDBusOSK0ProxyClass;
typedef struct _PhoshDBusOSK0ProxyPrivate PhoshDBusOSK0ProxyPrivate;

struct _PhoshDBusOSK0Proxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusOSK0ProxyPrivate *priv;
};

struct _PhoshDBusOSK0ProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_osk0_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusOSK0Proxy, g_object_unref)
#endif

void phosh_dbus_osk0_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusOSK0 *phosh_dbus_osk0_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusOSK0 *phosh_dbus_osk0_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_osk0_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusOSK0 *phosh_dbus_osk0_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusOSK0 *phosh_dbus_osk0_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_OSK0_SKELETON (phosh_dbus_osk0_skeleton_get_type ())
#define PHOSH_DBUS_OSK0_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_OSK0_SKELETON, PhoshDBusOSK0Skeleton))
#define PHOSH_DBUS_OSK0_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_OSK0_SKELETON, PhoshDBusOSK0SkeletonClass))
#define PHOSH_DBUS_OSK0_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_OSK0_SKELETON, PhoshDBusOSK0SkeletonClass))
#define PHOSH_DBUS_IS_OSK0_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_OSK0_SKELETON))
#define PHOSH_DBUS_IS_OSK0_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_OSK0_SKELETON))

typedef struct _PhoshDBusOSK0Skeleton PhoshDBusOSK0Skeleton;
typedef struct _PhoshDBusOSK0SkeletonClass PhoshDBusOSK0SkeletonClass;
typedef struct _PhoshDBusOSK0SkeletonPrivate PhoshDBusOSK0SkeletonPrivate;

struct _PhoshDBusOSK0Skeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusOSK0SkeletonPrivate *priv;
};

struct _PhoshDBusOSK0SkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_osk0_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusOSK0Skeleton, g_object_unref)
#endif

PhoshDBusOSK0 *phosh_dbus_osk0_skeleton_new (void);


G_END_DECLS

#endif /* __PHOSH_OSK0_DBUS_H__ */
