/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __ACCOUNTS_USER_DBUS_H__
#define __ACCOUNTS_USER_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.freedesktop.Accounts.User */

#define PHOSH_DBUS_TYPE_ACCOUNTS_USER (phosh_dbus_accounts_user_get_type ())
#define PHOSH_DBUS_ACCOUNTS_USER(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_ACCOUNTS_USER, PhoshDBusAccountsUser))
#define PHOSH_DBUS_IS_ACCOUNTS_USER(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_ACCOUNTS_USER))
#define PHOSH_DBUS_ACCOUNTS_USER_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_ACCOUNTS_USER, PhoshDBusAccountsUserIface))

struct _PhoshDBusAccountsUser;
typedef struct _PhoshDBusAccountsUser PhoshDBusAccountsUser;
typedef struct _PhoshDBusAccountsUserIface PhoshDBusAccountsUserIface;

struct _PhoshDBusAccountsUserIface
{
  GTypeInterface parent_iface;

  const gchar * (*get_icon_file) (PhoshDBusAccountsUser *object);

  const gchar * (*get_real_name) (PhoshDBusAccountsUser *object);

  const gchar * (*get_user_name) (PhoshDBusAccountsUser *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusAccountsUser, g_object_unref)
#endif

GType phosh_dbus_accounts_user_get_type (void);

GDBusInterfaceInfo *phosh_dbus_accounts_user_interface_info (void);
guint phosh_dbus_accounts_user_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus property accessors: */
const gchar *phosh_dbus_accounts_user_get_user_name (PhoshDBusAccountsUser *object);
gchar *phosh_dbus_accounts_user_dup_user_name (PhoshDBusAccountsUser *object);
void phosh_dbus_accounts_user_set_user_name (PhoshDBusAccountsUser *object, const gchar *value);

const gchar *phosh_dbus_accounts_user_get_real_name (PhoshDBusAccountsUser *object);
gchar *phosh_dbus_accounts_user_dup_real_name (PhoshDBusAccountsUser *object);
void phosh_dbus_accounts_user_set_real_name (PhoshDBusAccountsUser *object, const gchar *value);

const gchar *phosh_dbus_accounts_user_get_icon_file (PhoshDBusAccountsUser *object);
gchar *phosh_dbus_accounts_user_dup_icon_file (PhoshDBusAccountsUser *object);
void phosh_dbus_accounts_user_set_icon_file (PhoshDBusAccountsUser *object, const gchar *value);


/* ---- */

#define PHOSH_DBUS_TYPE_ACCOUNTS_USER_PROXY (phosh_dbus_accounts_user_proxy_get_type ())
#define PHOSH_DBUS_ACCOUNTS_USER_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_ACCOUNTS_USER_PROXY, PhoshDBusAccountsUserProxy))
#define PHOSH_DBUS_ACCOUNTS_USER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_ACCOUNTS_USER_PROXY, PhoshDBusAccountsUserProxyClass))
#define PHOSH_DBUS_ACCOUNTS_USER_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_ACCOUNTS_USER_PROXY, PhoshDBusAccountsUserProxyClass))
#define PHOSH_DBUS_IS_ACCOUNTS_USER_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_ACCOUNTS_USER_PROXY))
#define PHOSH_DBUS_IS_ACCOUNTS_USER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_ACCOUNTS_USER_PROXY))

typedef struct _PhoshDBusAccountsUserProxy PhoshDBusAccountsUserProxy;
typedef struct _PhoshDBusAccountsUserProxyClass PhoshDBusAccountsUserProxyClass;
typedef struct _PhoshDBusAccountsUserProxyPrivate PhoshDBusAccountsUserProxyPrivate;

struct _PhoshDBusAccountsUserProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusAccountsUserProxyPrivate *priv;
};

struct _PhoshDBusAccountsUserProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_accounts_user_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusAccountsUserProxy, g_object_unref)
#endif

void phosh_dbus_accounts_user_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusAccountsUser *phosh_dbus_accounts_user_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusAccountsUser *phosh_dbus_accounts_user_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_accounts_user_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusAccountsUser *phosh_dbus_accounts_user_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusAccountsUser *phosh_dbus_accounts_user_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_ACCOUNTS_USER_SKELETON (phosh_dbus_accounts_user_skeleton_get_type ())
#define PHOSH_DBUS_ACCOUNTS_USER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_ACCOUNTS_USER_SKELETON, PhoshDBusAccountsUserSkeleton))
#define PHOSH_DBUS_ACCOUNTS_USER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_ACCOUNTS_USER_SKELETON, PhoshDBusAccountsUserSkeletonClass))
#define PHOSH_DBUS_ACCOUNTS_USER_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_ACCOUNTS_USER_SKELETON, PhoshDBusAccountsUserSkeletonClass))
#define PHOSH_DBUS_IS_ACCOUNTS_USER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_ACCOUNTS_USER_SKELETON))
#define PHOSH_DBUS_IS_ACCOUNTS_USER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_ACCOUNTS_USER_SKELETON))

typedef struct _PhoshDBusAccountsUserSkeleton PhoshDBusAccountsUserSkeleton;
typedef struct _PhoshDBusAccountsUserSkeletonClass PhoshDBusAccountsUserSkeletonClass;
typedef struct _PhoshDBusAccountsUserSkeletonPrivate PhoshDBusAccountsUserSkeletonPrivate;

struct _PhoshDBusAccountsUserSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusAccountsUserSkeletonPrivate *priv;
};

struct _PhoshDBusAccountsUserSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_accounts_user_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusAccountsUserSkeleton, g_object_unref)
#endif

PhoshDBusAccountsUser *phosh_dbus_accounts_user_skeleton_new (void);


G_END_DECLS

#endif /* __ACCOUNTS_USER_DBUS_H__ */
