/* Generated by wayland-scanner 1.26.0 */

#include <stdbool.h>
#include <stdlib.h>
#include <stdint.h>
#include "wayland-util.h"

#ifndef __has_attribute
# define __has_attribute(x) 0  /* Compatibility with non-clang compilers. */
#endif

#if (__has_attribute(visibility) || defined(__GNUC__) && __GNUC__ >= 4)
#define WL_PRIVATE __attribute__ ((visibility("hidden")))
#else
#define WL_PRIVATE
#endif

extern const struct wl_interface phosh_private_keyboard_event_interface;
extern const struct wl_interface phosh_private_startup_tracker_interface;
extern const struct wl_interface phosh_private_xdg_switcher_interface;
extern const struct wl_interface wl_surface_interface;
extern const struct wl_interface zwlr_foreign_toplevel_handle_v1_interface;
extern const struct wl_interface zwlr_screencopy_frame_v1_interface;

static const struct wl_interface *phosh_types[] = {
	NULL,
	NULL,
	NULL,
	&wl_surface_interface,
	NULL,
	&phosh_private_xdg_switcher_interface,
	&zwlr_screencopy_frame_v1_interface,
	&zwlr_foreign_toplevel_handle_v1_interface,
	NULL,
	NULL,
	&phosh_private_keyboard_event_interface,
	&phosh_private_startup_tracker_interface,
};

static const struct wl_message phosh_private_requests[] = {
	{ "rotate_display", "ou", phosh_types + 3 },
	{ "get_xdg_switcher", "2n", phosh_types + 5 },
	{ "get_thumbnail", "4nouu", phosh_types + 6 },
	{ "get_keyboard_event", "5n", phosh_types + 10 },
	{ "get_startup_tracker", "6n", phosh_types + 11 },
	{ "set_shell_state", "6u", phosh_types + 0 },
};

WL_PRIVATE const struct wl_interface phosh_private_interface = {
	"phosh_private", 7,
	6, phosh_private_requests,
	0, NULL,
};

static const struct wl_message phosh_private_keyboard_event_requests[] = {
	{ "grab_accelerator_request", "5s", phosh_types + 0 },
	{ "ungrab_accelerator_request", "5u", phosh_types + 0 },
	{ "destroy", "5", phosh_types + 0 },
};

static const struct wl_message phosh_private_keyboard_event_events[] = {
	{ "accelerator_activated_event", "5uu", phosh_types + 0 },
	{ "grab_failed_event", "5su", phosh_types + 0 },
	{ "grab_success_event", "5su", phosh_types + 0 },
	{ "ungrab_failed_event", "5uu", phosh_types + 0 },
	{ "ungrab_success_event", "5u", phosh_types + 0 },
	{ "accelerator_released_event", "7uu", phosh_types + 0 },
};

WL_PRIVATE const struct wl_interface phosh_private_keyboard_event_interface = {
	"phosh_private_keyboard_event", 7,
	3, phosh_private_keyboard_event_requests,
	6, phosh_private_keyboard_event_events,
};

static const struct wl_message phosh_private_xdg_switcher_requests[] = {
	{ "list_xdg_surfaces", "2", phosh_types + 0 },
	{ "destroy", "2", phosh_types + 0 },
	{ "raise_xdg_surface", "2s?s", phosh_types + 0 },
	{ "close_xdg_surface", "3s?s", phosh_types + 0 },
};

static const struct wl_message phosh_private_xdg_switcher_events[] = {
	{ "xdg_surface", "2s?s", phosh_types + 0 },
	{ "list_xdg_surfaces_done", "2", phosh_types + 0 },
};

WL_PRIVATE const struct wl_interface phosh_private_xdg_switcher_interface = {
	"phosh_private_xdg_switcher", 7,
	4, phosh_private_xdg_switcher_requests,
	2, phosh_private_xdg_switcher_events,
};

static const struct wl_message phosh_private_startup_tracker_requests[] = {
	{ "destroy", "", phosh_types + 0 },
};

static const struct wl_message phosh_private_startup_tracker_events[] = {
	{ "startup_id", "suu", phosh_types + 0 },
	{ "launched", "suu", phosh_types + 0 },
};

WL_PRIVATE const struct wl_interface phosh_private_startup_tracker_interface = {
	"phosh_private_startup_tracker", 7,
	1, phosh_private_startup_tracker_requests,
	2, phosh_private_startup_tracker_events,
};

