/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __PHOSH_END_SESSION_DIALOG_DBUS_H__
#define __PHOSH_END_SESSION_DIALOG_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.SessionManager.EndSessionDialog */

#define PHOSH_DBUS_TYPE_END_SESSION_DIALOG (phosh_dbus_end_session_dialog_get_type ())
#define PHOSH_DBUS_END_SESSION_DIALOG(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_END_SESSION_DIALOG, PhoshDBusEndSessionDialog))
#define PHOSH_DBUS_IS_END_SESSION_DIALOG(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_END_SESSION_DIALOG))
#define PHOSH_DBUS_END_SESSION_DIALOG_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_END_SESSION_DIALOG, PhoshDBusEndSessionDialogIface))

struct _PhoshDBusEndSessionDialog;
typedef struct _PhoshDBusEndSessionDialog PhoshDBusEndSessionDialog;
typedef struct _PhoshDBusEndSessionDialogIface PhoshDBusEndSessionDialogIface;

struct _PhoshDBusEndSessionDialogIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_open) (
    PhoshDBusEndSessionDialog *object,
    GDBusMethodInvocation *invocation,
    guint arg_type,
    guint arg_timestamp,
    guint arg_seconds_to_stay_open,
    const gchar *const *arg_inhibitor_object_paths);

  void (*canceled) (
    PhoshDBusEndSessionDialog *object);

  void (*closed) (
    PhoshDBusEndSessionDialog *object);

  void (*confirmed_hibernate) (
    PhoshDBusEndSessionDialog *object);

  void (*confirmed_hybrid_sleep) (
    PhoshDBusEndSessionDialog *object);

  void (*confirmed_logout) (
    PhoshDBusEndSessionDialog *object);

  void (*confirmed_reboot) (
    PhoshDBusEndSessionDialog *object);

  void (*confirmed_shutdown) (
    PhoshDBusEndSessionDialog *object);

  void (*confirmed_suspend) (
    PhoshDBusEndSessionDialog *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusEndSessionDialog, g_object_unref)
#endif

GType phosh_dbus_end_session_dialog_get_type (void);

GDBusInterfaceInfo *phosh_dbus_end_session_dialog_interface_info (void);
guint phosh_dbus_end_session_dialog_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void phosh_dbus_end_session_dialog_complete_open (
    PhoshDBusEndSessionDialog *object,
    GDBusMethodInvocation *invocation);



/* D-Bus signal emissions functions: */
void phosh_dbus_end_session_dialog_emit_confirmed_logout (
    PhoshDBusEndSessionDialog *object);

void phosh_dbus_end_session_dialog_emit_confirmed_reboot (
    PhoshDBusEndSessionDialog *object);

void phosh_dbus_end_session_dialog_emit_confirmed_shutdown (
    PhoshDBusEndSessionDialog *object);

void phosh_dbus_end_session_dialog_emit_confirmed_hibernate (
    PhoshDBusEndSessionDialog *object);

void phosh_dbus_end_session_dialog_emit_confirmed_suspend (
    PhoshDBusEndSessionDialog *object);

void phosh_dbus_end_session_dialog_emit_confirmed_hybrid_sleep (
    PhoshDBusEndSessionDialog *object);

void phosh_dbus_end_session_dialog_emit_canceled (
    PhoshDBusEndSessionDialog *object);

void phosh_dbus_end_session_dialog_emit_closed (
    PhoshDBusEndSessionDialog *object);



/* D-Bus method calls: */
void phosh_dbus_end_session_dialog_call_open (
    PhoshDBusEndSessionDialog *proxy,
    guint arg_type,
    guint arg_timestamp,
    guint arg_seconds_to_stay_open,
    const gchar *const *arg_inhibitor_object_paths,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_end_session_dialog_call_open_finish (
    PhoshDBusEndSessionDialog *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_end_session_dialog_call_open_sync (
    PhoshDBusEndSessionDialog *proxy,
    guint arg_type,
    guint arg_timestamp,
    guint arg_seconds_to_stay_open,
    const gchar *const *arg_inhibitor_object_paths,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define PHOSH_DBUS_TYPE_END_SESSION_DIALOG_PROXY (phosh_dbus_end_session_dialog_proxy_get_type ())
#define PHOSH_DBUS_END_SESSION_DIALOG_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_END_SESSION_DIALOG_PROXY, PhoshDBusEndSessionDialogProxy))
#define PHOSH_DBUS_END_SESSION_DIALOG_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_END_SESSION_DIALOG_PROXY, PhoshDBusEndSessionDialogProxyClass))
#define PHOSH_DBUS_END_SESSION_DIALOG_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_END_SESSION_DIALOG_PROXY, PhoshDBusEndSessionDialogProxyClass))
#define PHOSH_DBUS_IS_END_SESSION_DIALOG_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_END_SESSION_DIALOG_PROXY))
#define PHOSH_DBUS_IS_END_SESSION_DIALOG_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_END_SESSION_DIALOG_PROXY))

typedef struct _PhoshDBusEndSessionDialogProxy PhoshDBusEndSessionDialogProxy;
typedef struct _PhoshDBusEndSessionDialogProxyClass PhoshDBusEndSessionDialogProxyClass;
typedef struct _PhoshDBusEndSessionDialogProxyPrivate PhoshDBusEndSessionDialogProxyPrivate;

struct _PhoshDBusEndSessionDialogProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusEndSessionDialogProxyPrivate *priv;
};

struct _PhoshDBusEndSessionDialogProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_end_session_dialog_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusEndSessionDialogProxy, g_object_unref)
#endif

void phosh_dbus_end_session_dialog_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusEndSessionDialog *phosh_dbus_end_session_dialog_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusEndSessionDialog *phosh_dbus_end_session_dialog_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_end_session_dialog_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusEndSessionDialog *phosh_dbus_end_session_dialog_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusEndSessionDialog *phosh_dbus_end_session_dialog_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_END_SESSION_DIALOG_SKELETON (phosh_dbus_end_session_dialog_skeleton_get_type ())
#define PHOSH_DBUS_END_SESSION_DIALOG_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_END_SESSION_DIALOG_SKELETON, PhoshDBusEndSessionDialogSkeleton))
#define PHOSH_DBUS_END_SESSION_DIALOG_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_END_SESSION_DIALOG_SKELETON, PhoshDBusEndSessionDialogSkeletonClass))
#define PHOSH_DBUS_END_SESSION_DIALOG_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_END_SESSION_DIALOG_SKELETON, PhoshDBusEndSessionDialogSkeletonClass))
#define PHOSH_DBUS_IS_END_SESSION_DIALOG_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_END_SESSION_DIALOG_SKELETON))
#define PHOSH_DBUS_IS_END_SESSION_DIALOG_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_END_SESSION_DIALOG_SKELETON))

typedef struct _PhoshDBusEndSessionDialogSkeleton PhoshDBusEndSessionDialogSkeleton;
typedef struct _PhoshDBusEndSessionDialogSkeletonClass PhoshDBusEndSessionDialogSkeletonClass;
typedef struct _PhoshDBusEndSessionDialogSkeletonPrivate PhoshDBusEndSessionDialogSkeletonPrivate;

struct _PhoshDBusEndSessionDialogSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusEndSessionDialogSkeletonPrivate *priv;
};

struct _PhoshDBusEndSessionDialogSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_end_session_dialog_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusEndSessionDialogSkeleton, g_object_unref)
#endif

PhoshDBusEndSessionDialog *phosh_dbus_end_session_dialog_skeleton_new (void);


G_END_DECLS

#endif /* __PHOSH_END_SESSION_DIALOG_DBUS_H__ */
