/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __CALLS_EMERGENCY_DBUS_H__
#define __CALLS_EMERGENCY_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.Calls.EmergencyCalls */

#define PHOSH_DBUS_TYPE_EMERGENCY_CALLS (phosh_dbus_emergency_calls_get_type ())
#define PHOSH_DBUS_EMERGENCY_CALLS(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_EMERGENCY_CALLS, PhoshDBusEmergencyCalls))
#define PHOSH_DBUS_IS_EMERGENCY_CALLS(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_EMERGENCY_CALLS))
#define PHOSH_DBUS_EMERGENCY_CALLS_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_EMERGENCY_CALLS, PhoshDBusEmergencyCallsIface))

struct _PhoshDBusEmergencyCalls;
typedef struct _PhoshDBusEmergencyCalls PhoshDBusEmergencyCalls;
typedef struct _PhoshDBusEmergencyCallsIface PhoshDBusEmergencyCallsIface;

struct _PhoshDBusEmergencyCallsIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_call_emergency_contact) (
    PhoshDBusEmergencyCalls *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_id);

  gboolean (*handle_get_emergency_contacts) (
    PhoshDBusEmergencyCalls *object,
    GDBusMethodInvocation *invocation);

  void (*emergency_numbers_changed) (
    PhoshDBusEmergencyCalls *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusEmergencyCalls, g_object_unref)
#endif

GType phosh_dbus_emergency_calls_get_type (void);

GDBusInterfaceInfo *phosh_dbus_emergency_calls_interface_info (void);
guint phosh_dbus_emergency_calls_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void phosh_dbus_emergency_calls_complete_get_emergency_contacts (
    PhoshDBusEmergencyCalls *object,
    GDBusMethodInvocation *invocation,
    GVariant *contacts);

void phosh_dbus_emergency_calls_complete_call_emergency_contact (
    PhoshDBusEmergencyCalls *object,
    GDBusMethodInvocation *invocation);



/* D-Bus signal emissions functions: */
void phosh_dbus_emergency_calls_emit_emergency_numbers_changed (
    PhoshDBusEmergencyCalls *object);



/* D-Bus method calls: */
void phosh_dbus_emergency_calls_call_get_emergency_contacts (
    PhoshDBusEmergencyCalls *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_emergency_calls_call_get_emergency_contacts_finish (
    PhoshDBusEmergencyCalls *proxy,
    GVariant ** out_contacts,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_emergency_calls_call_get_emergency_contacts_sync (
    PhoshDBusEmergencyCalls *proxy,
    GVariant ** out_contacts,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_emergency_calls_call_call_emergency_contact (
    PhoshDBusEmergencyCalls *proxy,
    const gchar *arg_id,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_emergency_calls_call_call_emergency_contact_finish (
    PhoshDBusEmergencyCalls *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_emergency_calls_call_call_emergency_contact_sync (
    PhoshDBusEmergencyCalls *proxy,
    const gchar *arg_id,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define PHOSH_DBUS_TYPE_EMERGENCY_CALLS_PROXY (phosh_dbus_emergency_calls_proxy_get_type ())
#define PHOSH_DBUS_EMERGENCY_CALLS_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_EMERGENCY_CALLS_PROXY, PhoshDBusEmergencyCallsProxy))
#define PHOSH_DBUS_EMERGENCY_CALLS_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_EMERGENCY_CALLS_PROXY, PhoshDBusEmergencyCallsProxyClass))
#define PHOSH_DBUS_EMERGENCY_CALLS_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_EMERGENCY_CALLS_PROXY, PhoshDBusEmergencyCallsProxyClass))
#define PHOSH_DBUS_IS_EMERGENCY_CALLS_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_EMERGENCY_CALLS_PROXY))
#define PHOSH_DBUS_IS_EMERGENCY_CALLS_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_EMERGENCY_CALLS_PROXY))

typedef struct _PhoshDBusEmergencyCallsProxy PhoshDBusEmergencyCallsProxy;
typedef struct _PhoshDBusEmergencyCallsProxyClass PhoshDBusEmergencyCallsProxyClass;
typedef struct _PhoshDBusEmergencyCallsProxyPrivate PhoshDBusEmergencyCallsProxyPrivate;

struct _PhoshDBusEmergencyCallsProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusEmergencyCallsProxyPrivate *priv;
};

struct _PhoshDBusEmergencyCallsProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_emergency_calls_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusEmergencyCallsProxy, g_object_unref)
#endif

void phosh_dbus_emergency_calls_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusEmergencyCalls *phosh_dbus_emergency_calls_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusEmergencyCalls *phosh_dbus_emergency_calls_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_emergency_calls_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusEmergencyCalls *phosh_dbus_emergency_calls_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusEmergencyCalls *phosh_dbus_emergency_calls_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_EMERGENCY_CALLS_SKELETON (phosh_dbus_emergency_calls_skeleton_get_type ())
#define PHOSH_DBUS_EMERGENCY_CALLS_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_EMERGENCY_CALLS_SKELETON, PhoshDBusEmergencyCallsSkeleton))
#define PHOSH_DBUS_EMERGENCY_CALLS_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_EMERGENCY_CALLS_SKELETON, PhoshDBusEmergencyCallsSkeletonClass))
#define PHOSH_DBUS_EMERGENCY_CALLS_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_EMERGENCY_CALLS_SKELETON, PhoshDBusEmergencyCallsSkeletonClass))
#define PHOSH_DBUS_IS_EMERGENCY_CALLS_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_EMERGENCY_CALLS_SKELETON))
#define PHOSH_DBUS_IS_EMERGENCY_CALLS_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_EMERGENCY_CALLS_SKELETON))

typedef struct _PhoshDBusEmergencyCallsSkeleton PhoshDBusEmergencyCallsSkeleton;
typedef struct _PhoshDBusEmergencyCallsSkeletonClass PhoshDBusEmergencyCallsSkeletonClass;
typedef struct _PhoshDBusEmergencyCallsSkeletonPrivate PhoshDBusEmergencyCallsSkeletonPrivate;

struct _PhoshDBusEmergencyCallsSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusEmergencyCallsSkeletonPrivate *priv;
};

struct _PhoshDBusEmergencyCallsSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_emergency_calls_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusEmergencyCallsSkeleton, g_object_unref)
#endif

PhoshDBusEmergencyCalls *phosh_dbus_emergency_calls_skeleton_new (void);


G_END_DECLS

#endif /* __CALLS_EMERGENCY_DBUS_H__ */
