
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#pragma once

            #include <glib-object.h>


            G_BEGIN_DECLS

/* enumerations from "bluetooth-enums.h" */


GType bluetooth_type_get_type (void);
#define BLUETOOTH_TYPE_TYPE (bluetooth_type_get_type())


GType bluetooth_battery_type_get_type (void);
#define BLUETOOTH_TYPE_BATTERY_TYPE (bluetooth_battery_type_get_type())


GType bluetooth_adapter_state_get_type (void);
#define BLUETOOTH_TYPE_ADAPTER_STATE (bluetooth_adapter_state_get_type())

G_END_DECLS

/* Generated data ends here */

