/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __PHOSH_IDLE_DBUS_H__
#define __PHOSH_IDLE_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.Mutter.IdleMonitor */

#define PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR (phosh_idle_dbus_idle_monitor_get_type ())
#define PHOSH_IDLE_DBUS_IDLE_MONITOR(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR, PhoshIdleDBusIdleMonitor))
#define PHOSH_IDLE_DBUS_IS_IDLE_MONITOR(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR))
#define PHOSH_IDLE_DBUS_IDLE_MONITOR_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR, PhoshIdleDBusIdleMonitorIface))

struct _PhoshIdleDBusIdleMonitor;
typedef struct _PhoshIdleDBusIdleMonitor PhoshIdleDBusIdleMonitor;
typedef struct _PhoshIdleDBusIdleMonitorIface PhoshIdleDBusIdleMonitorIface;

struct _PhoshIdleDBusIdleMonitorIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_add_idle_watch) (
    PhoshIdleDBusIdleMonitor *object,
    GDBusMethodInvocation *invocation,
    guint64 arg_interval);

  gboolean (*handle_add_user_active_watch) (
    PhoshIdleDBusIdleMonitor *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_get_idletime) (
    PhoshIdleDBusIdleMonitor *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_remove_watch) (
    PhoshIdleDBusIdleMonitor *object,
    GDBusMethodInvocation *invocation,
    guint arg_id);

  void (*watch_fired) (
    PhoshIdleDBusIdleMonitor *object,
    guint arg_id);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshIdleDBusIdleMonitor, g_object_unref)
#endif

GType phosh_idle_dbus_idle_monitor_get_type (void);

GDBusInterfaceInfo *phosh_idle_dbus_idle_monitor_interface_info (void);
guint phosh_idle_dbus_idle_monitor_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void phosh_idle_dbus_idle_monitor_complete_get_idletime (
    PhoshIdleDBusIdleMonitor *object,
    GDBusMethodInvocation *invocation,
    guint64 idletime);

void phosh_idle_dbus_idle_monitor_complete_add_idle_watch (
    PhoshIdleDBusIdleMonitor *object,
    GDBusMethodInvocation *invocation,
    guint id);

void phosh_idle_dbus_idle_monitor_complete_add_user_active_watch (
    PhoshIdleDBusIdleMonitor *object,
    GDBusMethodInvocation *invocation,
    guint id);

void phosh_idle_dbus_idle_monitor_complete_remove_watch (
    PhoshIdleDBusIdleMonitor *object,
    GDBusMethodInvocation *invocation);



/* D-Bus signal emissions functions: */
void phosh_idle_dbus_idle_monitor_emit_watch_fired (
    PhoshIdleDBusIdleMonitor *object,
    guint arg_id);



/* D-Bus method calls: */
void phosh_idle_dbus_idle_monitor_call_get_idletime (
    PhoshIdleDBusIdleMonitor *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_idle_dbus_idle_monitor_call_get_idletime_finish (
    PhoshIdleDBusIdleMonitor *proxy,
    guint64* out_idletime,
    GAsyncResult *res,
    GError **error);

gboolean phosh_idle_dbus_idle_monitor_call_get_idletime_sync (
    PhoshIdleDBusIdleMonitor *proxy,
    guint64* out_idletime,
    GCancellable *cancellable,
    GError **error);

void phosh_idle_dbus_idle_monitor_call_add_idle_watch (
    PhoshIdleDBusIdleMonitor *proxy,
    guint64 arg_interval,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_idle_dbus_idle_monitor_call_add_idle_watch_finish (
    PhoshIdleDBusIdleMonitor *proxy,
    guint* out_id,
    GAsyncResult *res,
    GError **error);

gboolean phosh_idle_dbus_idle_monitor_call_add_idle_watch_sync (
    PhoshIdleDBusIdleMonitor *proxy,
    guint64 arg_interval,
    guint* out_id,
    GCancellable *cancellable,
    GError **error);

void phosh_idle_dbus_idle_monitor_call_add_user_active_watch (
    PhoshIdleDBusIdleMonitor *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_idle_dbus_idle_monitor_call_add_user_active_watch_finish (
    PhoshIdleDBusIdleMonitor *proxy,
    guint* out_id,
    GAsyncResult *res,
    GError **error);

gboolean phosh_idle_dbus_idle_monitor_call_add_user_active_watch_sync (
    PhoshIdleDBusIdleMonitor *proxy,
    guint* out_id,
    GCancellable *cancellable,
    GError **error);

void phosh_idle_dbus_idle_monitor_call_remove_watch (
    PhoshIdleDBusIdleMonitor *proxy,
    guint arg_id,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_idle_dbus_idle_monitor_call_remove_watch_finish (
    PhoshIdleDBusIdleMonitor *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_idle_dbus_idle_monitor_call_remove_watch_sync (
    PhoshIdleDBusIdleMonitor *proxy,
    guint arg_id,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR_PROXY (phosh_idle_dbus_idle_monitor_proxy_get_type ())
#define PHOSH_IDLE_DBUS_IDLE_MONITOR_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR_PROXY, PhoshIdleDBusIdleMonitorProxy))
#define PHOSH_IDLE_DBUS_IDLE_MONITOR_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR_PROXY, PhoshIdleDBusIdleMonitorProxyClass))
#define PHOSH_IDLE_DBUS_IDLE_MONITOR_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR_PROXY, PhoshIdleDBusIdleMonitorProxyClass))
#define PHOSH_IDLE_DBUS_IS_IDLE_MONITOR_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR_PROXY))
#define PHOSH_IDLE_DBUS_IS_IDLE_MONITOR_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR_PROXY))

typedef struct _PhoshIdleDBusIdleMonitorProxy PhoshIdleDBusIdleMonitorProxy;
typedef struct _PhoshIdleDBusIdleMonitorProxyClass PhoshIdleDBusIdleMonitorProxyClass;
typedef struct _PhoshIdleDBusIdleMonitorProxyPrivate PhoshIdleDBusIdleMonitorProxyPrivate;

struct _PhoshIdleDBusIdleMonitorProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshIdleDBusIdleMonitorProxyPrivate *priv;
};

struct _PhoshIdleDBusIdleMonitorProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_idle_dbus_idle_monitor_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshIdleDBusIdleMonitorProxy, g_object_unref)
#endif

void phosh_idle_dbus_idle_monitor_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshIdleDBusIdleMonitor *phosh_idle_dbus_idle_monitor_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshIdleDBusIdleMonitor *phosh_idle_dbus_idle_monitor_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_idle_dbus_idle_monitor_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshIdleDBusIdleMonitor *phosh_idle_dbus_idle_monitor_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshIdleDBusIdleMonitor *phosh_idle_dbus_idle_monitor_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR_SKELETON (phosh_idle_dbus_idle_monitor_skeleton_get_type ())
#define PHOSH_IDLE_DBUS_IDLE_MONITOR_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR_SKELETON, PhoshIdleDBusIdleMonitorSkeleton))
#define PHOSH_IDLE_DBUS_IDLE_MONITOR_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR_SKELETON, PhoshIdleDBusIdleMonitorSkeletonClass))
#define PHOSH_IDLE_DBUS_IDLE_MONITOR_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR_SKELETON, PhoshIdleDBusIdleMonitorSkeletonClass))
#define PHOSH_IDLE_DBUS_IS_IDLE_MONITOR_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR_SKELETON))
#define PHOSH_IDLE_DBUS_IS_IDLE_MONITOR_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_IDLE_DBUS_TYPE_IDLE_MONITOR_SKELETON))

typedef struct _PhoshIdleDBusIdleMonitorSkeleton PhoshIdleDBusIdleMonitorSkeleton;
typedef struct _PhoshIdleDBusIdleMonitorSkeletonClass PhoshIdleDBusIdleMonitorSkeletonClass;
typedef struct _PhoshIdleDBusIdleMonitorSkeletonPrivate PhoshIdleDBusIdleMonitorSkeletonPrivate;

struct _PhoshIdleDBusIdleMonitorSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshIdleDBusIdleMonitorSkeletonPrivate *priv;
};

struct _PhoshIdleDBusIdleMonitorSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_idle_dbus_idle_monitor_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshIdleDBusIdleMonitorSkeleton, g_object_unref)
#endif

PhoshIdleDBusIdleMonitor *phosh_idle_dbus_idle_monitor_skeleton_new (void);


/* ---- */

#define PHOSH_IDLE_DBUS_TYPE_OBJECT (phosh_idle_dbus_object_get_type ())
#define PHOSH_IDLE_DBUS_OBJECT(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_IDLE_DBUS_TYPE_OBJECT, PhoshIdleDBusObject))
#define PHOSH_IDLE_DBUS_IS_OBJECT(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_IDLE_DBUS_TYPE_OBJECT))
#define PHOSH_IDLE_DBUS_OBJECT_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_IDLE_DBUS_TYPE_OBJECT, PhoshIdleDBusObject))

struct _PhoshIdleDBusObject;
typedef struct _PhoshIdleDBusObject PhoshIdleDBusObject;
typedef struct _PhoshIdleDBusObjectIface PhoshIdleDBusObjectIface;

struct _PhoshIdleDBusObjectIface
{
  GTypeInterface parent_iface;
};

GType phosh_idle_dbus_object_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshIdleDBusObject, g_object_unref)
#endif

PhoshIdleDBusIdleMonitor *phosh_idle_dbus_object_get_idle_monitor (PhoshIdleDBusObject *object);
PhoshIdleDBusIdleMonitor *phosh_idle_dbus_object_peek_idle_monitor (PhoshIdleDBusObject *object);

#define PHOSH_IDLE_DBUS_TYPE_OBJECT_PROXY (phosh_idle_dbus_object_proxy_get_type ())
#define PHOSH_IDLE_DBUS_OBJECT_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_IDLE_DBUS_TYPE_OBJECT_PROXY, PhoshIdleDBusObjectProxy))
#define PHOSH_IDLE_DBUS_OBJECT_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_IDLE_DBUS_TYPE_OBJECT_PROXY, PhoshIdleDBusObjectProxyClass))
#define PHOSH_IDLE_DBUS_OBJECT_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_IDLE_DBUS_TYPE_OBJECT_PROXY, PhoshIdleDBusObjectProxyClass))
#define PHOSH_IDLE_DBUS_IS_OBJECT_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_IDLE_DBUS_TYPE_OBJECT_PROXY))
#define PHOSH_IDLE_DBUS_IS_OBJECT_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_IDLE_DBUS_TYPE_OBJECT_PROXY))

typedef struct _PhoshIdleDBusObjectProxy PhoshIdleDBusObjectProxy;
typedef struct _PhoshIdleDBusObjectProxyClass PhoshIdleDBusObjectProxyClass;
typedef struct _PhoshIdleDBusObjectProxyPrivate PhoshIdleDBusObjectProxyPrivate;

struct _PhoshIdleDBusObjectProxy
{
  /*< private >*/
  GDBusObjectProxy parent_instance;
  PhoshIdleDBusObjectProxyPrivate *priv;
};

struct _PhoshIdleDBusObjectProxyClass
{
  GDBusObjectProxyClass parent_class;
};

GType phosh_idle_dbus_object_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshIdleDBusObjectProxy, g_object_unref)
#endif

PhoshIdleDBusObjectProxy *phosh_idle_dbus_object_proxy_new (GDBusConnection *connection, const gchar *object_path);

#define PHOSH_IDLE_DBUS_TYPE_OBJECT_SKELETON (phosh_idle_dbus_object_skeleton_get_type ())
#define PHOSH_IDLE_DBUS_OBJECT_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_IDLE_DBUS_TYPE_OBJECT_SKELETON, PhoshIdleDBusObjectSkeleton))
#define PHOSH_IDLE_DBUS_OBJECT_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_IDLE_DBUS_TYPE_OBJECT_SKELETON, PhoshIdleDBusObjectSkeletonClass))
#define PHOSH_IDLE_DBUS_OBJECT_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_IDLE_DBUS_TYPE_OBJECT_SKELETON, PhoshIdleDBusObjectSkeletonClass))
#define PHOSH_IDLE_DBUS_IS_OBJECT_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_IDLE_DBUS_TYPE_OBJECT_SKELETON))
#define PHOSH_IDLE_DBUS_IS_OBJECT_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_IDLE_DBUS_TYPE_OBJECT_SKELETON))

typedef struct _PhoshIdleDBusObjectSkeleton PhoshIdleDBusObjectSkeleton;
typedef struct _PhoshIdleDBusObjectSkeletonClass PhoshIdleDBusObjectSkeletonClass;
typedef struct _PhoshIdleDBusObjectSkeletonPrivate PhoshIdleDBusObjectSkeletonPrivate;

struct _PhoshIdleDBusObjectSkeleton
{
  /*< private >*/
  GDBusObjectSkeleton parent_instance;
  PhoshIdleDBusObjectSkeletonPrivate *priv;
};

struct _PhoshIdleDBusObjectSkeletonClass
{
  GDBusObjectSkeletonClass parent_class;
};

GType phosh_idle_dbus_object_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshIdleDBusObjectSkeleton, g_object_unref)
#endif

PhoshIdleDBusObjectSkeleton *phosh_idle_dbus_object_skeleton_new (const gchar *object_path);
void phosh_idle_dbus_object_skeleton_set_idle_monitor (PhoshIdleDBusObjectSkeleton *object, PhoshIdleDBusIdleMonitor *interface_);

/* ---- */

#define PHOSH_IDLE_DBUS_TYPE_OBJECT_MANAGER_CLIENT (phosh_idle_dbus_object_manager_client_get_type ())
#define PHOSH_IDLE_DBUS_OBJECT_MANAGER_CLIENT(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_IDLE_DBUS_TYPE_OBJECT_MANAGER_CLIENT, PhoshIdleDBusObjectManagerClient))
#define PHOSH_IDLE_DBUS_OBJECT_MANAGER_CLIENT_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_IDLE_DBUS_TYPE_OBJECT_MANAGER_CLIENT, PhoshIdleDBusObjectManagerClientClass))
#define PHOSH_IDLE_DBUS_OBJECT_MANAGER_CLIENT_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_IDLE_DBUS_TYPE_OBJECT_MANAGER_CLIENT, PhoshIdleDBusObjectManagerClientClass))
#define PHOSH_IDLE_DBUS_IS_OBJECT_MANAGER_CLIENT(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_IDLE_DBUS_TYPE_OBJECT_MANAGER_CLIENT))
#define PHOSH_IDLE_DBUS_IS_OBJECT_MANAGER_CLIENT_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_IDLE_DBUS_TYPE_OBJECT_MANAGER_CLIENT))

typedef struct _PhoshIdleDBusObjectManagerClient PhoshIdleDBusObjectManagerClient;
typedef struct _PhoshIdleDBusObjectManagerClientClass PhoshIdleDBusObjectManagerClientClass;
typedef struct _PhoshIdleDBusObjectManagerClientPrivate PhoshIdleDBusObjectManagerClientPrivate;

struct _PhoshIdleDBusObjectManagerClient
{
  /*< private >*/
  GDBusObjectManagerClient parent_instance;
  PhoshIdleDBusObjectManagerClientPrivate *priv;
};

struct _PhoshIdleDBusObjectManagerClientClass
{
  GDBusObjectManagerClientClass parent_class;
};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshIdleDBusObjectManagerClient, g_object_unref)
#endif

GType phosh_idle_dbus_object_manager_client_get_type (void);

GType phosh_idle_dbus_object_manager_client_get_proxy_type (GDBusObjectManagerClient *manager, const gchar *object_path, const gchar *interface_name, gpointer user_data);

void phosh_idle_dbus_object_manager_client_new (
    GDBusConnection        *connection,
    GDBusObjectManagerClientFlags  flags,
    const gchar            *name,
    const gchar            *object_path,
    GCancellable           *cancellable,
    GAsyncReadyCallback     callback,
    gpointer                user_data);
GDBusObjectManager *phosh_idle_dbus_object_manager_client_new_finish (
    GAsyncResult        *res,
    GError             **error);
GDBusObjectManager *phosh_idle_dbus_object_manager_client_new_sync (
    GDBusConnection        *connection,
    GDBusObjectManagerClientFlags  flags,
    const gchar            *name,
    const gchar            *object_path,
    GCancellable           *cancellable,
    GError                **error);

void phosh_idle_dbus_object_manager_client_new_for_bus (
    GBusType                bus_type,
    GDBusObjectManagerClientFlags  flags,
    const gchar            *name,
    const gchar            *object_path,
    GCancellable           *cancellable,
    GAsyncReadyCallback     callback,
    gpointer                user_data);
GDBusObjectManager *phosh_idle_dbus_object_manager_client_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
GDBusObjectManager *phosh_idle_dbus_object_manager_client_new_for_bus_sync (
    GBusType                bus_type,
    GDBusObjectManagerClientFlags  flags,
    const gchar            *name,
    const gchar            *object_path,
    GCancellable           *cancellable,
    GError                **error);


G_END_DECLS

#endif /* __PHOSH_IDLE_DBUS_H__ */
