/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __GEOCLUE_MANAGER_DBUS_H__
#define __GEOCLUE_MANAGER_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.freedesktop.GeoClue2.Manager */

#define PHOSH_DBUS_TYPE_GEO_CLUE2_MANAGER (phosh_dbus_geo_clue2_manager_get_type ())
#define PHOSH_DBUS_GEO_CLUE2_MANAGER(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_GEO_CLUE2_MANAGER, PhoshDBusGeoClue2Manager))
#define PHOSH_DBUS_IS_GEO_CLUE2_MANAGER(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_GEO_CLUE2_MANAGER))
#define PHOSH_DBUS_GEO_CLUE2_MANAGER_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_GEO_CLUE2_MANAGER, PhoshDBusGeoClue2ManagerIface))

struct _PhoshDBusGeoClue2Manager;
typedef struct _PhoshDBusGeoClue2Manager PhoshDBusGeoClue2Manager;
typedef struct _PhoshDBusGeoClue2ManagerIface PhoshDBusGeoClue2ManagerIface;

struct _PhoshDBusGeoClue2ManagerIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_add_agent) (
    PhoshDBusGeoClue2Manager *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_id);

  guint  (*get_available_accuracy_level) (PhoshDBusGeoClue2Manager *object);

  gboolean  (*get_in_use) (PhoshDBusGeoClue2Manager *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusGeoClue2Manager, g_object_unref)
#endif

GType phosh_dbus_geo_clue2_manager_get_type (void);

GDBusInterfaceInfo *phosh_dbus_geo_clue2_manager_interface_info (void);
guint phosh_dbus_geo_clue2_manager_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void phosh_dbus_geo_clue2_manager_complete_add_agent (
    PhoshDBusGeoClue2Manager *object,
    GDBusMethodInvocation *invocation);



/* D-Bus method calls: */
void phosh_dbus_geo_clue2_manager_call_add_agent (
    PhoshDBusGeoClue2Manager *proxy,
    const gchar *arg_id,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_geo_clue2_manager_call_add_agent_finish (
    PhoshDBusGeoClue2Manager *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_geo_clue2_manager_call_add_agent_sync (
    PhoshDBusGeoClue2Manager *proxy,
    const gchar *arg_id,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
gboolean phosh_dbus_geo_clue2_manager_get_in_use (PhoshDBusGeoClue2Manager *object);
void phosh_dbus_geo_clue2_manager_set_in_use (PhoshDBusGeoClue2Manager *object, gboolean value);

guint phosh_dbus_geo_clue2_manager_get_available_accuracy_level (PhoshDBusGeoClue2Manager *object);
void phosh_dbus_geo_clue2_manager_set_available_accuracy_level (PhoshDBusGeoClue2Manager *object, guint value);


/* ---- */

#define PHOSH_DBUS_TYPE_GEO_CLUE2_MANAGER_PROXY (phosh_dbus_geo_clue2_manager_proxy_get_type ())
#define PHOSH_DBUS_GEO_CLUE2_MANAGER_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_GEO_CLUE2_MANAGER_PROXY, PhoshDBusGeoClue2ManagerProxy))
#define PHOSH_DBUS_GEO_CLUE2_MANAGER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_GEO_CLUE2_MANAGER_PROXY, PhoshDBusGeoClue2ManagerProxyClass))
#define PHOSH_DBUS_GEO_CLUE2_MANAGER_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_GEO_CLUE2_MANAGER_PROXY, PhoshDBusGeoClue2ManagerProxyClass))
#define PHOSH_DBUS_IS_GEO_CLUE2_MANAGER_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_GEO_CLUE2_MANAGER_PROXY))
#define PHOSH_DBUS_IS_GEO_CLUE2_MANAGER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_GEO_CLUE2_MANAGER_PROXY))

typedef struct _PhoshDBusGeoClue2ManagerProxy PhoshDBusGeoClue2ManagerProxy;
typedef struct _PhoshDBusGeoClue2ManagerProxyClass PhoshDBusGeoClue2ManagerProxyClass;
typedef struct _PhoshDBusGeoClue2ManagerProxyPrivate PhoshDBusGeoClue2ManagerProxyPrivate;

struct _PhoshDBusGeoClue2ManagerProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusGeoClue2ManagerProxyPrivate *priv;
};

struct _PhoshDBusGeoClue2ManagerProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_geo_clue2_manager_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusGeoClue2ManagerProxy, g_object_unref)
#endif

void phosh_dbus_geo_clue2_manager_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusGeoClue2Manager *phosh_dbus_geo_clue2_manager_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusGeoClue2Manager *phosh_dbus_geo_clue2_manager_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_geo_clue2_manager_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusGeoClue2Manager *phosh_dbus_geo_clue2_manager_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusGeoClue2Manager *phosh_dbus_geo_clue2_manager_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_GEO_CLUE2_MANAGER_SKELETON (phosh_dbus_geo_clue2_manager_skeleton_get_type ())
#define PHOSH_DBUS_GEO_CLUE2_MANAGER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_GEO_CLUE2_MANAGER_SKELETON, PhoshDBusGeoClue2ManagerSkeleton))
#define PHOSH_DBUS_GEO_CLUE2_MANAGER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_GEO_CLUE2_MANAGER_SKELETON, PhoshDBusGeoClue2ManagerSkeletonClass))
#define PHOSH_DBUS_GEO_CLUE2_MANAGER_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_GEO_CLUE2_MANAGER_SKELETON, PhoshDBusGeoClue2ManagerSkeletonClass))
#define PHOSH_DBUS_IS_GEO_CLUE2_MANAGER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_GEO_CLUE2_MANAGER_SKELETON))
#define PHOSH_DBUS_IS_GEO_CLUE2_MANAGER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_GEO_CLUE2_MANAGER_SKELETON))

typedef struct _PhoshDBusGeoClue2ManagerSkeleton PhoshDBusGeoClue2ManagerSkeleton;
typedef struct _PhoshDBusGeoClue2ManagerSkeletonClass PhoshDBusGeoClue2ManagerSkeletonClass;
typedef struct _PhoshDBusGeoClue2ManagerSkeletonPrivate PhoshDBusGeoClue2ManagerSkeletonPrivate;

struct _PhoshDBusGeoClue2ManagerSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusGeoClue2ManagerSkeletonPrivate *priv;
};

struct _PhoshDBusGeoClue2ManagerSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_geo_clue2_manager_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusGeoClue2ManagerSkeleton, g_object_unref)
#endif

PhoshDBusGeoClue2Manager *phosh_dbus_geo_clue2_manager_skeleton_new (void);


G_END_DECLS

#endif /* __GEOCLUE_MANAGER_DBUS_H__ */
