/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __GSD_COLOR_DBUS_H__
#define __GSD_COLOR_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.SettingsDaemon.Color */

#define PHOSH_DBUS_TYPE_COLOR (phosh_dbus_color_get_type ())
#define PHOSH_DBUS_COLOR(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_COLOR, PhoshDBusColor))
#define PHOSH_DBUS_IS_COLOR(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_COLOR))
#define PHOSH_DBUS_COLOR_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_COLOR, PhoshDBusColorIface))

struct _PhoshDBusColor;
typedef struct _PhoshDBusColor PhoshDBusColor;
typedef struct _PhoshDBusColorIface PhoshDBusColorIface;

struct _PhoshDBusColorIface
{
  GTypeInterface parent_iface;

  gboolean  (*get_night_light_active) (PhoshDBusColor *object);

  guint  (*get_temperature) (PhoshDBusColor *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusColor, g_object_unref)
#endif

GType phosh_dbus_color_get_type (void);

GDBusInterfaceInfo *phosh_dbus_color_interface_info (void);
guint phosh_dbus_color_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus property accessors: */
gboolean phosh_dbus_color_get_night_light_active (PhoshDBusColor *object);
void phosh_dbus_color_set_night_light_active (PhoshDBusColor *object, gboolean value);

guint phosh_dbus_color_get_temperature (PhoshDBusColor *object);
void phosh_dbus_color_set_temperature (PhoshDBusColor *object, guint value);


/* ---- */

#define PHOSH_DBUS_TYPE_COLOR_PROXY (phosh_dbus_color_proxy_get_type ())
#define PHOSH_DBUS_COLOR_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_COLOR_PROXY, PhoshDBusColorProxy))
#define PHOSH_DBUS_COLOR_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_COLOR_PROXY, PhoshDBusColorProxyClass))
#define PHOSH_DBUS_COLOR_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_COLOR_PROXY, PhoshDBusColorProxyClass))
#define PHOSH_DBUS_IS_COLOR_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_COLOR_PROXY))
#define PHOSH_DBUS_IS_COLOR_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_COLOR_PROXY))

typedef struct _PhoshDBusColorProxy PhoshDBusColorProxy;
typedef struct _PhoshDBusColorProxyClass PhoshDBusColorProxyClass;
typedef struct _PhoshDBusColorProxyPrivate PhoshDBusColorProxyPrivate;

struct _PhoshDBusColorProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusColorProxyPrivate *priv;
};

struct _PhoshDBusColorProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_color_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusColorProxy, g_object_unref)
#endif

void phosh_dbus_color_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusColor *phosh_dbus_color_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusColor *phosh_dbus_color_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_color_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusColor *phosh_dbus_color_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusColor *phosh_dbus_color_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_COLOR_SKELETON (phosh_dbus_color_skeleton_get_type ())
#define PHOSH_DBUS_COLOR_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_COLOR_SKELETON, PhoshDBusColorSkeleton))
#define PHOSH_DBUS_COLOR_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_COLOR_SKELETON, PhoshDBusColorSkeletonClass))
#define PHOSH_DBUS_COLOR_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_COLOR_SKELETON, PhoshDBusColorSkeletonClass))
#define PHOSH_DBUS_IS_COLOR_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_COLOR_SKELETON))
#define PHOSH_DBUS_IS_COLOR_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_COLOR_SKELETON))

typedef struct _PhoshDBusColorSkeleton PhoshDBusColorSkeleton;
typedef struct _PhoshDBusColorSkeletonClass PhoshDBusColorSkeletonClass;
typedef struct _PhoshDBusColorSkeletonPrivate PhoshDBusColorSkeletonPrivate;

struct _PhoshDBusColorSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusColorSkeletonPrivate *priv;
};

struct _PhoshDBusColorSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_color_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusColorSkeleton, g_object_unref)
#endif

PhoshDBusColor *phosh_dbus_color_skeleton_new (void);


G_END_DECLS

#endif /* __GSD_COLOR_DBUS_H__ */
