/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __PHOSH_GNOME_SHELL_DBUS_H__
#define __PHOSH_GNOME_SHELL_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.Shell */

#define PHOSH_DBUS_TYPE_GNOME_SHELL (phosh_dbus_gnome_shell_get_type ())
#define PHOSH_DBUS_GNOME_SHELL(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_GNOME_SHELL, PhoshDBusGnomeShell))
#define PHOSH_DBUS_IS_GNOME_SHELL(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_GNOME_SHELL))
#define PHOSH_DBUS_GNOME_SHELL_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_GNOME_SHELL, PhoshDBusGnomeShellIface))

struct _PhoshDBusGnomeShell;
typedef struct _PhoshDBusGnomeShell PhoshDBusGnomeShell;
typedef struct _PhoshDBusGnomeShellIface PhoshDBusGnomeShellIface;

struct _PhoshDBusGnomeShellIface
{
  GTypeInterface parent_iface;



  gboolean (*handle_grab_accelerator) (
    PhoshDBusGnomeShell *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_accelerator,
    guint arg_modeFlags,
    guint arg_grabFlags);

  gboolean (*handle_grab_accelerators) (
    PhoshDBusGnomeShell *object,
    GDBusMethodInvocation *invocation,
    GVariant *arg_accelerators);

  gboolean (*handle_hide_monitor_labels) (
    PhoshDBusGnomeShell *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_show_monitor_labels) (
    PhoshDBusGnomeShell *object,
    GDBusMethodInvocation *invocation,
    GVariant *arg_params);

  gboolean (*handle_show_osd) (
    PhoshDBusGnomeShell *object,
    GDBusMethodInvocation *invocation,
    GVariant *arg_params);

  gboolean (*handle_ungrab_accelerator) (
    PhoshDBusGnomeShell *object,
    GDBusMethodInvocation *invocation,
    guint arg_action);

  gboolean (*handle_ungrab_accelerators) (
    PhoshDBusGnomeShell *object,
    GDBusMethodInvocation *invocation,
    GVariant *arg_action);

  gboolean  (*get_overview_active) (PhoshDBusGnomeShell *object);

  const gchar * (*get_shell_version) (PhoshDBusGnomeShell *object);

  void (*accelerator_activated) (
    PhoshDBusGnomeShell *object,
    guint arg_action,
    GVariant *arg_parameters);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusGnomeShell, g_object_unref)
#endif

GType phosh_dbus_gnome_shell_get_type (void);

GDBusInterfaceInfo *phosh_dbus_gnome_shell_interface_info (void);
guint phosh_dbus_gnome_shell_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void phosh_dbus_gnome_shell_complete_show_monitor_labels (
    PhoshDBusGnomeShell *object,
    GDBusMethodInvocation *invocation);

void phosh_dbus_gnome_shell_complete_hide_monitor_labels (
    PhoshDBusGnomeShell *object,
    GDBusMethodInvocation *invocation);

void phosh_dbus_gnome_shell_complete_show_osd (
    PhoshDBusGnomeShell *object,
    GDBusMethodInvocation *invocation);

void phosh_dbus_gnome_shell_complete_grab_accelerator (
    PhoshDBusGnomeShell *object,
    GDBusMethodInvocation *invocation,
    guint action);

void phosh_dbus_gnome_shell_complete_grab_accelerators (
    PhoshDBusGnomeShell *object,
    GDBusMethodInvocation *invocation,
    GVariant *actions);

void phosh_dbus_gnome_shell_complete_ungrab_accelerator (
    PhoshDBusGnomeShell *object,
    GDBusMethodInvocation *invocation,
    gboolean success);

void phosh_dbus_gnome_shell_complete_ungrab_accelerators (
    PhoshDBusGnomeShell *object,
    GDBusMethodInvocation *invocation,
    gboolean success);



/* D-Bus signal emissions functions: */
void phosh_dbus_gnome_shell_emit_accelerator_activated (
    PhoshDBusGnomeShell *object,
    guint arg_action,
    GVariant *arg_parameters);



/* D-Bus method calls: */
void phosh_dbus_gnome_shell_call_show_monitor_labels (
    PhoshDBusGnomeShell *proxy,
    GVariant *arg_params,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_gnome_shell_call_show_monitor_labels_finish (
    PhoshDBusGnomeShell *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_gnome_shell_call_show_monitor_labels_sync (
    PhoshDBusGnomeShell *proxy,
    GVariant *arg_params,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_gnome_shell_call_hide_monitor_labels (
    PhoshDBusGnomeShell *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_gnome_shell_call_hide_monitor_labels_finish (
    PhoshDBusGnomeShell *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_gnome_shell_call_hide_monitor_labels_sync (
    PhoshDBusGnomeShell *proxy,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_gnome_shell_call_show_osd (
    PhoshDBusGnomeShell *proxy,
    GVariant *arg_params,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_gnome_shell_call_show_osd_finish (
    PhoshDBusGnomeShell *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_gnome_shell_call_show_osd_sync (
    PhoshDBusGnomeShell *proxy,
    GVariant *arg_params,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_gnome_shell_call_grab_accelerator (
    PhoshDBusGnomeShell *proxy,
    const gchar *arg_accelerator,
    guint arg_modeFlags,
    guint arg_grabFlags,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_gnome_shell_call_grab_accelerator_finish (
    PhoshDBusGnomeShell *proxy,
    guint* out_action,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_gnome_shell_call_grab_accelerator_sync (
    PhoshDBusGnomeShell *proxy,
    const gchar *arg_accelerator,
    guint arg_modeFlags,
    guint arg_grabFlags,
    guint* out_action,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_gnome_shell_call_grab_accelerators (
    PhoshDBusGnomeShell *proxy,
    GVariant *arg_accelerators,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_gnome_shell_call_grab_accelerators_finish (
    PhoshDBusGnomeShell *proxy,
    GVariant ** out_actions,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_gnome_shell_call_grab_accelerators_sync (
    PhoshDBusGnomeShell *proxy,
    GVariant *arg_accelerators,
    GVariant ** out_actions,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_gnome_shell_call_ungrab_accelerator (
    PhoshDBusGnomeShell *proxy,
    guint arg_action,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_gnome_shell_call_ungrab_accelerator_finish (
    PhoshDBusGnomeShell *proxy,
    gboolean* out_success,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_gnome_shell_call_ungrab_accelerator_sync (
    PhoshDBusGnomeShell *proxy,
    guint arg_action,
    gboolean* out_success,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_gnome_shell_call_ungrab_accelerators (
    PhoshDBusGnomeShell *proxy,
    GVariant *arg_action,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_gnome_shell_call_ungrab_accelerators_finish (
    PhoshDBusGnomeShell *proxy,
    gboolean* out_success,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_gnome_shell_call_ungrab_accelerators_sync (
    PhoshDBusGnomeShell *proxy,
    GVariant *arg_action,
    gboolean* out_success,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
gboolean phosh_dbus_gnome_shell_get_overview_active (PhoshDBusGnomeShell *object);
void phosh_dbus_gnome_shell_set_overview_active (PhoshDBusGnomeShell *object, gboolean value);

const gchar *phosh_dbus_gnome_shell_get_shell_version (PhoshDBusGnomeShell *object);
gchar *phosh_dbus_gnome_shell_dup_shell_version (PhoshDBusGnomeShell *object);
void phosh_dbus_gnome_shell_set_shell_version (PhoshDBusGnomeShell *object, const gchar *value);


/* ---- */

#define PHOSH_DBUS_TYPE_GNOME_SHELL_PROXY (phosh_dbus_gnome_shell_proxy_get_type ())
#define PHOSH_DBUS_GNOME_SHELL_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_GNOME_SHELL_PROXY, PhoshDBusGnomeShellProxy))
#define PHOSH_DBUS_GNOME_SHELL_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_GNOME_SHELL_PROXY, PhoshDBusGnomeShellProxyClass))
#define PHOSH_DBUS_GNOME_SHELL_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_GNOME_SHELL_PROXY, PhoshDBusGnomeShellProxyClass))
#define PHOSH_DBUS_IS_GNOME_SHELL_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_GNOME_SHELL_PROXY))
#define PHOSH_DBUS_IS_GNOME_SHELL_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_GNOME_SHELL_PROXY))

typedef struct _PhoshDBusGnomeShellProxy PhoshDBusGnomeShellProxy;
typedef struct _PhoshDBusGnomeShellProxyClass PhoshDBusGnomeShellProxyClass;
typedef struct _PhoshDBusGnomeShellProxyPrivate PhoshDBusGnomeShellProxyPrivate;

struct _PhoshDBusGnomeShellProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusGnomeShellProxyPrivate *priv;
};

struct _PhoshDBusGnomeShellProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_gnome_shell_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusGnomeShellProxy, g_object_unref)
#endif

void phosh_dbus_gnome_shell_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusGnomeShell *phosh_dbus_gnome_shell_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusGnomeShell *phosh_dbus_gnome_shell_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_gnome_shell_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusGnomeShell *phosh_dbus_gnome_shell_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusGnomeShell *phosh_dbus_gnome_shell_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_GNOME_SHELL_SKELETON (phosh_dbus_gnome_shell_skeleton_get_type ())
#define PHOSH_DBUS_GNOME_SHELL_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_GNOME_SHELL_SKELETON, PhoshDBusGnomeShellSkeleton))
#define PHOSH_DBUS_GNOME_SHELL_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_GNOME_SHELL_SKELETON, PhoshDBusGnomeShellSkeletonClass))
#define PHOSH_DBUS_GNOME_SHELL_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_GNOME_SHELL_SKELETON, PhoshDBusGnomeShellSkeletonClass))
#define PHOSH_DBUS_IS_GNOME_SHELL_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_GNOME_SHELL_SKELETON))
#define PHOSH_DBUS_IS_GNOME_SHELL_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_GNOME_SHELL_SKELETON))

typedef struct _PhoshDBusGnomeShellSkeleton PhoshDBusGnomeShellSkeleton;
typedef struct _PhoshDBusGnomeShellSkeletonClass PhoshDBusGnomeShellSkeletonClass;
typedef struct _PhoshDBusGnomeShellSkeletonPrivate PhoshDBusGnomeShellSkeletonPrivate;

struct _PhoshDBusGnomeShellSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusGnomeShellSkeletonPrivate *priv;
};

struct _PhoshDBusGnomeShellSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_gnome_shell_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusGnomeShellSkeleton, g_object_unref)
#endif

PhoshDBusGnomeShell *phosh_dbus_gnome_shell_skeleton_new (void);


G_END_DECLS

#endif /* __PHOSH_GNOME_SHELL_DBUS_H__ */
