/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __PHOSH_SCREEN_SAVER_DBUS_H__
#define __PHOSH_SCREEN_SAVER_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.ScreenSaver */

#define PHOSH_DBUS_TYPE_SCREEN_SAVER (phosh_dbus_screen_saver_get_type ())
#define PHOSH_DBUS_SCREEN_SAVER(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_SCREEN_SAVER, PhoshDBusScreenSaver))
#define PHOSH_DBUS_IS_SCREEN_SAVER(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_SCREEN_SAVER))
#define PHOSH_DBUS_SCREEN_SAVER_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_SCREEN_SAVER, PhoshDBusScreenSaverIface))

struct _PhoshDBusScreenSaver;
typedef struct _PhoshDBusScreenSaver PhoshDBusScreenSaver;
typedef struct _PhoshDBusScreenSaverIface PhoshDBusScreenSaverIface;

struct _PhoshDBusScreenSaverIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_get_active) (
    PhoshDBusScreenSaver *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_get_active_time) (
    PhoshDBusScreenSaver *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_lock) (
    PhoshDBusScreenSaver *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_set_active) (
    PhoshDBusScreenSaver *object,
    GDBusMethodInvocation *invocation,
    gboolean arg_value);

  void (*active_changed) (
    PhoshDBusScreenSaver *object,
    gboolean arg_new_value);

  void (*wake_up_screen) (
    PhoshDBusScreenSaver *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusScreenSaver, g_object_unref)
#endif

GType phosh_dbus_screen_saver_get_type (void);

GDBusInterfaceInfo *phosh_dbus_screen_saver_interface_info (void);
guint phosh_dbus_screen_saver_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void phosh_dbus_screen_saver_complete_lock (
    PhoshDBusScreenSaver *object,
    GDBusMethodInvocation *invocation);

void phosh_dbus_screen_saver_complete_get_active (
    PhoshDBusScreenSaver *object,
    GDBusMethodInvocation *invocation,
    gboolean active);

void phosh_dbus_screen_saver_complete_set_active (
    PhoshDBusScreenSaver *object,
    GDBusMethodInvocation *invocation);

void phosh_dbus_screen_saver_complete_get_active_time (
    PhoshDBusScreenSaver *object,
    GDBusMethodInvocation *invocation,
    guint value);



/* D-Bus signal emissions functions: */
void phosh_dbus_screen_saver_emit_active_changed (
    PhoshDBusScreenSaver *object,
    gboolean arg_new_value);

void phosh_dbus_screen_saver_emit_wake_up_screen (
    PhoshDBusScreenSaver *object);



/* D-Bus method calls: */
void phosh_dbus_screen_saver_call_lock (
    PhoshDBusScreenSaver *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_screen_saver_call_lock_finish (
    PhoshDBusScreenSaver *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_screen_saver_call_lock_sync (
    PhoshDBusScreenSaver *proxy,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_screen_saver_call_get_active (
    PhoshDBusScreenSaver *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_screen_saver_call_get_active_finish (
    PhoshDBusScreenSaver *proxy,
    gboolean* out_active,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_screen_saver_call_get_active_sync (
    PhoshDBusScreenSaver *proxy,
    gboolean* out_active,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_screen_saver_call_set_active (
    PhoshDBusScreenSaver *proxy,
    gboolean arg_value,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_screen_saver_call_set_active_finish (
    PhoshDBusScreenSaver *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_screen_saver_call_set_active_sync (
    PhoshDBusScreenSaver *proxy,
    gboolean arg_value,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_screen_saver_call_get_active_time (
    PhoshDBusScreenSaver *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_screen_saver_call_get_active_time_finish (
    PhoshDBusScreenSaver *proxy,
    guint* out_value,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_screen_saver_call_get_active_time_sync (
    PhoshDBusScreenSaver *proxy,
    guint* out_value,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define PHOSH_DBUS_TYPE_SCREEN_SAVER_PROXY (phosh_dbus_screen_saver_proxy_get_type ())
#define PHOSH_DBUS_SCREEN_SAVER_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_SCREEN_SAVER_PROXY, PhoshDBusScreenSaverProxy))
#define PHOSH_DBUS_SCREEN_SAVER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_SCREEN_SAVER_PROXY, PhoshDBusScreenSaverProxyClass))
#define PHOSH_DBUS_SCREEN_SAVER_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_SCREEN_SAVER_PROXY, PhoshDBusScreenSaverProxyClass))
#define PHOSH_DBUS_IS_SCREEN_SAVER_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_SCREEN_SAVER_PROXY))
#define PHOSH_DBUS_IS_SCREEN_SAVER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_SCREEN_SAVER_PROXY))

typedef struct _PhoshDBusScreenSaverProxy PhoshDBusScreenSaverProxy;
typedef struct _PhoshDBusScreenSaverProxyClass PhoshDBusScreenSaverProxyClass;
typedef struct _PhoshDBusScreenSaverProxyPrivate PhoshDBusScreenSaverProxyPrivate;

struct _PhoshDBusScreenSaverProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusScreenSaverProxyPrivate *priv;
};

struct _PhoshDBusScreenSaverProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_screen_saver_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusScreenSaverProxy, g_object_unref)
#endif

void phosh_dbus_screen_saver_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusScreenSaver *phosh_dbus_screen_saver_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusScreenSaver *phosh_dbus_screen_saver_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_screen_saver_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusScreenSaver *phosh_dbus_screen_saver_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusScreenSaver *phosh_dbus_screen_saver_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_SCREEN_SAVER_SKELETON (phosh_dbus_screen_saver_skeleton_get_type ())
#define PHOSH_DBUS_SCREEN_SAVER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_SCREEN_SAVER_SKELETON, PhoshDBusScreenSaverSkeleton))
#define PHOSH_DBUS_SCREEN_SAVER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_SCREEN_SAVER_SKELETON, PhoshDBusScreenSaverSkeletonClass))
#define PHOSH_DBUS_SCREEN_SAVER_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_SCREEN_SAVER_SKELETON, PhoshDBusScreenSaverSkeletonClass))
#define PHOSH_DBUS_IS_SCREEN_SAVER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_SCREEN_SAVER_SKELETON))
#define PHOSH_DBUS_IS_SCREEN_SAVER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_SCREEN_SAVER_SKELETON))

typedef struct _PhoshDBusScreenSaverSkeleton PhoshDBusScreenSaverSkeleton;
typedef struct _PhoshDBusScreenSaverSkeletonClass PhoshDBusScreenSaverSkeletonClass;
typedef struct _PhoshDBusScreenSaverSkeletonPrivate PhoshDBusScreenSaverSkeletonPrivate;

struct _PhoshDBusScreenSaverSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusScreenSaverSkeletonPrivate *priv;
};

struct _PhoshDBusScreenSaverSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_screen_saver_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusScreenSaverSkeleton, g_object_unref)
#endif

PhoshDBusScreenSaver *phosh_dbus_screen_saver_skeleton_new (void);


G_END_DECLS

#endif /* __PHOSH_SCREEN_SAVER_DBUS_H__ */
