/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __GNOME_SESSION_CLIENT_PRIVATE_DBUS_H__
#define __GNOME_SESSION_CLIENT_PRIVATE_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.SessionManager.ClientPrivate */

#define PHOSH_DBUS_TYPE_SESSION_MANAGER_CLIENT_PRIVATE (phosh_dbus_session_manager_client_private_get_type ())
#define PHOSH_DBUS_SESSION_MANAGER_CLIENT_PRIVATE(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_SESSION_MANAGER_CLIENT_PRIVATE, PhoshDBusSessionManagerClientPrivate))
#define PHOSH_DBUS_IS_SESSION_MANAGER_CLIENT_PRIVATE(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_SESSION_MANAGER_CLIENT_PRIVATE))
#define PHOSH_DBUS_SESSION_MANAGER_CLIENT_PRIVATE_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_SESSION_MANAGER_CLIENT_PRIVATE, PhoshDBusSessionManagerClientPrivateIface))

struct _PhoshDBusSessionManagerClientPrivate;
typedef struct _PhoshDBusSessionManagerClientPrivate PhoshDBusSessionManagerClientPrivate;
typedef struct _PhoshDBusSessionManagerClientPrivateIface PhoshDBusSessionManagerClientPrivateIface;

struct _PhoshDBusSessionManagerClientPrivateIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_end_session_response) (
    PhoshDBusSessionManagerClientPrivate *object,
    GDBusMethodInvocation *invocation,
    gboolean arg_is_ok,
    const gchar *arg_reason);

  void (*cancel_end_session) (
    PhoshDBusSessionManagerClientPrivate *object);

  void (*end_session) (
    PhoshDBusSessionManagerClientPrivate *object,
    guint arg_flags);

  void (*query_end_session) (
    PhoshDBusSessionManagerClientPrivate *object,
    guint arg_flags);

  void (*stop) (
    PhoshDBusSessionManagerClientPrivate *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusSessionManagerClientPrivate, g_object_unref)
#endif

GType phosh_dbus_session_manager_client_private_get_type (void);

GDBusInterfaceInfo *phosh_dbus_session_manager_client_private_interface_info (void);
guint phosh_dbus_session_manager_client_private_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void phosh_dbus_session_manager_client_private_complete_end_session_response (
    PhoshDBusSessionManagerClientPrivate *object,
    GDBusMethodInvocation *invocation);



/* D-Bus signal emissions functions: */
void phosh_dbus_session_manager_client_private_emit_stop (
    PhoshDBusSessionManagerClientPrivate *object);

void phosh_dbus_session_manager_client_private_emit_query_end_session (
    PhoshDBusSessionManagerClientPrivate *object,
    guint arg_flags);

void phosh_dbus_session_manager_client_private_emit_end_session (
    PhoshDBusSessionManagerClientPrivate *object,
    guint arg_flags);

void phosh_dbus_session_manager_client_private_emit_cancel_end_session (
    PhoshDBusSessionManagerClientPrivate *object);



/* D-Bus method calls: */
void phosh_dbus_session_manager_client_private_call_end_session_response (
    PhoshDBusSessionManagerClientPrivate *proxy,
    gboolean arg_is_ok,
    const gchar *arg_reason,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_session_manager_client_private_call_end_session_response_finish (
    PhoshDBusSessionManagerClientPrivate *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_session_manager_client_private_call_end_session_response_sync (
    PhoshDBusSessionManagerClientPrivate *proxy,
    gboolean arg_is_ok,
    const gchar *arg_reason,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define PHOSH_DBUS_TYPE_SESSION_MANAGER_CLIENT_PRIVATE_PROXY (phosh_dbus_session_manager_client_private_proxy_get_type ())
#define PHOSH_DBUS_SESSION_MANAGER_CLIENT_PRIVATE_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_SESSION_MANAGER_CLIENT_PRIVATE_PROXY, PhoshDBusSessionManagerClientPrivateProxy))
#define PHOSH_DBUS_SESSION_MANAGER_CLIENT_PRIVATE_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_SESSION_MANAGER_CLIENT_PRIVATE_PROXY, PhoshDBusSessionManagerClientPrivateProxyClass))
#define PHOSH_DBUS_SESSION_MANAGER_CLIENT_PRIVATE_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_SESSION_MANAGER_CLIENT_PRIVATE_PROXY, PhoshDBusSessionManagerClientPrivateProxyClass))
#define PHOSH_DBUS_IS_SESSION_MANAGER_CLIENT_PRIVATE_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_SESSION_MANAGER_CLIENT_PRIVATE_PROXY))
#define PHOSH_DBUS_IS_SESSION_MANAGER_CLIENT_PRIVATE_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_SESSION_MANAGER_CLIENT_PRIVATE_PROXY))

typedef struct _PhoshDBusSessionManagerClientPrivateProxy PhoshDBusSessionManagerClientPrivateProxy;
typedef struct _PhoshDBusSessionManagerClientPrivateProxyClass PhoshDBusSessionManagerClientPrivateProxyClass;
typedef struct _PhoshDBusSessionManagerClientPrivateProxyPrivate PhoshDBusSessionManagerClientPrivateProxyPrivate;

struct _PhoshDBusSessionManagerClientPrivateProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusSessionManagerClientPrivateProxyPrivate *priv;
};

struct _PhoshDBusSessionManagerClientPrivateProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_session_manager_client_private_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusSessionManagerClientPrivateProxy, g_object_unref)
#endif

void phosh_dbus_session_manager_client_private_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusSessionManagerClientPrivate *phosh_dbus_session_manager_client_private_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusSessionManagerClientPrivate *phosh_dbus_session_manager_client_private_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_session_manager_client_private_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusSessionManagerClientPrivate *phosh_dbus_session_manager_client_private_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusSessionManagerClientPrivate *phosh_dbus_session_manager_client_private_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_SESSION_MANAGER_CLIENT_PRIVATE_SKELETON (phosh_dbus_session_manager_client_private_skeleton_get_type ())
#define PHOSH_DBUS_SESSION_MANAGER_CLIENT_PRIVATE_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_SESSION_MANAGER_CLIENT_PRIVATE_SKELETON, PhoshDBusSessionManagerClientPrivateSkeleton))
#define PHOSH_DBUS_SESSION_MANAGER_CLIENT_PRIVATE_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_SESSION_MANAGER_CLIENT_PRIVATE_SKELETON, PhoshDBusSessionManagerClientPrivateSkeletonClass))
#define PHOSH_DBUS_SESSION_MANAGER_CLIENT_PRIVATE_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_SESSION_MANAGER_CLIENT_PRIVATE_SKELETON, PhoshDBusSessionManagerClientPrivateSkeletonClass))
#define PHOSH_DBUS_IS_SESSION_MANAGER_CLIENT_PRIVATE_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_SESSION_MANAGER_CLIENT_PRIVATE_SKELETON))
#define PHOSH_DBUS_IS_SESSION_MANAGER_CLIENT_PRIVATE_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_SESSION_MANAGER_CLIENT_PRIVATE_SKELETON))

typedef struct _PhoshDBusSessionManagerClientPrivateSkeleton PhoshDBusSessionManagerClientPrivateSkeleton;
typedef struct _PhoshDBusSessionManagerClientPrivateSkeletonClass PhoshDBusSessionManagerClientPrivateSkeletonClass;
typedef struct _PhoshDBusSessionManagerClientPrivateSkeletonPrivate PhoshDBusSessionManagerClientPrivateSkeletonPrivate;

struct _PhoshDBusSessionManagerClientPrivateSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusSessionManagerClientPrivateSkeletonPrivate *priv;
};

struct _PhoshDBusSessionManagerClientPrivateSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_session_manager_client_private_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusSessionManagerClientPrivateSkeleton, g_object_unref)
#endif

PhoshDBusSessionManagerClientPrivate *phosh_dbus_session_manager_client_private_skeleton_new (void);


G_END_DECLS

#endif /* __GNOME_SESSION_CLIENT_PRIVATE_DBUS_H__ */
