/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __CALLS_DBUS_H__
#define __CALLS_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.Calls.Call */

#define PHOSH_DBUS_TYPE_CALLS_CALL (phosh_dbus_calls_call_get_type ())
#define PHOSH_DBUS_CALLS_CALL(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_CALLS_CALL, PhoshDBusCallsCall))
#define PHOSH_DBUS_IS_CALLS_CALL(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_CALLS_CALL))
#define PHOSH_DBUS_CALLS_CALL_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_CALLS_CALL, PhoshDBusCallsCallIface))

struct _PhoshDBusCallsCall;
typedef struct _PhoshDBusCallsCall PhoshDBusCallsCall;
typedef struct _PhoshDBusCallsCallIface PhoshDBusCallsCallIface;

struct _PhoshDBusCallsCallIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_accept) (
    PhoshDBusCallsCall *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_hangup) (
    PhoshDBusCallsCall *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_send_dtmf) (
    PhoshDBusCallsCall *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_Tone);

  gboolean  (*get_can_dtmf) (PhoshDBusCallsCall *object);

  const gchar * (*get_display_name) (PhoshDBusCallsCall *object);

  gboolean  (*get_encrypted) (PhoshDBusCallsCall *object);

  const gchar * (*get_id) (PhoshDBusCallsCall *object);

  const gchar * (*get_image_path) (PhoshDBusCallsCall *object);

  gboolean  (*get_inbound) (PhoshDBusCallsCall *object);

  const gchar * (*get_protocol) (PhoshDBusCallsCall *object);

  guint  (*get_state) (PhoshDBusCallsCall *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusCallsCall, g_object_unref)
#endif

GType phosh_dbus_calls_call_get_type (void);

GDBusInterfaceInfo *phosh_dbus_calls_call_interface_info (void);
guint phosh_dbus_calls_call_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void phosh_dbus_calls_call_complete_accept (
    PhoshDBusCallsCall *object,
    GDBusMethodInvocation *invocation);

void phosh_dbus_calls_call_complete_hangup (
    PhoshDBusCallsCall *object,
    GDBusMethodInvocation *invocation);

void phosh_dbus_calls_call_complete_send_dtmf (
    PhoshDBusCallsCall *object,
    GDBusMethodInvocation *invocation);



/* D-Bus method calls: */
void phosh_dbus_calls_call_call_accept (
    PhoshDBusCallsCall *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_calls_call_call_accept_finish (
    PhoshDBusCallsCall *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_calls_call_call_accept_sync (
    PhoshDBusCallsCall *proxy,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_calls_call_call_hangup (
    PhoshDBusCallsCall *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_calls_call_call_hangup_finish (
    PhoshDBusCallsCall *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_calls_call_call_hangup_sync (
    PhoshDBusCallsCall *proxy,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_calls_call_call_send_dtmf (
    PhoshDBusCallsCall *proxy,
    const gchar *arg_Tone,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_calls_call_call_send_dtmf_finish (
    PhoshDBusCallsCall *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_calls_call_call_send_dtmf_sync (
    PhoshDBusCallsCall *proxy,
    const gchar *arg_Tone,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
gboolean phosh_dbus_calls_call_get_inbound (PhoshDBusCallsCall *object);
void phosh_dbus_calls_call_set_inbound (PhoshDBusCallsCall *object, gboolean value);

guint phosh_dbus_calls_call_get_state (PhoshDBusCallsCall *object);
void phosh_dbus_calls_call_set_state (PhoshDBusCallsCall *object, guint value);

const gchar *phosh_dbus_calls_call_get_id (PhoshDBusCallsCall *object);
gchar *phosh_dbus_calls_call_dup_id (PhoshDBusCallsCall *object);
void phosh_dbus_calls_call_set_id (PhoshDBusCallsCall *object, const gchar *value);

const gchar *phosh_dbus_calls_call_get_display_name (PhoshDBusCallsCall *object);
gchar *phosh_dbus_calls_call_dup_display_name (PhoshDBusCallsCall *object);
void phosh_dbus_calls_call_set_display_name (PhoshDBusCallsCall *object, const gchar *value);

const gchar *phosh_dbus_calls_call_get_image_path (PhoshDBusCallsCall *object);
gchar *phosh_dbus_calls_call_dup_image_path (PhoshDBusCallsCall *object);
void phosh_dbus_calls_call_set_image_path (PhoshDBusCallsCall *object, const gchar *value);

const gchar *phosh_dbus_calls_call_get_protocol (PhoshDBusCallsCall *object);
gchar *phosh_dbus_calls_call_dup_protocol (PhoshDBusCallsCall *object);
void phosh_dbus_calls_call_set_protocol (PhoshDBusCallsCall *object, const gchar *value);

gboolean phosh_dbus_calls_call_get_encrypted (PhoshDBusCallsCall *object);
void phosh_dbus_calls_call_set_encrypted (PhoshDBusCallsCall *object, gboolean value);

gboolean phosh_dbus_calls_call_get_can_dtmf (PhoshDBusCallsCall *object);
void phosh_dbus_calls_call_set_can_dtmf (PhoshDBusCallsCall *object, gboolean value);


/* ---- */

#define PHOSH_DBUS_TYPE_CALLS_CALL_PROXY (phosh_dbus_calls_call_proxy_get_type ())
#define PHOSH_DBUS_CALLS_CALL_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_CALLS_CALL_PROXY, PhoshDBusCallsCallProxy))
#define PHOSH_DBUS_CALLS_CALL_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_CALLS_CALL_PROXY, PhoshDBusCallsCallProxyClass))
#define PHOSH_DBUS_CALLS_CALL_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_CALLS_CALL_PROXY, PhoshDBusCallsCallProxyClass))
#define PHOSH_DBUS_IS_CALLS_CALL_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_CALLS_CALL_PROXY))
#define PHOSH_DBUS_IS_CALLS_CALL_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_CALLS_CALL_PROXY))

typedef struct _PhoshDBusCallsCallProxy PhoshDBusCallsCallProxy;
typedef struct _PhoshDBusCallsCallProxyClass PhoshDBusCallsCallProxyClass;
typedef struct _PhoshDBusCallsCallProxyPrivate PhoshDBusCallsCallProxyPrivate;

struct _PhoshDBusCallsCallProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusCallsCallProxyPrivate *priv;
};

struct _PhoshDBusCallsCallProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_calls_call_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusCallsCallProxy, g_object_unref)
#endif

void phosh_dbus_calls_call_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusCallsCall *phosh_dbus_calls_call_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusCallsCall *phosh_dbus_calls_call_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_calls_call_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusCallsCall *phosh_dbus_calls_call_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusCallsCall *phosh_dbus_calls_call_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_CALLS_CALL_SKELETON (phosh_dbus_calls_call_skeleton_get_type ())
#define PHOSH_DBUS_CALLS_CALL_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_CALLS_CALL_SKELETON, PhoshDBusCallsCallSkeleton))
#define PHOSH_DBUS_CALLS_CALL_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_CALLS_CALL_SKELETON, PhoshDBusCallsCallSkeletonClass))
#define PHOSH_DBUS_CALLS_CALL_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_CALLS_CALL_SKELETON, PhoshDBusCallsCallSkeletonClass))
#define PHOSH_DBUS_IS_CALLS_CALL_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_CALLS_CALL_SKELETON))
#define PHOSH_DBUS_IS_CALLS_CALL_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_CALLS_CALL_SKELETON))

typedef struct _PhoshDBusCallsCallSkeleton PhoshDBusCallsCallSkeleton;
typedef struct _PhoshDBusCallsCallSkeletonClass PhoshDBusCallsCallSkeletonClass;
typedef struct _PhoshDBusCallsCallSkeletonPrivate PhoshDBusCallsCallSkeletonPrivate;

struct _PhoshDBusCallsCallSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusCallsCallSkeletonPrivate *priv;
};

struct _PhoshDBusCallsCallSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_calls_call_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusCallsCallSkeleton, g_object_unref)
#endif

PhoshDBusCallsCall *phosh_dbus_calls_call_skeleton_new (void);


/* ---- */

#define PHOSH_DBUS_TYPE_OBJECT (phosh_dbus_object_get_type ())
#define PHOSH_DBUS_OBJECT(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_OBJECT, PhoshDBusObject))
#define PHOSH_DBUS_IS_OBJECT(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_OBJECT))
#define PHOSH_DBUS_OBJECT_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_OBJECT, PhoshDBusObject))

struct _PhoshDBusObject;
typedef struct _PhoshDBusObject PhoshDBusObject;
typedef struct _PhoshDBusObjectIface PhoshDBusObjectIface;

struct _PhoshDBusObjectIface
{
  GTypeInterface parent_iface;
};

GType phosh_dbus_object_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusObject, g_object_unref)
#endif

PhoshDBusCallsCall *phosh_dbus_object_get_calls_call (PhoshDBusObject *object);
PhoshDBusCallsCall *phosh_dbus_object_peek_calls_call (PhoshDBusObject *object);

#define PHOSH_DBUS_TYPE_OBJECT_PROXY (phosh_dbus_object_proxy_get_type ())
#define PHOSH_DBUS_OBJECT_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_OBJECT_PROXY, PhoshDBusObjectProxy))
#define PHOSH_DBUS_OBJECT_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_OBJECT_PROXY, PhoshDBusObjectProxyClass))
#define PHOSH_DBUS_OBJECT_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_OBJECT_PROXY, PhoshDBusObjectProxyClass))
#define PHOSH_DBUS_IS_OBJECT_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_OBJECT_PROXY))
#define PHOSH_DBUS_IS_OBJECT_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_OBJECT_PROXY))

typedef struct _PhoshDBusObjectProxy PhoshDBusObjectProxy;
typedef struct _PhoshDBusObjectProxyClass PhoshDBusObjectProxyClass;
typedef struct _PhoshDBusObjectProxyPrivate PhoshDBusObjectProxyPrivate;

struct _PhoshDBusObjectProxy
{
  /*< private >*/
  GDBusObjectProxy parent_instance;
  PhoshDBusObjectProxyPrivate *priv;
};

struct _PhoshDBusObjectProxyClass
{
  GDBusObjectProxyClass parent_class;
};

GType phosh_dbus_object_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusObjectProxy, g_object_unref)
#endif

PhoshDBusObjectProxy *phosh_dbus_object_proxy_new (GDBusConnection *connection, const gchar *object_path);

#define PHOSH_DBUS_TYPE_OBJECT_SKELETON (phosh_dbus_object_skeleton_get_type ())
#define PHOSH_DBUS_OBJECT_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_OBJECT_SKELETON, PhoshDBusObjectSkeleton))
#define PHOSH_DBUS_OBJECT_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_OBJECT_SKELETON, PhoshDBusObjectSkeletonClass))
#define PHOSH_DBUS_OBJECT_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_OBJECT_SKELETON, PhoshDBusObjectSkeletonClass))
#define PHOSH_DBUS_IS_OBJECT_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_OBJECT_SKELETON))
#define PHOSH_DBUS_IS_OBJECT_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_OBJECT_SKELETON))

typedef struct _PhoshDBusObjectSkeleton PhoshDBusObjectSkeleton;
typedef struct _PhoshDBusObjectSkeletonClass PhoshDBusObjectSkeletonClass;
typedef struct _PhoshDBusObjectSkeletonPrivate PhoshDBusObjectSkeletonPrivate;

struct _PhoshDBusObjectSkeleton
{
  /*< private >*/
  GDBusObjectSkeleton parent_instance;
  PhoshDBusObjectSkeletonPrivate *priv;
};

struct _PhoshDBusObjectSkeletonClass
{
  GDBusObjectSkeletonClass parent_class;
};

GType phosh_dbus_object_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusObjectSkeleton, g_object_unref)
#endif

PhoshDBusObjectSkeleton *phosh_dbus_object_skeleton_new (const gchar *object_path);
void phosh_dbus_object_skeleton_set_calls_call (PhoshDBusObjectSkeleton *object, PhoshDBusCallsCall *interface_);

/* ---- */

#define PHOSH_DBUS_TYPE_OBJECT_MANAGER_CLIENT (phosh_dbus_object_manager_client_get_type ())
#define PHOSH_DBUS_OBJECT_MANAGER_CLIENT(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_OBJECT_MANAGER_CLIENT, PhoshDBusObjectManagerClient))
#define PHOSH_DBUS_OBJECT_MANAGER_CLIENT_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_OBJECT_MANAGER_CLIENT, PhoshDBusObjectManagerClientClass))
#define PHOSH_DBUS_OBJECT_MANAGER_CLIENT_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_OBJECT_MANAGER_CLIENT, PhoshDBusObjectManagerClientClass))
#define PHOSH_DBUS_IS_OBJECT_MANAGER_CLIENT(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_OBJECT_MANAGER_CLIENT))
#define PHOSH_DBUS_IS_OBJECT_MANAGER_CLIENT_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_OBJECT_MANAGER_CLIENT))

typedef struct _PhoshDBusObjectManagerClient PhoshDBusObjectManagerClient;
typedef struct _PhoshDBusObjectManagerClientClass PhoshDBusObjectManagerClientClass;
typedef struct _PhoshDBusObjectManagerClientPrivate PhoshDBusObjectManagerClientPrivate;

struct _PhoshDBusObjectManagerClient
{
  /*< private >*/
  GDBusObjectManagerClient parent_instance;
  PhoshDBusObjectManagerClientPrivate *priv;
};

struct _PhoshDBusObjectManagerClientClass
{
  GDBusObjectManagerClientClass parent_class;
};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusObjectManagerClient, g_object_unref)
#endif

GType phosh_dbus_object_manager_client_get_type (void);

GType phosh_dbus_object_manager_client_get_proxy_type (GDBusObjectManagerClient *manager, const gchar *object_path, const gchar *interface_name, gpointer user_data);

void phosh_dbus_object_manager_client_new (
    GDBusConnection        *connection,
    GDBusObjectManagerClientFlags  flags,
    const gchar            *name,
    const gchar            *object_path,
    GCancellable           *cancellable,
    GAsyncReadyCallback     callback,
    gpointer                user_data);
GDBusObjectManager *phosh_dbus_object_manager_client_new_finish (
    GAsyncResult        *res,
    GError             **error);
GDBusObjectManager *phosh_dbus_object_manager_client_new_sync (
    GDBusConnection        *connection,
    GDBusObjectManagerClientFlags  flags,
    const gchar            *name,
    const gchar            *object_path,
    GCancellable           *cancellable,
    GError                **error);

void phosh_dbus_object_manager_client_new_for_bus (
    GBusType                bus_type,
    GDBusObjectManagerClientFlags  flags,
    const gchar            *name,
    const gchar            *object_path,
    GCancellable           *cancellable,
    GAsyncReadyCallback     callback,
    gpointer                user_data);
GDBusObjectManager *phosh_dbus_object_manager_client_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
GDBusObjectManager *phosh_dbus_object_manager_client_new_for_bus_sync (
    GBusType                bus_type,
    GDBusObjectManagerClientFlags  flags,
    const gchar            *name,
    const gchar            *object_path,
    GCancellable           *cancellable,
    GError                **error);


G_END_DECLS

#endif /* __CALLS_DBUS_H__ */
