/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __LOGIN1_SESSION_DBUS_H__
#define __LOGIN1_SESSION_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.freedesktop.login1.Session */

#define PHOSH_DBUS_TYPE_LOGIN_SESSION (phosh_dbus_login_session_get_type ())
#define PHOSH_DBUS_LOGIN_SESSION(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_LOGIN_SESSION, PhoshDBusLoginSession))
#define PHOSH_DBUS_IS_LOGIN_SESSION(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_LOGIN_SESSION))
#define PHOSH_DBUS_LOGIN_SESSION_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_LOGIN_SESSION, PhoshDBusLoginSessionIface))

struct _PhoshDBusLoginSession;
typedef struct _PhoshDBusLoginSession PhoshDBusLoginSession;
typedef struct _PhoshDBusLoginSessionIface PhoshDBusLoginSessionIface;

struct _PhoshDBusLoginSessionIface
{
  GTypeInterface parent_iface;



  gboolean (*handle_set_brightness) (
    PhoshDBusLoginSession *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_subsystem,
    const gchar *arg_name,
    guint arg_brightness);

  gboolean (*handle_set_locked_hint) (
    PhoshDBusLoginSession *object,
    GDBusMethodInvocation *invocation,
    gboolean arg_unnamed_arg0);

  gboolean  (*get_active) (PhoshDBusLoginSession *object);

  gboolean  (*get_locked_hint) (PhoshDBusLoginSession *object);

  void (*lock) (
    PhoshDBusLoginSession *object);

  void (*unlock) (
    PhoshDBusLoginSession *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusLoginSession, g_object_unref)
#endif

GType phosh_dbus_login_session_get_type (void);

GDBusInterfaceInfo *phosh_dbus_login_session_interface_info (void);
guint phosh_dbus_login_session_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void phosh_dbus_login_session_complete_set_locked_hint (
    PhoshDBusLoginSession *object,
    GDBusMethodInvocation *invocation);

void phosh_dbus_login_session_complete_set_brightness (
    PhoshDBusLoginSession *object,
    GDBusMethodInvocation *invocation);



/* D-Bus signal emissions functions: */
void phosh_dbus_login_session_emit_lock (
    PhoshDBusLoginSession *object);

void phosh_dbus_login_session_emit_unlock (
    PhoshDBusLoginSession *object);



/* D-Bus method calls: */
void phosh_dbus_login_session_call_set_locked_hint (
    PhoshDBusLoginSession *proxy,
    gboolean arg_unnamed_arg0,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_login_session_call_set_locked_hint_finish (
    PhoshDBusLoginSession *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_login_session_call_set_locked_hint_sync (
    PhoshDBusLoginSession *proxy,
    gboolean arg_unnamed_arg0,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_login_session_call_set_brightness (
    PhoshDBusLoginSession *proxy,
    const gchar *arg_subsystem,
    const gchar *arg_name,
    guint arg_brightness,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_login_session_call_set_brightness_finish (
    PhoshDBusLoginSession *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_login_session_call_set_brightness_sync (
    PhoshDBusLoginSession *proxy,
    const gchar *arg_subsystem,
    const gchar *arg_name,
    guint arg_brightness,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
gboolean phosh_dbus_login_session_get_active (PhoshDBusLoginSession *object);
void phosh_dbus_login_session_set_active (PhoshDBusLoginSession *object, gboolean value);

gboolean phosh_dbus_login_session_get_locked_hint (PhoshDBusLoginSession *object);
void phosh_dbus_login_session_set_locked_hint (PhoshDBusLoginSession *object, gboolean value);


/* ---- */

#define PHOSH_DBUS_TYPE_LOGIN_SESSION_PROXY (phosh_dbus_login_session_proxy_get_type ())
#define PHOSH_DBUS_LOGIN_SESSION_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_LOGIN_SESSION_PROXY, PhoshDBusLoginSessionProxy))
#define PHOSH_DBUS_LOGIN_SESSION_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_LOGIN_SESSION_PROXY, PhoshDBusLoginSessionProxyClass))
#define PHOSH_DBUS_LOGIN_SESSION_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_LOGIN_SESSION_PROXY, PhoshDBusLoginSessionProxyClass))
#define PHOSH_DBUS_IS_LOGIN_SESSION_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_LOGIN_SESSION_PROXY))
#define PHOSH_DBUS_IS_LOGIN_SESSION_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_LOGIN_SESSION_PROXY))

typedef struct _PhoshDBusLoginSessionProxy PhoshDBusLoginSessionProxy;
typedef struct _PhoshDBusLoginSessionProxyClass PhoshDBusLoginSessionProxyClass;
typedef struct _PhoshDBusLoginSessionProxyPrivate PhoshDBusLoginSessionProxyPrivate;

struct _PhoshDBusLoginSessionProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusLoginSessionProxyPrivate *priv;
};

struct _PhoshDBusLoginSessionProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_login_session_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusLoginSessionProxy, g_object_unref)
#endif

void phosh_dbus_login_session_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusLoginSession *phosh_dbus_login_session_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusLoginSession *phosh_dbus_login_session_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_login_session_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusLoginSession *phosh_dbus_login_session_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusLoginSession *phosh_dbus_login_session_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_LOGIN_SESSION_SKELETON (phosh_dbus_login_session_skeleton_get_type ())
#define PHOSH_DBUS_LOGIN_SESSION_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_LOGIN_SESSION_SKELETON, PhoshDBusLoginSessionSkeleton))
#define PHOSH_DBUS_LOGIN_SESSION_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_LOGIN_SESSION_SKELETON, PhoshDBusLoginSessionSkeletonClass))
#define PHOSH_DBUS_LOGIN_SESSION_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_LOGIN_SESSION_SKELETON, PhoshDBusLoginSessionSkeletonClass))
#define PHOSH_DBUS_IS_LOGIN_SESSION_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_LOGIN_SESSION_SKELETON))
#define PHOSH_DBUS_IS_LOGIN_SESSION_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_LOGIN_SESSION_SKELETON))

typedef struct _PhoshDBusLoginSessionSkeleton PhoshDBusLoginSessionSkeleton;
typedef struct _PhoshDBusLoginSessionSkeletonClass PhoshDBusLoginSessionSkeletonClass;
typedef struct _PhoshDBusLoginSessionSkeletonPrivate PhoshDBusLoginSessionSkeletonPrivate;

struct _PhoshDBusLoginSessionSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusLoginSessionSkeletonPrivate *priv;
};

struct _PhoshDBusLoginSessionSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_login_session_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusLoginSessionSkeleton, g_object_unref)
#endif

PhoshDBusLoginSession *phosh_dbus_login_session_skeleton_new (void);


G_END_DECLS

#endif /* __LOGIN1_SESSION_DBUS_H__ */
