/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __PHOSH_BRIGHTNESS_DBUS_H__
#define __PHOSH_BRIGHTNESS_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.Shell.Brightness */

#define PHOSH_DBUS_TYPE_BRIGHTNESS (phosh_dbus_brightness_get_type ())
#define PHOSH_DBUS_BRIGHTNESS(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_BRIGHTNESS, PhoshDBusBrightness))
#define PHOSH_DBUS_IS_BRIGHTNESS(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_BRIGHTNESS))
#define PHOSH_DBUS_BRIGHTNESS_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_BRIGHTNESS, PhoshDBusBrightnessIface))

struct _PhoshDBusBrightness;
typedef struct _PhoshDBusBrightness PhoshDBusBrightness;
typedef struct _PhoshDBusBrightnessIface PhoshDBusBrightnessIface;

struct _PhoshDBusBrightnessIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_set_auto_brightness_target) (
    PhoshDBusBrightness *object,
    GDBusMethodInvocation *invocation,
    gdouble arg_target);

  gboolean (*handle_set_dimming) (
    PhoshDBusBrightness *object,
    GDBusMethodInvocation *invocation,
    gboolean arg_enable);

  gboolean  (*get_has_brightness_control) (PhoshDBusBrightness *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusBrightness, g_object_unref)
#endif

GType phosh_dbus_brightness_get_type (void);

GDBusInterfaceInfo *phosh_dbus_brightness_interface_info (void);
guint phosh_dbus_brightness_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void phosh_dbus_brightness_complete_set_dimming (
    PhoshDBusBrightness *object,
    GDBusMethodInvocation *invocation);

void phosh_dbus_brightness_complete_set_auto_brightness_target (
    PhoshDBusBrightness *object,
    GDBusMethodInvocation *invocation);



/* D-Bus method calls: */
void phosh_dbus_brightness_call_set_dimming (
    PhoshDBusBrightness *proxy,
    gboolean arg_enable,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_brightness_call_set_dimming_finish (
    PhoshDBusBrightness *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_brightness_call_set_dimming_sync (
    PhoshDBusBrightness *proxy,
    gboolean arg_enable,
    GCancellable *cancellable,
    GError **error);

void phosh_dbus_brightness_call_set_auto_brightness_target (
    PhoshDBusBrightness *proxy,
    gdouble arg_target,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_brightness_call_set_auto_brightness_target_finish (
    PhoshDBusBrightness *proxy,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_brightness_call_set_auto_brightness_target_sync (
    PhoshDBusBrightness *proxy,
    gdouble arg_target,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
gboolean phosh_dbus_brightness_get_has_brightness_control (PhoshDBusBrightness *object);
void phosh_dbus_brightness_set_has_brightness_control (PhoshDBusBrightness *object, gboolean value);


/* ---- */

#define PHOSH_DBUS_TYPE_BRIGHTNESS_PROXY (phosh_dbus_brightness_proxy_get_type ())
#define PHOSH_DBUS_BRIGHTNESS_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_BRIGHTNESS_PROXY, PhoshDBusBrightnessProxy))
#define PHOSH_DBUS_BRIGHTNESS_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_BRIGHTNESS_PROXY, PhoshDBusBrightnessProxyClass))
#define PHOSH_DBUS_BRIGHTNESS_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_BRIGHTNESS_PROXY, PhoshDBusBrightnessProxyClass))
#define PHOSH_DBUS_IS_BRIGHTNESS_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_BRIGHTNESS_PROXY))
#define PHOSH_DBUS_IS_BRIGHTNESS_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_BRIGHTNESS_PROXY))

typedef struct _PhoshDBusBrightnessProxy PhoshDBusBrightnessProxy;
typedef struct _PhoshDBusBrightnessProxyClass PhoshDBusBrightnessProxyClass;
typedef struct _PhoshDBusBrightnessProxyPrivate PhoshDBusBrightnessProxyPrivate;

struct _PhoshDBusBrightnessProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusBrightnessProxyPrivate *priv;
};

struct _PhoshDBusBrightnessProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_brightness_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusBrightnessProxy, g_object_unref)
#endif

void phosh_dbus_brightness_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusBrightness *phosh_dbus_brightness_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusBrightness *phosh_dbus_brightness_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_brightness_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusBrightness *phosh_dbus_brightness_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusBrightness *phosh_dbus_brightness_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_BRIGHTNESS_SKELETON (phosh_dbus_brightness_skeleton_get_type ())
#define PHOSH_DBUS_BRIGHTNESS_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_BRIGHTNESS_SKELETON, PhoshDBusBrightnessSkeleton))
#define PHOSH_DBUS_BRIGHTNESS_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_BRIGHTNESS_SKELETON, PhoshDBusBrightnessSkeletonClass))
#define PHOSH_DBUS_BRIGHTNESS_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_BRIGHTNESS_SKELETON, PhoshDBusBrightnessSkeletonClass))
#define PHOSH_DBUS_IS_BRIGHTNESS_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_BRIGHTNESS_SKELETON))
#define PHOSH_DBUS_IS_BRIGHTNESS_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_BRIGHTNESS_SKELETON))

typedef struct _PhoshDBusBrightnessSkeleton PhoshDBusBrightnessSkeleton;
typedef struct _PhoshDBusBrightnessSkeletonClass PhoshDBusBrightnessSkeletonClass;
typedef struct _PhoshDBusBrightnessSkeletonPrivate PhoshDBusBrightnessSkeletonPrivate;

struct _PhoshDBusBrightnessSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusBrightnessSkeletonPrivate *priv;
};

struct _PhoshDBusBrightnessSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_brightness_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusBrightnessSkeleton, g_object_unref)
#endif

PhoshDBusBrightness *phosh_dbus_brightness_skeleton_new (void);


G_END_DECLS

#endif /* __PHOSH_BRIGHTNESS_DBUS_H__ */
