/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __GEOCLUE_AGENT_DBUS_H__
#define __GEOCLUE_AGENT_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.freedesktop.GeoClue2.Agent */

#define PHOSH_DBUS_TYPE_GEO_CLUE2_AGENT (phosh_dbus_geo_clue2_agent_get_type ())
#define PHOSH_DBUS_GEO_CLUE2_AGENT(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_GEO_CLUE2_AGENT, PhoshDBusGeoClue2Agent))
#define PHOSH_DBUS_IS_GEO_CLUE2_AGENT(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_GEO_CLUE2_AGENT))
#define PHOSH_DBUS_GEO_CLUE2_AGENT_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), PHOSH_DBUS_TYPE_GEO_CLUE2_AGENT, PhoshDBusGeoClue2AgentIface))

struct _PhoshDBusGeoClue2Agent;
typedef struct _PhoshDBusGeoClue2Agent PhoshDBusGeoClue2Agent;
typedef struct _PhoshDBusGeoClue2AgentIface PhoshDBusGeoClue2AgentIface;

struct _PhoshDBusGeoClue2AgentIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_authorize_app) (
    PhoshDBusGeoClue2Agent *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_desktop_id,
    guint arg_req_accuracy_level);

  guint  (*get_max_accuracy_level) (PhoshDBusGeoClue2Agent *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusGeoClue2Agent, g_object_unref)
#endif

GType phosh_dbus_geo_clue2_agent_get_type (void);

GDBusInterfaceInfo *phosh_dbus_geo_clue2_agent_interface_info (void);
guint phosh_dbus_geo_clue2_agent_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void phosh_dbus_geo_clue2_agent_complete_authorize_app (
    PhoshDBusGeoClue2Agent *object,
    GDBusMethodInvocation *invocation,
    gboolean authorized,
    guint allowed_accuracy_level);



/* D-Bus method calls: */
void phosh_dbus_geo_clue2_agent_call_authorize_app (
    PhoshDBusGeoClue2Agent *proxy,
    const gchar *arg_desktop_id,
    guint arg_req_accuracy_level,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean phosh_dbus_geo_clue2_agent_call_authorize_app_finish (
    PhoshDBusGeoClue2Agent *proxy,
    gboolean* out_authorized,
    guint* out_allowed_accuracy_level,
    GAsyncResult *res,
    GError **error);

gboolean phosh_dbus_geo_clue2_agent_call_authorize_app_sync (
    PhoshDBusGeoClue2Agent *proxy,
    const gchar *arg_desktop_id,
    guint arg_req_accuracy_level,
    gboolean* out_authorized,
    guint* out_allowed_accuracy_level,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
guint phosh_dbus_geo_clue2_agent_get_max_accuracy_level (PhoshDBusGeoClue2Agent *object);
void phosh_dbus_geo_clue2_agent_set_max_accuracy_level (PhoshDBusGeoClue2Agent *object, guint value);


/* ---- */

#define PHOSH_DBUS_TYPE_GEO_CLUE2_AGENT_PROXY (phosh_dbus_geo_clue2_agent_proxy_get_type ())
#define PHOSH_DBUS_GEO_CLUE2_AGENT_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_GEO_CLUE2_AGENT_PROXY, PhoshDBusGeoClue2AgentProxy))
#define PHOSH_DBUS_GEO_CLUE2_AGENT_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_GEO_CLUE2_AGENT_PROXY, PhoshDBusGeoClue2AgentProxyClass))
#define PHOSH_DBUS_GEO_CLUE2_AGENT_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_GEO_CLUE2_AGENT_PROXY, PhoshDBusGeoClue2AgentProxyClass))
#define PHOSH_DBUS_IS_GEO_CLUE2_AGENT_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_GEO_CLUE2_AGENT_PROXY))
#define PHOSH_DBUS_IS_GEO_CLUE2_AGENT_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_GEO_CLUE2_AGENT_PROXY))

typedef struct _PhoshDBusGeoClue2AgentProxy PhoshDBusGeoClue2AgentProxy;
typedef struct _PhoshDBusGeoClue2AgentProxyClass PhoshDBusGeoClue2AgentProxyClass;
typedef struct _PhoshDBusGeoClue2AgentProxyPrivate PhoshDBusGeoClue2AgentProxyPrivate;

struct _PhoshDBusGeoClue2AgentProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  PhoshDBusGeoClue2AgentProxyPrivate *priv;
};

struct _PhoshDBusGeoClue2AgentProxyClass
{
  GDBusProxyClass parent_class;
};

GType phosh_dbus_geo_clue2_agent_proxy_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusGeoClue2AgentProxy, g_object_unref)
#endif

void phosh_dbus_geo_clue2_agent_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusGeoClue2Agent *phosh_dbus_geo_clue2_agent_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusGeoClue2Agent *phosh_dbus_geo_clue2_agent_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void phosh_dbus_geo_clue2_agent_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
PhoshDBusGeoClue2Agent *phosh_dbus_geo_clue2_agent_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
PhoshDBusGeoClue2Agent *phosh_dbus_geo_clue2_agent_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define PHOSH_DBUS_TYPE_GEO_CLUE2_AGENT_SKELETON (phosh_dbus_geo_clue2_agent_skeleton_get_type ())
#define PHOSH_DBUS_GEO_CLUE2_AGENT_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), PHOSH_DBUS_TYPE_GEO_CLUE2_AGENT_SKELETON, PhoshDBusGeoClue2AgentSkeleton))
#define PHOSH_DBUS_GEO_CLUE2_AGENT_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), PHOSH_DBUS_TYPE_GEO_CLUE2_AGENT_SKELETON, PhoshDBusGeoClue2AgentSkeletonClass))
#define PHOSH_DBUS_GEO_CLUE2_AGENT_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PHOSH_DBUS_TYPE_GEO_CLUE2_AGENT_SKELETON, PhoshDBusGeoClue2AgentSkeletonClass))
#define PHOSH_DBUS_IS_GEO_CLUE2_AGENT_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PHOSH_DBUS_TYPE_GEO_CLUE2_AGENT_SKELETON))
#define PHOSH_DBUS_IS_GEO_CLUE2_AGENT_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), PHOSH_DBUS_TYPE_GEO_CLUE2_AGENT_SKELETON))

typedef struct _PhoshDBusGeoClue2AgentSkeleton PhoshDBusGeoClue2AgentSkeleton;
typedef struct _PhoshDBusGeoClue2AgentSkeletonClass PhoshDBusGeoClue2AgentSkeletonClass;
typedef struct _PhoshDBusGeoClue2AgentSkeletonPrivate PhoshDBusGeoClue2AgentSkeletonPrivate;

struct _PhoshDBusGeoClue2AgentSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  PhoshDBusGeoClue2AgentSkeletonPrivate *priv;
};

struct _PhoshDBusGeoClue2AgentSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType phosh_dbus_geo_clue2_agent_skeleton_get_type (void);

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PhoshDBusGeoClue2AgentSkeleton, g_object_unref)
#endif

PhoshDBusGeoClue2Agent *phosh_dbus_geo_clue2_agent_skeleton_new (void);


G_END_DECLS

#endif /* __GEOCLUE_AGENT_DBUS_H__ */
